!{\src2tex{textfont=tt}}
!!****f* ABINIT/partial_dos_fractions_paw
!! NAME
!! partial_dos_fractions_paw
!!
!! FUNCTION
!!  Calculate PAW contributions to the partial DOS fractions (tetrahedron method)
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 ABINIT group (SM,MT)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors .
!!
!! INPUTS
!!  cprj(natom,nspinor*mband*mkmem*nsppol)= <p_lmn|Cnk> coefficients for each WF |Cnk>
!!                                          and each |p_lmn> non-local projector
!!  dtset     structured datatype, from which one uses :
!!   iatsph(nasph)=number of atoms used to project dos
!!   indlmn(6,lmnmax,ntypat)= array giving l,m,n,lm,ln,s for i=lmn
!!   lmnmax=max. number of (l,m,n) numbers over all types of atom
!!   kpt(3,nkpt)  =irreducible kpoints
!!   mband        =max number of bands per k-point
!!   mkmem        =number of kpoints in memory
!!   natom        =number of atoms in total
!!   natsph       =number of atoms ofor which the spherical decomposition must be done
!!   nband        =number of electronic bands for each kpoint
!!   nkpt         =number of irreducible kpoints
!!   nspinor      =number of spinor components
!!   nsppol       =1 or 2 spin polarization channels
!!  mbesslang=maximum angular momentum for Bessel function expansion
!!  mpi_enreg=informations about MPI parallelization
!!  ndosfraction=natsph*mbesslang
!!  pawprtdos=option to output the individual contributions to the partial DOS (0 or 1)
!!  pawrad(ntypat) <type(pawrad_type)>=paw radial mesh and related data:
!!     %mesh_size=Dimension of radial mesh
!!  pawtab(ntypat) <type(pawtab_type)>=paw tabulated starting data
!!  prtdos= option for DOS printing
!!
!! OUTPUT
!!  === If pawprtdos==1:
!!   dos_fractions_paw1(ikpt,iband,isppol,natom*mbesslang) = contribution to
!!       dos fractions from the PAW partial waves (phi)
!!   dos_fractions_pawt1(ikpt,iband,isppol,natom*mbesslang) = contribution to
!!       dos fractions from the PAW pseudo partial waves (phi_tild)
!!
!! SIDE EFFECTS
!!  dos_fractions(ikpt,iband,isppol,natom*mbesslang) = percentage of s, p, d..
!!    character on each atom for the wavefunction # ikpt,iband, isppol
!!    As input: contains only the pseudo contribution
!!    As output: contains pseudo contribution + PAW correction
!!
!! PARENTS
!!      outscfcv
!!
!! CHILDREN
!!      simp_gen
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine partial_dos_fractions_paw(cprj,dos_fractions,dos_fractions_paw1,dos_fractions_pawt1,dtset,indlmn,&
&          lmnmax,mbesslang,mkmem,mpi_enreg,ndosfraction,pawprtdos,pawrad,pawtab,prtdos)

 use defs_basis
 use defs_datatypes

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_11util
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: lmnmax,mbesslang,mkmem,ndosfraction,pawprtdos,prtdos
 type(MPI_type),intent(inout) :: mpi_enreg
 type(dataset_type),intent(inout) :: dtset
!arrays
 integer,intent(in) :: indlmn(6,lmnmax,dtset%ntypat)
 real(dp),intent(inout) :: dos_fractions(dtset%nkpt,dtset%mband,dtset%nsppol,ndosfraction)
 real(dp),intent(out) :: dos_fractions_paw1(dtset%nkpt,dtset%mband,dtset%nsppol,ndosfraction*pawprtdos)
 real(dp),intent(out) :: dos_fractions_pawt1(dtset%nkpt,dtset%mband,dtset%nsppol,ndosfraction*pawprtdos)
 type(cprj_type) :: cprj(dtset%natom,dtset%nspinor*dtset%mband*dtset%mkmem*dtset%nsppol)
 type(pawrad_type),intent(in) :: pawrad(dtset%ntypat)
 type(pawtab_type),intent(in) :: pawtab(dtset%ntypat)

!Local variables-------------------------------
!scalars
 integer :: basis_size,iat,iatom,iband,ibg,ikpt,il,ilang,ilmn,iln,im,isppol
 integer :: itypat,j0lmn,j0ln,jl,jlmn,jln,jm,klmn,kln,lmn_size,nband_k
 real(dp) :: cpij
!arrays
 real(dp),allocatable :: int1(:),int2(:)

!******************************************************************************************

 if (pawprtdos==1) then
  dos_fractions_paw1 =zero
  dos_fractions_pawt1=zero
 end if

 do iat=1,dtset%natsph
  iatom=dtset%iatsph(iat)

  itypat= dtset%typat(iatom)
  basis_size=pawtab(itypat)%basis_size
  lmn_size=pawtab(itypat)%lmn_size
  if(allocated(int1))deallocate(int1,int2)
  allocate(int1(basis_size*(basis_size+1)/2),int2(basis_size*(basis_size+1)/2))

  do jln=1,basis_size
   j0ln=jln*(jln-1)/2
   do iln=1,jln
    kln=j0ln+iln
    call simp_gen(int1(kln),pawtab(itypat)%phiphj(:,kln),pawrad(itypat))
    call simp_gen(int2(kln),pawtab(itypat)%tphitphj(:,kln),pawrad(itypat))
   end do !iln
  end do !jln

  ibg=0
  do isppol =1,dtset%nsppol
   do ikpt=1,dtset%nkpt
    nband_k=dtset%nband(ikpt+(isppol-1)*dtset%nkpt)
    do iband =1,nband_k
     do ilang=1,mbesslang

      do jlmn=1,lmn_size
       jl=indlmn(1,jlmn,itypat)
       jm=indlmn(2,jlmn,itypat)
       j0lmn=jlmn*(jlmn-1)/2
       do ilmn=1,jlmn
        il=indlmn(1,ilmn,itypat)
        im=indlmn(2,ilmn,itypat)
        klmn=j0lmn+ilmn
        kln=pawtab(itypat)%indklmn(2,klmn)

        if (il==ilang-1.and.jl==ilang-1.and.im==jm) then
         cpij=pawtab(itypat)%dltij(klmn)*&
&                 (cprj(iatom,iband+ibg)%cp(1,ilmn)*cprj(iatom,iband+ibg)%cp(1,jlmn) &
&                 +cprj(iatom,iband+ibg)%cp(2,ilmn)*cprj(iatom,iband+ibg)%cp(2,jlmn))
         dos_fractions(ikpt,iband,isppol,mbesslang*(iatom-1)+ilang)=  &
&            dos_fractions(ikpt,iband,isppol,mbesslang*(iatom-1)+ilang) + &
&            cpij*(int1(kln)-int2(kln))
         if (pawprtdos==1) then
          dos_fractions_paw1(ikpt,iband,isppol,mbesslang*(iatom-1)+ilang)=  &
&             dos_fractions_paw1(ikpt,iband,isppol,mbesslang*(iatom-1)+ilang) + &
&             cpij*int1(kln)
          dos_fractions_pawt1(ikpt,iband,isppol,mbesslang*(iatom-1)+ilang)=  &
&             dos_fractions_pawt1(ikpt,iband,isppol,mbesslang*(iatom-1)+ilang) + &
&             cpij*int2(kln)
         end if
        end if

       end do !ilmn
      end do   !jlmn

     end do ! ilang
    end do ! iband
    if (mkmem/=0) ibg = ibg + dtset%nspinor*nband_k
   end do ! ikpt
  end do ! isppol

 end do !iatom

 deallocate(int1,int2)

end subroutine partial_dos_fractions_paw
!!***
