!{\src2tex{textfont=tt}}
!!****f* ABINIT/Psolver_hartree
!! NAME
!! Psolver_hartree
!!
!! FUNCTION
!! Given rho(r), compute Hartree potential considering the system as
!! an isolated one. This potential is obtained from the convolution
!! of 1/r and rho(r), treated in Fourier space. This method is a wrapper around
!! Psolver() developped for BigDFT.
!! It can compute the xc energy and potential if required. This computation is
!! built on the drivexc() routine of ABINIT but access it directly from real
!! space. The present routine is a real space counter part to rhohxc().
!!
!! COPYRIGHT
!! Copyright (C) 1998-2005 ABINIT group (DCA, XG, GMR).
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!  dtset <type(dataset_type)>=all input variables in this dataset
!!  mpi_enreg=MPI-parallelisation information.
!!  rhor(nfft,nspden)=electron density in real space in electrons/bohr**3
!!   In the Poisson solver case, nspden = 1 for the moment.
!!  rprimd(3,3)=dimensional primitive translations in real space (bohr)
!!
!! OUTPUT
!!  enhartr=returned Hartree energy (hartree).
!!  enxc=returned exchange and correlation energy (hartree).
!!  envxc=returned energy of the Vxc potential (hartree).
!!  vhartr(nfft)=Hartree potential.
!!  vxc(nfft,nspden)=xc potential (in the present case, nspden = 1)
!!  vxcavg=<Vxc>=unit cell average of Vxc = (1/ucvol) Int [Vxc(r) d^3 r].
!!
!! PARENTS
!!
!! CHILDREN
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine Psolver_hartree(dtset, enhartr, mpi_enreg, rhor, rprimd, vhartr)

  use defs_basis
  use defs_datatypes
#if defined HAVE_BIGDFT
  use poisson_solver
#endif

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01manage_mpi
 use interfaces_14poisson, except_this_one => Psolver_hartree
#endif
!End of the abilint section

  implicit none

  !Arguments ------------------------------------
  !scalars
  real(dp), intent(out)         :: enhartr
  type(dataset_type),intent(in) :: dtset
  type(MPI_type),intent(inout)  :: mpi_enreg
  !arrays
  real(dp),intent(in)  :: rprimd(3,3)
  real(dp),intent(in)  :: rhor(dtset%nfft)
  real(dp),intent(out) :: vhartr(dtset%nfft)

  !Local variables-------------------------------
  !scalars
  real(dp) :: hgrid
  real(dp) :: enxc, evxc
  character(len=500) :: message
  character(len = 1) :: datacode
  !arrays
  real(dp) :: vxcmean(1)
  real(dp), pointer :: kernel(:)
  real(dp), dimension(1) :: pot_ion_dummy


  if(dtset%nspden /= 1)then
     write(message, '(a,a,a,a,a,a,i5)' ) ch10,&
          &  ' PSolver_hartree: BUG -',ch10,&
          &  '  Only non-spin-polarised is allowed,',ch10,&
          &  '  while the argument nspden = ', dtset%nspden
     call wrtout(6,message,'COLL')
     call leave_new('COLL')
  end if

  ! We get the kernel.
  call PSolver_kernel(dtset, 2, kernel, mpi_enreg, rprimd)
  ! If the kernel is not created, we do it now.
  if (.not.associated(kernel)) then
     call PSolver_kernel(dtset, 1, kernel, mpi_enreg, rprimd)
  end if

  ! We do the computation.
  write(message, "(A,A,A,3I6)") "Psolver_hartree(): compute potential...", ch10, &
       & " | dimension:", dtset%ngfft(1:3)
  call wrtout(06, message,'COLL')

  ! Poisson solver works in place, so we copy rhor to vhartr.
  vhartr(:) = rhor(:)

  ! The computation is done with 'F'ree boundary counditions. The data are
  ! 'D'istributed in the wavelet case or 'G'lobal otherwise.
#if defined HAVE_BIGDFT
  if (dtset%usewvl == 0) then
     hgrid = rprimd(1, 1) / dtset%ngfft(1)
     ! This may not work with MPI in the planewave code...
     call PSolver('F', 'G', mpi_enreg%me, mpi_enreg%nproc, dtset%ngfft(1), &
          & dtset%ngfft(2), dtset%ngfft(3), 0, hgrid, hgrid, hgrid, &
          & vhartr, kernel, pot_ion_dummy, enhartr, enxc, evxc, 0.d0, .false.)
  else
     hgrid = 0.5d0 * dtset%wvl_hgrid
     if (mpi_enreg%paral_compil_fft == 1) then
        datacode = 'D'
     else
        datacode = 'G'
     end if
     ! Data are always distributed when using the wavelets, even if nproc = 1.
     ! The size given is the complete size of the box, not the distributed size
     ! stored in ngfft.
     call PSolver('F', datacode, mpi_enreg%me, mpi_enreg%nproc, &
          & dtset%wvl_internal%dpSize(1), dtset%wvl_internal%dpSize(2), &
          & dtset%wvl_internal%dpSize(3), 0, hgrid, hgrid, hgrid, &
          & vhartr, kernel, pot_ion_dummy, enhartr, enxc, evxc, 0.d0, .false.)
  end if
#else
  write(message, '(a,a,a,a)' ) ch10,&
       & ' Psolver_hartree: BUG -',ch10,&
       & '  BigDFT is not compile. Use --enable-bigdft during configure.'
  call wrtout(06, message, 'COLL')
  call leave_new('COLL')
#endif

end subroutine Psolver_hartree
!!***
