!{\src2tex{textfont=tt}}
!!****f* ABINIT/mlwfovlp_proj
!! NAME
!! mlwfovlp_proj
!!
!! FUNCTION
!! Routine which computes projection A_{mn}(k)
!! for Wannier code (www.wannier.org f90 version).
!!
!! COPYRIGHT
!! Copyright (C) 2005-2007 ABINIT group (BAmadon,FJollet)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!  cprj(natom,nspinor*mband*mkmem*nsppol)= <p_lmn|Cnk> coefficients for each WF |Cnk>
!!                                          and each |p_lmn> non-local projector
!!  dtset <type(dataset_type)>=all input variables for this dataset
!!  eigen(mband*nkpt*nsppol)=array for holding eigenvalues (hartree)
!!  lproj= flag 0: no projections, 1: random projections, 2: projections on projectors
!!              3: projections on wavefunctions.
!!  mband=maximum number of bands
!!  mkmem =number of k points which can fit in memory; set to 0 if use disk
!!  mpw=maximum dimensioned size of npw.
!!  natom=number of atoms in cell.
!!  nattyp(ntypat)= # atoms of each type.
!!  nkpt=number of k points.
!!  nspden=number of spin-density components
!!  nspinor=number of spinorial components of the wavefunctions
!!  nsppol=1 for unpolarized, 2 for spin-polarized
!!  ntypat=number of types of atoms in unit cell.
!!  num_bands=number of bands actually used to construct the wannier function
!!  nwan= number of wannier fonctions (read in wannier90.win).
!!  proj_l(mband)= angular part of the projection function (quantum number l)
!!  proj_m(mband)= angular part of the projection function (quantum number m)
!!  proj_radial(mband)= radial part of the projection.
!!  proj_site(3,mband)= site of the projection.
!!  proj_x(3,mband)= x axis for the projection.
!!  proj_z(3,mband)= z axis for the projection.
!!  proj_zona(mband)= extension of the radial part.
!!  prtvol=control print volume and debugging output
!!  psps <type(pseudopotential_type)>=variables related to pseudopotentials
!!
!! OUTPUT
!!  A_matrix(num_bands,nwan,nkpt)= Matrix of projections needed by wannier_run
!!  ( also wannier90random.amn is written)
!!
!! SIDE EFFECTS
!!  (only writing, printing)
!!
!! NOTES
!!
!! PARENTS
!!      mlwfovlp_setup
!!
!! CHILDREN
!!      (none)
!!
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

 subroutine mlwfovlp_proj(A_matrix,band_in,cprj,dtset,eigen,lproj,mband,mkmem,mpw,natom,&
& nattyp,nkpt,nspden,nspinor,nsppol,ntypat,num_bands,nwan,pawtab,&
& proj_l,proj_m,proj_radial,proj_site,proj_x,proj_z,proj_zona,prtvol,psps)                           

 use defs_basis
 use defs_datatypes
 use defs_wannier90

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01manage_mpi
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: lproj,mband,mkmem,mpw,natom,nkpt,nspden,nspinor,nsppol,ntypat,num_bands,nwan
 integer,intent(in) :: prtvol
 type(dataset_type),intent(in) :: dtset
 type(pseudopotential_type),intent(in) :: psps
!arrays
 integer :: nattyp(ntypat)
 integer,intent(in) :: proj_l(mband),proj_m(mband),proj_radial(mband)
 real(dp),intent(in) :: eigen(mband*nkpt*nsppol),proj_site(3,mband)
 real(dp),intent(in) :: proj_x(3,mband),proj_z(3,mband),proj_zona(mband)
 complex(dpc),intent(out) :: A_matrix(num_bands,nwan,nkpt)
 logical,intent(in) :: band_in(mband)
 type(cprj_type) :: cprj(natom,nspinor*mband*mkmem*nsppol)
 type(pawtab_type),intent(in) :: pawtab(psps%ntypat*psps%usepaw)

!Local variables-------------------------------
!scalars
 integer :: iatom,iatprjn,iband,iband1,iband2,ibg,icat,idum,ikpt,ilmn,ispden
 integer :: ispinor,isppol,itypat,iwan,jband,jj1,libprjn,lmn_size,mesh_size
 integer :: natprjn,nband_k,nbprjn,ndosfraction,partial_dos_flag,prtdos,sumtmp
 real(dp) :: uniformrandom,x1,x2,xnorm,xnormb
 character(len=500) :: message
 character(len=fnlen) :: fildata
 type(pseudopotential_type) :: pspsphi
!arrays
 integer,allocatable :: lprjn(:),npprjn(:)
 real(dp),allocatable :: amn(:,:,:,:),amn2(:,:,:,:,:,:,:),chiphimphitint(:,:)
 real(dp),allocatable :: dos_fractions(:,:,:,:),ff(:),pjPsi(:,:,:,:,:,:,:)
 type(cprj_type),allocatable :: cprj_dum(:,:)
!no_abirules
 integer,parameter :: mtransfo(0:3,7)=&
&  reshape((/1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,-2,-1,2,1,0,0,0,-1,1,2,-2,-3,3/),(/4,7/))

!************************************************************************


!TODO AND POSSIBLE BUGS:
!MAGNETISM

 if(lproj/=1) then
       write(message, '(a,a,a,a,a,a)' ) ch10,&
&      '  mlwfovlp_proj : ERROR -',ch10,&
&      '  Value of lproj no allowed ',ch10,&
&      '  Action : change lproj.'
       call wrtout(06,  message,'COLL')
       call leave_new('COLL')
 end if

   write(message, '(a,a)' )ch10,&
&   '** mlwfovlp_proj:  compute A_matrix of initial guess for wannier functions'
   call wrtout(6,message,'COLL')

!********************* Write Random projectors 
 if(lproj==1) then
 open(unit=220,file='wannier90random.amn',form='formatted',status='unknown')
 idum=123456
! Compute random projections
 allocate(amn(2,mband,nkpt,nwan))
 amn=zero
 write(220,*) "Random Projections from Abinit"
 write(220,*) num_bands,nkpt,nwan
 do ikpt=1,nkpt
  do iband1=1,mband
   xnormb=0.d0
   do iband2=1,nwan
   x1=uniformrandom(idum)
   x2=uniformrandom(idum)
   xnorm=sqrt(x1**2+x2**2)
   xnormb=xnormb+xnorm
   amn(1,iband1,ikpt,iband2)=x1
   amn(2,iband1,ikpt,iband2)=x2
   end do
   do iband2=1,nwan
   amn(1,iband1,ikpt,iband2)=amn(1,iband1,ikpt,iband2)/xnormb
   amn(2,iband1,ikpt,iband2)=amn(1,iband1,ikpt,iband2)/xnormb
   end do
  end do
 end do
 do ikpt=1,nkpt
  do iband2=1,nwan
   jband=0
   do iband1=1,mband
    if(band_in(iband1)) then
     jband=jband+1
     write(220,'(3i6,3x,3x,2f11.7)')iband1,iband2,ikpt,amn(1,iband1,ikpt,iband2),amn(2,iband1,ikpt,iband2)
     if(jband.gt.num_bands) then
       write(message, '(a,a,a,a,a,a)' ) ch10,&
&      '  mlwfovlp_proj : ERROR -',ch10,&
&      '  Value of jband is above num_bands ',ch10,&
&      '  Action : contact Abinit group'
       call wrtout(06,  message,'COLL')
       call leave_new('COLL')
     end if
     A_matrix(jband,iband2,ikpt)=cmplx(amn(1,iband1,ikpt,iband2),amn(2,iband1,ikpt,iband2))
    end if
   end do
  end do
 end do
 close(220)
 deallocate(amn)
 end if


! *************** computes projection  from PROJECTORS ********************
 IF(lproj==2) THEN  !! IF LPROJPRJ
!----- set values for projections --------------------- ! INPUT
! nbprjn:number of  different l-values for projectors
! lprjn: value of l for each projectors par ordre croissant
! npprjn: number of projectors for each lprjn
 natprjn=1  ! atoms with wannier functions are first 
 if(natprjn.ne.1) stop ! in this case lprjn should depend on iatprjn
 nbprjn=2
 allocate(lprjn(nbprjn))
 lprjn(1)=0
 lprjn(2)=1
 allocate(npprjn(0:lprjn(nbprjn)))
 npprjn(0)=1
 npprjn(1)=1
!--- test coherence of nbprjn and nwan                          
 sumtmp=0                                                     
 do iatprjn=1,natprjn
  do libprjn=0,lprjn(nbprjn)
   sumtmp=sumtmp+(2*libprjn+1)*npprjn(libprjn)
  end do
 end do
 if(sumtmp/=nwan) then
  write(6,*) "Number of Wannier orbitals is not equal to number of projections"
  write(6,*) "Action: check values of lprjn,npprjn % nwan"
  write(6,*) "nwan, sumtmp=",nwan,sumtmp
  stop
 end if
!--- end test of coherence
 allocate(amn2(2,natom,nsppol,nkpt,mband,nspinor,nwan))
 if(psps%usepaw==1) then
  open(unit=219,file='wannier90.amn',form='formatted',status='unknown')
  amn2=zero
  ibg=0
  do isppol=1,nsppol
   do ikpt=1,nkpt
    nband_k=dtset%nband(ikpt+(isppol-1)*nkpt)
    do iband=1,nband_k
!             write(6,*)"amn2",iband,ibg,ikpt
     do ispinor=1,nspinor
      icat=1
      do itypat=1,dtset%ntypat
       lmn_size=pawtab(itypat)%lmn_size
       do iatom=icat,icat+nattyp(itypat)-1
        jj1=0
        do ilmn=1,lmn_size
         if(iatom.le.natprjn) then
!         do iwan=1,nwan
          do libprjn=0,lprjn(nbprjn)
!           if (psps%indlmn(1,ilmn,itypat)==proj_l(iwan)) then
!           if (psps%indlmn(2,ilmn,itypat)==mtransfo(proj_l(iwan),proj_m(iwan))) then
           if (psps%indlmn(1,ilmn,itypat)==libprjn) then
            if (psps%indlmn(3,ilmn,itypat)<=npprjn(libprjn)) then
             if(band_in(iband)) then
              jj1=jj1+1
              if(jj1>nwan) then
               write(6,*) "number of wannier orbitals is lower than lmn_size"
               write(6,*) jj1,nwan
               stop
              end if
              amn2(1,iatom,isppol,ikpt,iband,ispinor,jj1)=cprj(iatom,iband+ibg)%cp(1,ilmn)
              amn2(2,iatom,isppol,ikpt,iband,ispinor,jj1)=cprj(iatom,iband+ibg)%cp(2,ilmn)
             end if
            end if
           end if
          end do ! libprjn
!          endif
!          endif
!          enddo ! iwan
         end if ! natprjn
        end do !ilmn
       end do ! iatom
       icat=icat+nattyp(itypat)
      end do ! itypat
     end do ! ispinor
    end do !iband
    ibg=ibg+nband_k*nspinor
!             write(6,*)'amn2b',iband,ibg,ikpt
   end do !ikpt
  end do ! isppol

!-----------------------  write      projections    --------------------

  write(219,*) "Projections from Abinit : mband,nkpt,nwan. indices: iband1,iwan,ikpt"
  write(219,*) num_bands,nkpt,nwan
  do ikpt=1,nkpt
   do iband2=1,nwan
    jband=0
    do iband1=1,mband
     if(band_in(iband1)) then
       jband=jband+1
!     write(219,*) iband1,ikpt,iband2
     write(219,'(3i6,3x,3x,2f18.14)')jband,iband2,ikpt,amn2(1,1,1,ikpt,iband1,1,iband2),amn2(2,1,1,ikpt,iband1,1,iband2)
!     write(219,*)iband1,iband2,ikpt,amn(1,iband1,ikpt,iband2),amn(2,iband1,ikpt,iband2)
     A_matrix(jband,iband2,ikpt)=&
&     cmplx(amn2(1,1,1,ikpt,iband1,1,iband2),amn2(2,1,1,ikpt,iband1,1,iband2))
     end if
    end do
   end do
  end do

 end if !usepaw
 close(219)
 deallocate(amn2)
 deallocate(npprjn,lprjn)

 end if ! lproj==2

 end subroutine mlwfovlp_proj
!!***
