!{\src2tex{textfont=tt}}
!!****f* ABINIT/assemblychi0_sym
!! NAME
!! assemblychi0_sym
!!
!! FUNCTION
!! Update the independent particle susceptibility for the contribution
!! of one pair of occupied-unoccupied band, for each frequencies
!! taking into account the symmetries of the little group of the external 
!! point q
!!
!! Compute chi0(G,G'',io)=chi0(G,G'',io)+\sum_S (rhotwg(G)*rhotwg*(G''))*den(io)
!! where S are the symmetries of the little group of the external q-point.
!!
!! COPYRIGHT
!! Copyright (C) 1999-2007 ABINIT group (MG, GMR, VO, LR, RWG)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!  npwsigx=number of plane waves for sigma exchange or epsilon (input variable)
!!  npwvec=maximum number of G vectors
!!  nomega=number of frequencies
!!  rhotwg(npwsigx)=density of a pair of occupied-unoccupied states, in reciprocal space
!!  den(nomega)=denominator of the susceptibility expression
!!  nop=number of symmetry operations
!!  ninv=if 2, inversion is considered; if 1, inversion is not considered
!!  nkbz=number of kpoints in the full BZ
!!  phgt(npwsigx,nop)=phase factors associated to non-simmorphic operations
!!  wtksym(2,nop)= equal to 1 if the symmetry (with or without time reversal) for this kpoint must be considered
!!   in the calculation of \chi_o, 0 otherwise  
!!  grottbm1(npwvec,2,nop)= contains the index of (IR) G where I is the identity or the inversion 
!!   operation and G is one of the npwvec vectors in reciprocal space 
!!  fumklp(ninv,nop)= flag for umklapp processes 
!!                    if 1 that the particular operation (IS) requires a G_o to preserve q
!!                       0 otherwise 
!!  iumklp(npwvec,ninv,nop) index of G-G0 in the FFT array
!!    
!! OUTPUT
!!  (see side effects)
!!
!! SIDE EFFECTS
!!  chi0(npwsigx,npwsigx,nomega)=independent-particle susceptibility matrix in reciprocal space
!!
!! PARENTS
!!   
!!
!! CHILDREN
!!   
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine assemblychi0_sym(npwvec,npwsigx,ninv,nop,nkbz,wtksym,iumklp,&
& fumklp,phgt,grottbm1,nomega,chi0,rhotwg,den)

 use defs_basis

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: ninv,nkbz,nomega,nop,npwsigx,npwvec
!arrays
 integer,intent(in) :: grottbm1(npwvec,ninv,nop),wtksym(ninv,nop)
 integer,intent(in) :: iumklp(npwvec,ninv,nop),fumklp(ninv,nop)
 complex,intent(in) :: rhotwg(npwsigx)
 complex,target :: chi0(npwsigx,npwsigx,nomega)
 complex,intent(in) :: phgt(npwsigx,nop)
 complex(dpc),intent(in) :: den(nomega)

!Local variables-------------------------------
!scalars
 integer :: i,i2,ig,igp,iinv,io,iop,npwsigx2
 complex :: dd
 logical,parameter :: vectorialized=.false.
!arrays
 complex,allocatable :: rhotwg_sym(:),rhotwgs(:)

! *************************************************************************

#ifdef VMS
!DEC$ ATTRIBUTES ALIAS:'CGERC' :: cgerc
#endif

! Symmetrization
! If  Sq=q then  M_G^( Sk,q)= e^{-i(q+G)\cdot t} M_{ S^-1G}  (k,q)
! If -Sq=q then  M_G^(-Sk,q)= e^{-i(q+G)\cdot t} M_{-S^-1G}^*(k,q)
!
! In case of an umklapp process 
! If  Sq=q+G_o then  M_G( Sk,q)= e^{-i(q+G)\cdot t} M_{ S^-1(G-G_o}   (k,q)
! If -Sq=q+G_o then  M_G(-Sk,q)= e^{-i(q+G)\cdot t} M_{-S^-1(G-G_o)}^*(k,q)
!
! Note that there is no need to take into account the phases due to q, 
! since they cancel in the scalar product ==> phgt(G,iop) = e^{-iG \cdot t}
!
! iumklp(ig,iinv,iop) contains the FFT index of G-G0 where ISq=q+G0
! shuffling of indexes is done only if G0/=0 to optimize a little bit

 allocate(rhotwg_sym(npwsigx))

 do iop=1,nop
  do iinv=1,ninv
   if (wtksym(iinv,iop)==1) then 
    if (iinv==1) then 
     if (fumklp(iinv,iop)==0) then !G0=0
      rhotwg_sym(1:npwsigx)=rhotwg(grottbm1(1:npwsigx,iinv,iop))*phgt(1:npwsigx,iop)
     else
      print *, 'not implemented'
      stop
     !this is to test the umklapp  implementation, not so clever
     ! rhotwg_sym(1:npwsigx)=rhotwg(grottbm1(iumklp(1:npwsigx,iinv,iop),iinv,iop))*phgt(1:npwsigx,iop)
     end if 
    else 
     if (fumklp(iinv,iop)==0) then !G0=0
      rhotwg_sym(1:npwsigx)=conjg(rhotwg(grottbm1(1:npwsigx,iinv,iop)))*phgt(1:npwsigx,iop)
     else
     !this is to test the umklapp  implementation, not so clever
     !rhotwg_sym(1:npwsigx)=conjg(rhotwg(grottbm1(iumklp(1:npwsigx,iinv,iop),iinv,iop)))*phgt(1:npwsigx,iop)
      print *, 'not implemented'
      stop
     end if 
    end if 
    do io=1,nomega
     !dd is single precision needed for cgerc
     dd=den(io)
     !multiply elements G,Gp of rhotwg_sym/den(io) and accumulate in chi0(G,Gp,io)
     call cgerc(npwsigx,npwsigx,dd,rhotwg_sym,1,rhotwg_sym,1,chi0(:,:,io),npwsigx)
    end do

   end if !wtksym 
  end do !inv
 end do !iop
  
 deallocate(rhotwg_sym)

end subroutine assemblychi0_sym
!!***
