!{\src2tex{textfont=tt}}
!!****f* ABINIT/assemblychi0sf
!! NAME
!! assemblychi0sf
!!
!! FUNCTION
!! Update the imaginary part of the polarizability for the contribution
!! of one pair of occupied-unoccupied band, for each frequencies
!! taking into account the symmetries of the little group of the external point q
!!
!! Compute chi0(G,Gp,io)=chi0(G,Gp,io)+\sum_S (rhotwg(G)*rhotwg*(G''))*den(io)
!! where S are the symmetries of the little group of the external q-point.
!!
!! COPYRIGHT
!! Copyright (C) 1999-2007 ABINIT group (MG)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!  my_wl,my_wr= upper and lower frequency considered by this processor
!!  npwsigx=number of plane waves for sigma exchange or epsilon (input variable)
!!  npwvec=maximum number of G vectors
!!  nomegasf=number of frequencies for imaginary part of chi0
!!  rhotwg(npwsigx)=density of a pair of occupied-unoccupied states, in reciprocal space
!!  nop=number of symmetry operations
!!  ninv=if 2, inversion is considered; if 1, inversion is not considered
!!  nkbz=number of kpoints in the full BZ
!!  phgt(npwsigx,nop)=phase factors associated to non-simmorphic operations
!!  wtksym(2,nop)= equal to 1 if the symmetry (with or without time reversal) for this kpoint must be considered
!!   in the calculation of \chi_o, 0 otherwise  
!!  grottbm1(npwvec,2,nop)= contains the index of (IR) G where I is the identity or the inversion 
!!   operation and G is one of the npwvec vectors in reciprocal space 
!!  fumklp(ninv,nop)= flag for umklapp processes 
!!                    if 1 that the particular operation (IS) requires a G_o to preserve q
!!                       0 otherwise 
!!  iumklp(npwvec,ninv,nop) index of G-G0 in the FFT array
!!  symflag=flag used to switch on the symmetrization of the oscillator matrix elements
!!  factocc=occupation factor= f_occ*(ockp-occk) (see cchi0.F90)  
!!  wl,wr = weights used to approximate the Dirac function 
!!    
!! OUTPUT
!!  (see side effects)
!!
!! SIDE EFFECTS
!!  chi0sf(npwsigx,npwsigx,my_wl:my_wr)= update the imaginary part of the independent-particle 
!!   susceptibility matrix in reciprocal space
!!
!! NOTES
!!  Umklapp processes are not yet implemented 
!! 
!! PARENTS
!!      cchi0
!!
!! CHILDREN
!!      cgerc,leave_new,wrtout
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine assemblychi0sf(npwvec,npwsigx,ninv,nop,nkbz,wtksym,iumklp,&
& fumklp,phgt,grottbm1,rhotwg,factocc,my_wl,iomegal,wl,my_wr,iomegar,wr,sym_flag,nomegasf,chi0sf)

 use defs_basis

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01manage_mpi
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: iomegal,iomegar,my_wl,my_wr,ninv,nkbz,nomegasf,nop,npwsigx,npwvec,sym_flag
 real(dp),intent(in) :: factocc,wl,wr
!arrays
 integer,intent(in) :: grottbm1(npwvec,ninv,nop),wtksym(ninv,nop)
 integer,intent(in) :: iumklp(npwvec,ninv,nop),fumklp(ninv,nop)
 complex,intent(in) :: rhotwg(npwsigx)
 complex,target :: chi0sf(npwsigx,npwsigx,my_wl:my_wr)
 complex,intent(in) :: phgt(npwsigx,nop)

!Local variables-------------------------------
!scalars
 integer :: i,i2,ig,igp,iinv,io,iop,npwsigx2
 complex :: num
 character(len=500) :: message
!arrays
 complex,allocatable :: rhotwg_sym(:)

! *************************************************************************

#ifdef VMS
!DEC$ ATTRIBUTES ALIAS:'CGERC' :: cgerc
#endif

!DEBUG for the time being dont comment
 if (iomegal < my_wl .or. iomegar > my_wr) then 
  if (iomegal < my_wl) then 
   write(message,'(a)')' assemblychi0sf : iomegal < my_wl '
   call wrtout(06,message,'PERS')
   write(message,'(a,i5,a,i5)')'my_wl = ',my_wl,' iomegal =',iomegal
   call wrtout(06,message,'PERS')
   !write(message,'(2f8.2,2i4)')ep%omegasf(my_wl)*Ha_eV,ep%omegasf(iomegal)*Ha_eV,ibv,ibc
   !call wrtout(06,message,'PERS')
  end if 
  if (iomegar > my_wr) then 
   write(message,'(a)')' assemblychi0sf : iomegar > my_wr '
   call wrtout(06,message,'PERS')
   write(message,'(a,i5,a,i5)')'my_wr = ',my_wr,' iomegar = ',iomegar
   call wrtout(06,message,'PERS')
   !write(message,'(2f8.2,2i4)')ep%omegasf(my_wr)*Ha_eV,ep%omegasf(iomegar)*Ha_eV,ibv,ibc
   !call wrtout(06,message,'PERS')
  end if
  call leave_new('COLL')
 end if 
!ENDDEBUG


! Calculation without symmetries
 if (sym_flag==0) then 

  if (wl<huge(0.0_dp)*1.d-11) then
   !num is single precision needed for cgerc 
   num=-wl*factocc
   call cgerc(npwsigx,npwsigx,num,rhotwg,1,rhotwg,1,chi0sf(:,:,iomegal),npwsigx)
  end if

! Last point, must accumulate left point but not the right one
  if (iomegar/=nomegasf+1 .and. wr<huge(0.0_dp)*1.d-11) then 
   num=-wr*factocc
   call cgerc(npwsigx,npwsigx,num,rhotwg,1,rhotwg,1,chi0sf(:,:,iomegar),npwsigx)
  end if 

! Use symmetries to reconstruct oscillator matrix elements
 else

 allocate(rhotwg_sym(npwsigx))

! Perform Symmetrization
! If  Sq=q then  M_G^( Sk,q)= e^{-i(q+G)\cdot t} M_{ S^-1G}  (k,q)
! If -Sq=q then  M_G^(-Sk,q)= e^{-i(q+G)\cdot t} M_{-S^-1G}^*(k,q)
!
! In case of an umklapp process 
! If  Sq=q+G_o then  M_G( Sk,q)= e^{-i(q+G)\cdot t} M_{ S^-1(G-G_o}   (k,q)
! If -Sq=q+G_o then  M_G(-Sk,q)= e^{-i(q+G)\cdot t} M_{-S^-1(G-G_o)}^*(k,q)
!
! Note that there is no need to take into account the phases due to q, 
! since they cancel in the scalar product ==> phgt(G,iop) = e^{-iG \cdot t}
!
! iumklp(ig,iinv,iop) contains the FFT index of G-G0 where ISq=q+G0
! shuffling of indexes is done only if G0/=0 to optimize a little bit

 do iop=1,nop
  do iinv=1,ninv
   if (wtksym(iinv,iop)==1) then 

    if (iinv==1) then 
     if (fumklp(iinv,iop)==0) then !G0=0
      rhotwg_sym(1:npwsigx)=rhotwg(grottbm1(1:npwsigx,iinv,iop))*phgt(1:npwsigx,iop)
     else
      print *, 'not implemented'
      stop
     !this is to test the umklapp  implementation, not so clever
     ! rhotwg_sym(1:npwsigx)=rhotwg(grottbm1(iumklp(1:npwsigx,iinv,iop),iinv,iop))*phgt(1:npwsigx,iop)
     end if 
    else 
     if (fumklp(iinv,iop)==0) then !G0=0
      rhotwg_sym(1:npwsigx)=conjg(rhotwg(grottbm1(1:npwsigx,iinv,iop)))*phgt(1:npwsigx,iop)
     else
     !this is to test the umklapp  implementation, not so clever
     !rhotwg_sym(1:npwsigx)=conjg(rhotwg(grottbm1(iumklp(1:npwsigx,iinv,iop),iinv,iop)))*phgt(1:npwsigx,iop)
      print *, 'not implemented'
      stop
     end if 
    end if 

    !num is single precision needed for cgerc 
    if (wl<huge(0.0_dp)*1.d-11) then
     num=-wl*factocc
     call cgerc(npwsigx,npwsigx,num,rhotwg_sym,1,rhotwg_sym,1,chi0sf(:,:,iomegal),npwsigx)
    end if 

! Last point, must accumulate left point but not the right one
    if (iomegar/=nomegasf+1 .and. wr<huge(0.0_dp)*1.d-11) then 
     num=-wr*factocc
     call cgerc(npwsigx,npwsigx,num,rhotwg_sym,1,rhotwg_sym,1,chi0sf(:,:,iomegar),npwsigx)
    end if 

   end if !wtksym 

  end do !inv
 end do !iop
  
 deallocate(rhotwg_sym)

 end if 

end subroutine assemblychi0sf
!!***
