!{\src2tex{textfont=tt}}
!!****f* ABINIT/coulombian_driver
!! NAME
!! coulombian_driver
!!
!! FUNCTION
!!  driver for calculations in isolated systems using the Coulomb cutoff
!!  technique (see PRB ....)
!!
!! COPYRIGHT
!!  Copyright (C) 2007 ABINIT group (MG)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!  nq=numer of q-points 
!!  qpt(3,nqpt)= q-points in reduced coordiantes
!!  ng=number of g vectors
!!  gvec(3,ng)= G vectors in reduced coordinates
!!  cutp <type cutoff_data> datatype gathering information on the coulombian cutoff technique
!!  gmet(3,3)= metric in reciprocal space
!!  gprimd(3,3)=dimensional primitive translations for reciprocal space ($\textrm{bohr}^{-1}$)
!!
!! OUTPUT
!!  vccut(nq,ng)= modified Coulombian interaction
!!
!! SIDE EFFECTS
!!
!! NOTES
!!  The datatype cutoff_data is supposed to be correctly initialised in the calling subroutine
!!
!! PARENTS
!! 
!!
!! CHILDREN
!!  
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine coulombian_driver(nq,qpt,ng,gvec,cutp,gmet,gprimd,vccut)
    
 use defs_basis
 use defs_datatypes

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01manage_mpi
 use interfaces_11util
 use interfaces_12geometry
 use interfaces_15gw, except_this_one => coulombian_driver
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: ng,nq
 type(cutoff_data),intent(in) :: cutp
!arrays
 integer,intent(in) :: gvec(3,ng)
 real(dp),intent(in) :: qpt(3,nq),gmet(3,3),gprimd(3,3)
 real(dp),intent(out) :: vccut(nq,ng)

!Local variables-------------------------------
!scalars
 integer :: iq,ig,iqg
 character(len=500) :: message      
 character(len=fnlen) :: filnam
 logical,parameter :: DEBUG=.true.
!arrays
 integer,allocatable :: insort(:)
 real(dp),allocatable :: qpg(:)

 real(dp) :: h !for the moment
 
! *************************************************************************

! Perform general checks 
! Call appropriate subroutines to calculate the coulombian
!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifndef HAVE_FORTRAN_INTERFACES
 real(dp) :: normv
#endif
!End of the abilint section

 select case (trim(cutp%cutoffmode)) 

  case ('sphere')

   write(message,'(5a,f8.4,3a,f8.2,3a,3f8.5,2a)')ch10,&
&   ' Spherical cutoff info :',ch10,ch10,&
&   '  cutoff radius        =  ',cutp%rcut,' Bohr ',ch10,&
&   '  volume of the sphere =  ',4*pi/3*cutp%rcut**3,' Bohr^3 ',ch10,&
&   '  sphere centered at   =  ',cutp%boxcenter,' (r.l.u) ',ch10
   call wrtout(06,message,'COLL')
   call wrtout(ab_out,message,'COLL')

   call cutoff_sphere (nq,qpt,ng,gvec,gmet,cutp%rcut,vccut)

  case ('cylinder')

   write(message,'(5a,f8.4,3a,3f8.2,2a)')ch10,&
&   ' Cylindrical cutoff info :',ch10,ch10,&
&   '  cutoff radius        =  ',cutp%rcut,' Bohr ',ch10,&
&   '  centered at       =  ',cutp%boxcenter,' (r.l.u) ',ch10
   call wrtout(06,message,'COLL')
   call wrtout(ab_out,message,'COLL')

!XG070407 :Temporarily disabled : contained a common
   call leave_new('COLL')
!  call cutoff_cylinder (nq,qpt,ng,gvec,gprimd,gmet,cutp%rcut,h,vccut)
!

  case ('surface') 

   write(message,'(5a,f8.4,3a,3f8.2,2a)')ch10,&
&   ' Surface cutoff info :',ch10,ch10,&
&   '  cutoff radius        =  ',cutp%rcut,' Bohr ',ch10,&
&   '  centered at       =  ',cutp%boxcenter,' (r.l.u) ',ch10
   call wrtout(06,message,'COLL')
   call wrtout(ab_out,message,'COLL')

   call cutoff_surface (nq,qpt,ng,gvec,gprimd,gmet,cutp%rcut,vccut)

  case default 
   write(message,'(6a)')ch10,&
&   ' cutoff_driver : BUG -',ch10,&                               
&   '  cutoffmode ',trim(cutp%cutoffmode),' not yet implemented'
   call wrtout(6,message,'COLL')
   call leave_new('COLL')
 end select 

 if (DEBUG) then

! Plot vccut(q,G) as a function of |q+G|
  filnam='VCCUT'
  call isfile(filnam,'new')

  allocate(insort(nq*ng),qpg(nq*ng))
  iqg=1
  do iq=1,nq
   do ig=1,ng
    qpg(iqg)=normv(qpt(:,iq)+gvec(:,ig),gmet,'g')
    insort(iqg)=iqg
    iqg=iqg+1
   end do
  end do
  call sort_dp(nq*ng,qpg,insort,tol14)

  open(88,file=filnam,status='new',form='formatted')
  write(88,*)'|qpg| q (',ng,' ) gvec (',ng,' ) vccut'
  do iqg=1,nq*ng
    iq=(insort(iqg)-1)/ng +1
    ig=(insort(iqg))-(iq-1)*ng
    write(88,'(4f12.6,3i6,es14.6)') qpg(iqg),qpt(:,iq),gvec(:,ig),vccut(iq,ig)
   end do 
  close(88)

  deallocate(insort,qpg)

 end if 

end subroutine coulombian_driver
!!***
