!{\src2tex{textfont=tt}}
!!****f* ABINIT/cppm2par
!! NAME
!! cppm2par
!!
!! FUNCTION
!! Calculate the plasmon-pole parameters using Hybertsen and Louie model (PRB 34, 5390 (1986))
!!
!! COPYRIGHT
!! Copyright (C) 1999-2007 ABINIT group (RShaltaf, GMR, XG, MG)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!  epsm1(npwvec,npwvec,nomega,nq)=symmetrized inverse dielectric matrix at nomega frequencies, and nq wavevectors
!!  npwvec=number of plane waves in epsm1
!!  nomega=number of frequencies (usually 2 but this plasmon-pole requires only the static symmetrized dielectric matrices
!!  nq=number of q points
!!  q(3,nq)=q-points
!!  omega(nomega)=frequencies
!!  qratio(npwec,npvec,nq)= (q+G).(q+Gp)/|q+G|^2
!!  rho(nr)=charge density on the real space FFT grid
!!  ngff1,ngfft2,ngfft3= FFT dimensions
!!  nr= total number of points in the FFT mesh
!!  b1(3),b2(3),b3(3)= reciprocal lattice vectors
!!
!! OUTPUT
!!  bigomegatwsq(npwvec,npwvec,nq)= squared bare plasma frequencies
!!   \Omega^2_{G1 G2}(q) = 4\pi \frac {(q+G1).(q+G2)}/{|q+G1|^2} n(G1-G2)
!!
!!  omegatw(npwvec,npwvec,nq)= plasmon frequencies \tilde\omega_{G1 G2}(q) where:
!!  \tilde\omega^2_{G1 G2}(q) = 
!!    \frac {\Omega^2_{G1 G2}(q)} {\delta_{G1 G2}-\tilde\epsilon^{-1}_{G1 G2} (q, \omega=0)}
!!
!! PARENTS
!!      sigma
!!
!! CHILDREN
!!      cggfft,fourdp
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine cppm2par(npwvec,nq,nomega,epsm1,bigomegatwsq,omegatw,&
& ngfft1,ngfft2,ngfft3,gvec,qratio,rho,nr,qpt,b1,b2,b3)

 use defs_basis
 use defs_datatypes

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01manage_mpi
 use interfaces_12ffts
 use interfaces_15gw, except_this_one => cppm2par
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: ngfft1,ngfft2,ngfft3,nomega,npwvec,nq,nr
!arrays
 integer,intent(in) :: gvec(3,npwvec)
 !MG Here there was a bug since q was defined as q(nq), doesnt affect results if use_symmetrized=.true.
 real(dp),intent(in) :: b1(3),b2(3),b3(3),qpt(3,nq),qratio(npwvec,npwvec,nq)
 real(dp),intent(inout) :: rho(nr)
 complex,intent(in) :: epsm1(npwvec,npwvec,nomega,nq)
 complex,intent(out) :: bigomegatwsq(npwvec,npwvec,nq)
 complex,intent(out) :: omegatw(npwvec,npwvec,nq)

!Local variables-------------------------------
!scalars
 integer :: ig,ig_s,igp,igp_s,iq,istat,j,nimwp,tim_fourdp=2
 real(dp) :: lambda,phi,x1
 logical,parameter :: use_symmetrized=.true.
 logical,parameter :: verbose=.true.
 character(len=500) :: message
 type(MPI_type) :: mpi_enreg
!arrays
 integer :: ngfft(18)
 integer,allocatable :: igfft(:,:)
 real(dp),allocatable :: qplusg(:),rhog_dp(:,:)
 complex,allocatable :: omegatwsq(:,:),rhog(:),rhogg(:,:),temp(:,:)

!*************************************************************************

! Compute the density in G space rhog(G)
 allocate(rhog_dp(2,nr),rhog(nr))

 ngfft(1)=ngfft1
 ngfft(2)=ngfft2
 ngfft(3)=ngfft3
 ngfft(4)=2*(ngfft(1)/2)+1
 ngfft(5)=2*(ngfft(2)/2)+1
 ngfft(6)=ngfft(3)
 ngfft(7)=100
 ngfft(8)=256
 ngfft(9)=0
 ngfft(10)=1
 ngfft(11)=0
 ngfft(12)=ngfft2
 ngfft(13)=ngfft3
 ngfft(14)=0

! Conduct FFT to rhog
 mpi_enreg%me_fft=0
 mpi_enreg%nproc_fft=1

 call fourdp(1,rhog_dp,rho,-1,mpi_enreg,nr,ngfft,0)
 rhog(1:nr)=cmplx(rhog_dp(1,1:nr),rhog_dp(2,1:nr))

! Calculate the FFT index of each (G-Gp) vector and assign 
! the value of the correspondent density simultaneously
 allocate(igfft(npwvec,npwvec),rhogg(npwvec,npwvec))

 call cggfft(npwvec,ngfft1,ngfft2,ngfft3,gvec,igfft)

 do ig=1,npwvec
  do igp=1,npwvec
   rhogg(ig,igp)=rhog(igfft(ig,igp))
  end do
 end do


!MG commented this part since it has just been done 
!   moreover igfft <= nr by definition
! do ig=1,npwvec
!  do igp=1,npwvec
!   if(igfft(ig,igp)>nr)then
!    write (message,'(a,a,a)') &
!&    'BUG:can not find rho(G-Gpr) for some G, Gpr, contact ABINIT group',ch10,&
!&    'program will stop'
!    call wrtout(std_out,message,'COLL')
!    call leave_new('COLL')
!   end if
!   rhogg(ig,igp)=rhog(igfft(ig,igp))
!  end do
! end do
!END MG

 rhogg(:,:)= 4*pi*rhogg(:,:)

! Now we have rhogg
 deallocate(igfft,rhog_dp,rhog)

! Start calculating GPP parameters
! unsymmetrized epsm1 -> epsm1=|q+Gp|/|q+G|*epsm1
 allocate(qplusg(npwvec),stat=istat)
 if(istat/=0) stop 'qplusg out of memory'
 allocate(temp(npwvec,npwvec),stat=istat)
 if(istat/=0) stop 'temp out of memory'
 allocate(omegatwsq(npwvec,npwvec),stat=istat)
 if(istat/=0) stop 'temp out of memory'

 do iq=1,nq

  temp(:,:)=-1*epsm1(:,:,1,iq)
! RS still not obvious for me whether one shall use the symmetrized inverse DM or the unsymmetrized one
! the default here is to use the symmetrized one, I must discuss this with XG
!
! MG it turns out that using the symmetrized inverse DM in the plasmon-pole
! equations give the same results for the squared plasmon frequencies omegatwsq while the 
! squared bare plasma frequencies bigomegatwsq related to the symmetrized dielectric matrix 
! are obtained multipling by |q+G1|/|q+G2|   

  if (.not.use_symmetrized) then
   call cvc(nq,iq,qpt,b1,b2,b3,npwvec,gvec,qplusg)
   do ig=1,npwvec
    do igp=1,npwvec
     temp(ig,igp)=qplusg(igp)/qplusg(ig)*temp(ig,igp)
    end do
   end do
  end if

  nimwp=0
  do ig=1,npwvec
   temp(ig,ig)=temp(ig,ig)+1
   do igp=1,npwvec
    bigomegatwsq(ig,igp,iq)=rhogg(ig,igp)*qratio(ig,igp,iq)
    omegatwsq(ig,igp)=bigomegatwsq(ig,igp,iq)/temp(ig,igp)

! Set omegatw to any arbitrary number to avoid dealing with undefined numbers like (INF)
! simply ignore all cases of omegatw with imaginary values
! in principle these correspond to cases where the imaginary part of epsm1 does not have
! a well defined peak. The imaginary part of epsm1 in these cases oscillates  with a small amplitude
! since the amplitude A_GGpr=-pi/2*bigomegatwsq/omegatw, 
! it follows that bigomegatwsq shall be set to zero for these cases

    if (real(omegatwsq(ig,igp))<=zero .or. aimag(omegatwsq(ig,igp))**2*tol12>real(omegatwsq(ig,igp))**2)then
     bigomegatwsq(ig,igp,iq)=(0.,0.)
     omegatw(ig,igp,iq)=(ten,0.)
     nimwp=nimwp+1
     if (verbose) then 
      write(message,'(a,i3,2i8)')' imaginary plasmon frequency at : ',iq,ig,igp
      call wrtout(6,message,'COLL')
     end if 
    else 
     !MG this part has been added to deal with systems without inversion symmetry
     !this new implementation gives the same results as the previous one if 
     !omegatwsq is a pure real number and has the advantage of being an improved
     !approach for systems without an inversion center
     lambda=abs(omegatwsq(ig,igp))
     phi=atan(aimag(omegatwsq(ig,igp))/real(omegatwsq(ig,igp)))
     omegatw(ig,igp,iq)=sqrt(lambda/cos(phi))
     bigomegatwsq(ig,igp,iq)=bigomegatwsq(ig,igp,iq)*(1.-(0.,1.)*tan(phi))
     !MG OLD CODING , uncomment the following line and comment the previous to restore the old version
     !omegatw(ig,igp,iq)=sqrt(real(omegatwsq(ig,igp)))
     !END OLD CODING
    end if
   end do
  end do

  write(message,'(a,3f12.6,a,i8,a,i8)')' at q-point : ',qpt(:,iq),&
& ' the number of imaginary plasmon-pole frequencies is ',nimwp,' / ',npwvec**2
  call wrtout(6,message,'COLL')

 end do !iq

 deallocate(omegatwsq,rhogg,temp,qplusg)

 write(message,'(2a,f12.8,2a,3i5)')ch10,&
 ' cppm2par : omega twiddle minval [eV] = ',minval(abs(omegatw(:,:,:)))*Ha_eV,ch10,&
 '            omega twiddle min location',minloc(abs(omegatw(:,:,:)))
 call wrtout(6,message,'COLL')

end subroutine cppm2par
!!***
