!{\src2tex{textfont=tt}}
!!****f* ABINIT/cppm3par
!! NAME
!! cppm3par
!!
!! FUNCTION
!! Calculate the plasmon-pole parameters using the von Linden-Horsh model (PRB 37, 8351, 1988)
!! (see also Pag 22 of Quasiparticle Calculations in Solids. Aulbur et. al)  
!!
!! COPYRIGHT
!! Copyright (C) 1999-2007 ABINIT group (RShaltaf, GMR, XG)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!  epsm1(npwvec,npwvec,nomega,nq)= symmetrized inverse dielectric matrix at nomega frequencies, and nq wavevectors
!!  mpi_enreg=information about MPI parallelization
!!  npwvec=number of plane waves in epsm1
!!  qratio=(q+G1).(q+G2)/(|q+G1|.|q+G2|)
!!  rho(nr)=charge density on the real space FFT grid
!!  ngfft1,ngfft2,ngfft3=FFT grid
!!  nr=ngfft1*ngfft2*ngfft3=number of points in the FFT grid
!!  gvec(3,npwvec)= G vectors in reduced coordinates
!!
!! OUTPUT
!!  omegatw(npwvec,npwvec,nq)= plasmon pole positions
!!  bigomegatwsq(npwvec,npwvec,nq)=(E_{q,ii}^{-1}-1)*omegatw
!!   where E^{-1} is the eigenvalue of the inverse dielectric matrix
!!  eigtot(npwvec,npwvec,nq)=the eigvectors of the symmetrized inverse dielectric matrix 
!!   (first index for G, second index for bands)
!!
!! PARENTS
!!      sigma
!!
!! CHILDREN
!!      cggfft,chpev,fourdp,leave_new,wrtout,zhpev
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine cppm3par(npwvec,nq,nomega,epsm1,bigomegatwsq,omegatw,&
& ngfft1,ngfft2,ngfft3,gvec,rho,nr,eigtot,q,b1,b2,b3)

 use defs_basis
 use defs_datatypes

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01manage_mpi
 use interfaces_12ffts
 use interfaces_15gw, except_this_one => cppm3par
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: ngfft1,ngfft2,ngfft3,nomega,npwvec,nq,nr
!arrays
 integer,intent(in) :: gvec(3,npwvec)
 real(dp),intent(in) :: b1(3),b2(3),b3(3),q(3,nq)
 real(dp),intent(inout) :: rho(nr)
 complex,intent(in) :: epsm1(npwvec,npwvec,nomega,nq)
 complex,intent(out) :: bigomegatwsq(npwvec,1,nq),eigtot(npwvec,npwvec,nq)
 complex,intent(out) :: omegatw(npwvec,1,nq)

!Local variables-------------------------------
!scalars
 integer :: ierr,ig,igp,ii,index,iq,istat,jj,tim_fourdp=2
 real(dp) :: num,qpg_dot_qpgp
 character(len=1) :: jobs,uplo
 character(len=500) :: message
 type(MPI_type) :: mpi_enreg
!arrays
 integer :: ngfft(18)
 integer,allocatable :: igfft(:,:)
 real(dp) :: gppq(3),gpq(3)
 real(dp),allocatable :: eigval(:),qplusg(:),rhog_dp(:,:),zhpev2(:)
!no_abirules
 complex*16 ::conjg_eig
 complex*16,allocatable :: matr(:),zhpev1(:),eigvec(:,:)
 complex*16,allocatable :: zz(:,:),temp(:,:),mm(:,:,:)
 complex*16,allocatable :: omegatwsq(:,:),rhog(:),rhogg(:,:)

!*************************************************************************

#ifdef __VMS
!DEC$ ATTRIBUTES ALIAS:'ZHPEV' :: zhpev
#endif

! Compute the density in G space rhog(G)
  allocate(rhog_dp(2,nr),stat=istat)
  if(istat/=0) stop 'rhog_dp out of memory'
  allocate(rhog(nr),stat=istat)
  if(istat/=0) stop 'rhog out of memory'
  allocate(igfft(npwvec,npwvec),stat=istat)
  if(istat/=0) stop 'igfft out of memory'
  allocate(rhogg(npwvec,npwvec),stat=istat)
  if(istat/=0) stop 'rhogg out of memory'

  ngfft(1)=ngfft1
  ngfft(2)=ngfft2
  ngfft(3)=ngfft3
  ngfft(4)=2*(ngfft(1)/2)+1
  ngfft(5)=2*(ngfft(2)/2)+1
  ngfft(6)=ngfft(3)
  ngfft(7)=100
  ngfft(8)=256
  ngfft(9)=0
  ngfft(10)=1
  ngfft(11)=0
  ngfft(12)=ngfft2
  ngfft(13)=ngfft3
  ngfft(14)=0

! Conduct FFT to rhog
 call fourdp(1,rhog_dp,rho,-1,mpi_enreg,nr,ngfft,0)

 rhog(1:nr)=cmplx(rhog_dp(1,1:nr),rhog_dp(2,1:nr))

! Calculate the FFT index of each (G-Gp) vector and assign the value
! of the correspondent density simultaneously

 call cggfft(npwvec,ngfft1,ngfft2,ngfft3,gvec,igfft)

 do ig=1,npwvec
  do igp=1,npwvec
   if(igfft(ig,igp)>nr)then
    write (message,'(a,a,a)') &
&    'BUG:can not find rho(G-Gpr) for some G, Gpr, contact ABINIT group',ch10,&
&    'program will stop'
    call wrtout(std_out,message,'COLL')
    call leave_new('COLL')
   end if
   rhogg(ig,igp)=rhog(igfft(ig,igp))
  end do
 end do

! mm(G,Gp) = (q+G) \cdot (q+Gp) n(G-Gp)
 allocate(mm(npwvec,npwvec,nq),stat=istat)
 if(istat/=0) stop 'mm out of memory'

 do iq=1,nq

  do ig=1,npwvec
   if(all(abs(q(:,iq))<1.0e-3))then
! To be discussed with Riad, here we should use the small q 
! to be consistent and consider the limit q-->0
    gpq(1)=gvec(1,ig)
    gpq(2)=gvec(2,ig)
    gpq(3)=gvec(3,ig)
   else
    gpq(1)=gvec(1,ig)+q(1,iq)
    gpq(2)=gvec(2,ig)+q(2,iq)
    gpq(3)=gvec(3,ig)+q(3,iq)
   end if
   do igp=1,npwvec
    if(all(abs(q(:,iq))<1.0e-3))then
     gppq(1)=gvec(1,igp)
     gppq(2)=gvec(2,igp)
     gppq(3)=gvec(3,igp)
    else
     gppq(1)=gvec(1,igp)+q(1,iq)
     gppq(2)=gvec(2,igp)+q(2,iq)
     gppq(3)=gvec(3,igp)+q(3,iq)
    end if
    qpg_dot_qpgp=0
    do ii=1,3
     qpg_dot_qpgp=qpg_dot_qpgp+&
&     ( gpq(1)*b1(ii) +gpq(2)*b2(ii) +gpq(3)*b3(ii))*&
&     (gppq(1)*b1(ii)+gppq(2)*b2(ii)+gppq(3)*b3(ii))
    end do
    mm(ig,igp,iq)= rhogg(ig,igp)*qpg_dot_qpgp
   end do !igp
  end do !ig
 end do !iq

! Now we have rhogg,rho0

 deallocate(rhog_dp,rhog,igfft)

! Calculate the dielectric matrix eigenvalues and vectors
! Use only the static epsm1 i.e., only the w=0 part (eps(:,:,1,:))

 allocate(eigval(npwvec),stat=istat)
 if(istat/=0) stop 'eigval out of memory'
 allocate(eigvec(npwvec,npwvec),stat=istat) ! eigenvalues and vectors of DM
 if(istat/=0) stop 'eigvec out of memory'
 allocate(temp(npwvec,npwvec),stat=istat)
 if(istat/=0) stop 'temp out of memory'
 allocate(zz(npwvec,nq),stat=istat)
 if(istat/=0) stop 'zz of memory'

 zz(:,:)=czero

 allocate(qplusg(npwvec),stat=istat)
 if(istat/=0) stop 'qplusg out of memory'

 do iq=1,nq

! Store the susceptibility matrix in upper mode before calling zhpev for each iq value

  allocate(matr(npwvec*(npwvec+1)/2),stat=istat)
  if(istat/=0) stop 'matr of memory'

  index=1
  do ii=1,npwvec
   do jj=1,ii
    matr(index)=epsm1(jj,ii,1,iq)
    index=index+1
   end do
  end do

  allocate(zhpev2(3*npwvec-2),stat=istat)
  if(istat/=0) stop 'zhpev2 of memory'
  allocate(zhpev1(2*npwvec-1),stat=istat)
  if(istat/=0) stop 'zhpev1 of memory' ! working arrays for lapack

#if defined T3E
  call CHPEV('V','U',npwvec,matr,eigval,eigvec,npwvec,zhpev1,zhpev2,ierr)
#else
  call zhpev('V','U',npwvec,matr,eigval,eigvec,npwvec,zhpev1,zhpev2,ierr)
#endif

  deallocate(matr,zhpev2,zhpev1) ! working arrays for lapack

  if (ierr<0) then
   write (message,'(5a,i4,2a)')ch10,&
&   ' cppm3par : ERROR- ',ch10,&
&   '  failed to calculate the eigenvalues and eigenvectors of the dielectric matrix ',ch10,&
&   ierr*(-1),' th argument in the matrix has an illegal value',ch10
   call wrtout(std_out,message,'COLL')
   call leave_new('COLL')
  end if

  if (ierr>0) then
   write(message,'(6a,i4,5a)')ch10,&
&   ' cppm3par : ERROR- ',ch10,&
&   '  failed to calculate the eigenvalues and eigenvectors of the dielectric matrix ',ch10,&
&   '  the algorithm failed to converge; ', ierr,ch10,&
&   '  off-diagonal elements of an intermediate tridiagonal form ',ch10,&
&   '  did not converge to zero',ch10
   call wrtout(std_out,message,'COLL')
   call leave_new('COLL')
  end if

! Calculate the PPM parameters and the eigenpotentials needed for 
! the calculation of the generalized overlap matrix
! Note: the eigenpotentials has to be calculated on the FFT (G-Gp) index

! Save eigenvectors of \tilde\epsilon^{-1}
! MG well it is better to save \Theta otherwise 
! we have to calculare \Theta for each band, spin, k-point but oh well
  eigtot(:,:,iq)=eigvec(:,:)

  call cvc(nq,iq,q,b1,b2,b3,npwvec,gvec,qplusg)

! Basic Equation:
!
! \Theta_{q,ii}(G)=\Psi_{q,ii}(G)/|q+G|
! where \Psi_{q,ii}(G) is the eigenvector of \tilde\epsilon^{-1} 

! \tilde\omega_{ii,q}^2= 4\pi (1-eigenval(ii,q))) 
!  \sum_{G,Gp} \Theta^*_{q,ii}(G) (q+G)\cdot(q+Gp) n(G-Gp) \Theta_{q,ii}(Gp) 

  do ii=1,npwvec !DM band
! Calculate \Theta_{q,ii}(G)
! why the first element is not modified? if the problem is the small value of qplusg(1)
! we could multiply by sqrt(mod((q+G)(q+G'))) and then add the sing at the end 
   if(iq==1)then
    eigvec(2:,ii)=eigvec(2:,ii)/qplusg(2:)
   else
    eigvec(:,ii)=eigvec(:,ii)/qplusg(:)
   end if
   do ig=1,npwvec
    conjg_eig=conjg(eigvec(ig,ii))
    do igp=1,npwvec
     if(iq==1 .and. ig==1 .and. igp==1)then
      zz(ii,iq)=zz(ii,iq)+conjg_eig*rhogg(ig,igp)*eigvec(igp,ii)
     else
      zz(ii,iq)=zz(ii,iq)+conjg_eig*mm(ig,igp,iq)*eigvec(igp,ii)
     end if
    end do
   end do

   num=1-eigval(ii)
   if (num<=0.0_dp) then
!here I think we should set bigomegatwsq=0 and omegatw to an arbitrary value
!maybe we can output a warning TO BE discussed with Riad 
    if (abs(num)<1.0d-4) then
     num=1.0d-5
    else
     write(message,'(5a)')ch10,&
&     ' cppm3par : BUG - ',ch10,&
&     ' one or more imaginary plasmon pole energies',ch10
     call wrtout(std_out,message,'COLL')
     call leave_new('COLL')
    end if
   end if

   omegatw(ii,1,iq)=sqrt(4*pi*real(zz(ii,iq))/(num))
!this should be \alpha = 2\pi omegatw * (1-eigenval) 
!MG check this, in the review I found a factor 2\pi, maybe it is reintroduced later
   bigomegatwsq(ii,1,iq)=num*omegatw(ii,1,iq)

  end do
 end do !iq

 deallocate(rhogg)
 deallocate(temp,mm)
 deallocate(eigval,zz)
 deallocate(eigvec,qplusg)

 write(*,*)' cppm3par : omega twiddle minval [eV] = ',minval(abs(omegatw(:,:,:)))*Ha_eV
 write(*,*)' cppm3par : omega twiddle min location',minloc(abs(omegatw(:,:,:)))

end subroutine cppm3par
!!***
