!{\src2tex{textfont=tt}}
!!****f* ABINIT/crho_augmented
!! NAME
!! crho_augmented
!!
!! FUNCTION
!!  Calculate the density using an augmented FFT mesh to avoid wrap around 
!!  errors in the Fourier coefficients (needed if ppmodel ==2,3,4)
!!
!! COPYRIGHT
!!  Copyright (C) 2007 ABINIT group (MG)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!  gmet(3,3) = reciprocal space metric.
!!  enforce_sym = flag to enforce a FFT compatible with all the symmetry operations (should be default)
!!  gvec(3,npwvec) = G-vectors in reduced coordinates 
!!  mpi_enreg= datatype gathering information on parallelism, variables used 
!!   |gwpara= 0 no parallelism 
!!   |        1 parallelism is over k-points
!!   |        2 bands are spread btw processors
!!  npwvec= maximum number of G vectors used
!!  npwc= number of plane-waves in epsm1
!!  npwwfn = number of G-vectors for wavefunctions
!!  ninv= 2 if time reversal symmetry can be used, 1 otherwise
!!  nop=number of symmetry operations
!!  op(3,3,nop)= symmetry operations in reciprocal space
!!  ngfft1d,ngfft2d,ngfft3d= FFT paramenters related to the augmented mesh 
!!  ngfft1dd,ngfft2dd,ngfft3dd= FFT parameters (presently not used, needed for
!!   hystorical reasons and shuld be eliminated)
!!  nrd=FFT points in the augmented FFT mesh
!!  tnons(3,nop) = fractional translations
!!  nkibz=number of k-points in the IBZ 
!!  nkbz=number of points if the full BZ 
!!  nsppol=1 for unpolarized, 2 for spin-polarized
!!  occ(nkibz,nbnds_occ,nsppol) = occupation numbers for nbnds bands, nkibz irreducible k-points, and each spin
!!  nbnds_occ=number of (occupied) bands 
!!  ucvol = unit cell volume.
!!  wtk(nkibz) = irreducible k-points weights.
!!  wfg(npwwfn,nbnds_occ,nkibz,nsppol)= wavefunctions in reciprocal space, for each band, k point and spin
!!
!! OUTPUT
!!  rhod(nrd,nsppol)=charge density on the augmented FFT grid 
!!
!! SIDE EFFECTS
!!
!! NOTES
!!
!! PARENTS
!! 
!!
!! CHILDREN
!! 
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine crho_augmented(gmet,gvec,npwvec,npwwfn,npwc,nbnds_occ,ninv,nop,op,tnons,&
& nkibz,nkbz,nsppol,wfg,ngfft1d,ngfft2d,ngfft3d,nrd,ngfft1dd,ngfft2dd,ngfft3dd,    &
& occ,wtk,ucvol,mpi_enreg,min_band_proc,max_band_proc,rhod)

 use defs_basis
 use defs_datatypes

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01manage_mpi
 use interfaces_15gw, except_this_one => crho_augmented
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in)  :: npwvec,npwc,npwwfn,ninv,nop
 integer,intent(in)  :: nkibz,nkbz,nsppol,nbnds_occ
 integer,intent(in) ::  ngfft1d,ngfft2d,ngfft3d,nrd
 integer,intent(in) ::  ngfft1dd,ngfft2dd,ngfft3dd
 integer,intent(in) ::  min_band_proc,max_band_proc 
 real(dp),intent(in) :: ucvol
 type(MPI_type),intent(inout) :: mpi_enreg
!arrays
 integer,intent(in) :: gvec(3,npwvec) !or gvec(3,npwvec)
 real(dp),intent(in) :: gmet(3,3)
 real(dp),intent(in) :: tnons(3,nop),op(3,3,nop)
 real(dp),intent(in) :: occ(nkibz,min_band_proc:max_band_proc,nsppol)
 real(dp),intent(in) :: wtk(nkibz)
 real(dp),intent(out) :: rhod(nrd,nsppol)
 complex,intent(in) :: wfg(npwwfn,min_band_proc:max_band_proc,nkibz,nsppol)

!Local variables-------------------------------
 integer :: istat,tim_fourdp,ir,is
 real(dp) :: omegaplasma
 logical,parameter :: DEBUG=.true.
 character(len=500) :: message                 
!Arrays
 integer,allocatable :: igfftd(:,:,:,:)
 complex,allocatable :: wfrd(:,:,:,:)
 integer,allocatable  :: irottb(:,:),grottb(:,:,:),grottbm1(:,:,:) 
 
! *************************************************************************
 

 write(message,'(3a)')ch10,'crho_augmented : enter ',ch10
 call wrtout(6,message,'COLL')

! Set up tables for augmented FFT igfftd
 allocate(igfftd(npwwfn,5,5,5),stat=istat)
 if(istat/=0) stop 'out of memory in igfftd'
 
 call cigfft(npwwfn,ngfft1dd,ngfft1d,ngfft2d,ngfft3d,gvec,igfftd)

! Calculate KS wavefunctions in real space using augmented FFT
 allocate(wfrd(nrd,nbnds_occ,nkibz,nsppol),stat=istat)
 if(istat/=0) then
  call memerr('crho_augmented','wfrd',nrd*nbnds_occ*nkibz*nsppol,'spc')
 end if

 wfrd(:,:,:,:)=(0.,0.)

 tim_fourdp=5
!here wfg should be only related to occupied states
!should also remove min_band_proc,max_band_proc
 call fftwfn(npwwfn,min_band_proc,max_band_proc,nkibz,nrd,nsppol,wfg,wfrd,&
& igfftd(:,3,3,3),ngfft1d,ngfft1dd,ngfft2d,ngfft3d,tim_fourdp,mpi_enreg)

! Set up table indicating rotations of r-points
! I do not need grottb grottbm1, only irottb
! this can be solved by modifying sutrot 
 allocate(irottb(nrd,nop),grottb(npwwfn,ninv,nop),grottbm1(npwwfn,ninv,nop),stat=istat)
 if(istat/=0) stop ' out of memory in rotations tables'

 irottb(:,:)=0
 grottb(:,:,:)=0
 grottbm1(:,:,:)=0

 call surot(op,tnons,nop,ninv,ngfft1d,ngfft1dd,ngfft2d,ngfft3d,nrd,npwvec,gvec,grottb,irottb,grottbm1)

! If nsppol==2, crho reports the total charge in the first half
! and the spin up charge density in the second half
! if the  wavefunctions are not in memory we can use density.F90
! presently the wfr are supposed to be in memory
 call crho(irottb,nbnds_occ,ninv,nkbz,nkibz,nkibz,nop,nrd,nrd,nsppol,&
&          occ,omegaplasma,rhod,ucvol,wfrd,wtk,mpi_enreg,&
&          min_band_proc,max_band_proc)

! Got a density in the augmented FFT mesh in real-space

!DEBUG
!write total charge:
!if (DEBUG) then
! open(unit=66,file='rho_aug')
! do ir=1,nrd
!  write(66,'(2x,f7.4)') (rhod(ir,is),is=1,nsppol)
! end do
!close(66)
!end if 
!write(std_out,*) ' crho_augmented : exit'
!ENDDEBUG

end subroutine crho_augmented
!!***
