!{\src2tex{textfont=tt}}
!!****f* ABINIT/cutoff_density
!! NAME
!! cutoff_density
!!
!! FUNCTION
!!  Modify density in case of calculations with Coulomb cutoff
!!
!! COPYRIGHT
!!  Copyright (C) 2007 ABINIT group (MG)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!  ngfft(18)=information on the FFT grid 
!!  (at the moment only the first three elements are used and are checked
!!  against the values contained in the cutoff_data datatype)
!!  nspden=input variable 
!!  cutp <type cutoff_data>, only ctab(ngfft(1)*ngfft(2)*ngfft(3)) is used: 
!!   For each point of the FFT grid gives 
!!   1 if the point in inside the cutoff region
!!   0 otherwise
!!  ucvol=unit cell volume 
!!
!! OUTPUT
!!  rhor(ngfft(1)*ngfft(2)*ngfft(3),nspden) see side effects
!!
!! SIDE EFFECTS
!! rhor(ngfft(1)*ngfft(2)*ngfft(3),nspden)
!! Input: charge density on the FFT grid 
!! Output: the charge falling outside the cutoff region is set equal to zero
!! the overflowing charge is restored in order to preserve charge neutrality
!!
!! NOTES
!!
!! PARENTS
!!  
!!
!! CHILDREN
!!  
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine cutoff_density(ngfft,nspden,cutp,ucvol,rhor)
    
 use defs_basis
 use defs_datatypes

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01manage_mpi
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: nspden
 real(dp),intent(in) :: ucvol
 type(cutoff_data),intent(in) :: cutp
!arrays
 integer,intent(in) :: ngfft(18)
 real(dp),intent(inout) :: rhor(ngfft(1)*ngfft(2)*ngfft(3),nspden)

!Local variables-------------------------------
!scalars
 integer :: ix,iy,iz,ir,is,ngfft1,ngfft2,ngfft3   
 real(dp) :: eps,fact
 character(len=500) :: message      
!arrays
 real(dp) :: rr(3),rxy(3),rz(3) 
 real(dp) :: overflow(nspden),nel(nspden) 
 
! *************************************************************************
 
!check consistency 
 if (any(ngfft(1:3)/=cutp%ngfft(1:3))) stop 'BUG in cutoff_density'

 ngfft1=ngfft(1) 
 ngfft2=ngfft(2) 
 ngfft3=ngfft(3)

 overflow=zero
 nel=zero

 do ix=0,ngfft1-1
  do iy=0,ngfft2-1
   do iz=0,ngfft3-1

    rr(1)=dble(ix)/ngfft1
    rr(2)=dble(iy)/ngfft2
    rr(3)=dble(iz)/ngfft3
    ir= 1 + ix +iy*ngfft1 + iz*ngfft1*ngfft2

    nel(:)=nel(:)+rhor(ir,:)
   
    if (cutp%ctab(ir)==0) then 
     overflow(:)=overflow+rhor(ir,:)
     rhor(ir,:)=zero
    end if 

   end do 
  end do 
 end do

 nel(:)=nel(:)*ucvol/(ngfft1*ngfft2*ngfft3)
 write(*,*)' number of electrons = ',(nel(is),is=1,nspden)

 overflow(:)=overflow(:)*ucvol/(ngfft1*ngfft2*ngfft3)
 write(*,*)'overflow =',(overflow(is),is=1,nspden)

! If overflow is larger that few percent of the total charge warn or stop
! since one should increase the size of the supercell or the cutoff radius

 eps=1.d-2
 if (any(overflow(:) >  eps*nel(:)))  then 
  if (any(overflow(:) >  5*eps*nel(:))) then ! >5%
   write(message,'(4a,f8.5,3a)')ch10,&
&   ' cutoff_density : ERROR - ',ch10,&
&   '  more than ',5*eps,' % of the charge density is outside the cutoff region',ch10,&
&   '  increase the cutoff radius and check the size of the supercell'
   call wrtout(6,message,'COLL')
   call leave_new('COLL')
  else
! FIXME should write more information but Im lazy
   write(message,'(4a,f8.5,3a)')ch10,&
&   ' cutoff_density : WARNING - ',ch10,&
&   '  about ',eps,' % of the charge density is outside the cutoff region',ch10,&
&   '  You could try to increase the cutoff radius '
   call wrtout(6,message,'COLL')
   call leave_new('COLL')
  end if 
 end if 

 write(*,*)'restoring removed charge '

! Restore charge neutrality, spreading the overflow inside the cutoff region
! Using this approach if the charge is zero close to 
! the boundary then there should be no spurious effect
 do is=1,nspden
  fact=nel(is)/(nel(is)-overflow(is))
  rhor(:,is)=rhor(:,is)*fact
 end do

end subroutine cutoff_density
!!***
