!{\src2tex{textfont=tt}}
!!****f* ABINIT/cutoff_surface
!! NAME
!! cutoff_surface
!!
!! FUNCTION
!!  Calculate the Fourier components of an effective Coulombian interaction 
!!  within a slab of thickness 2*rcut which is symmetric wrt the xy plane 
!!
!! COPYRIGHT
!!  Copyright (C) 2007 ABINIT group (MG)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!  gprimd(3,3)=dimensional primitive translations for reciprocal space ($\textrm{bohr}^{-1}$)
!!  gvec(3,ng)=G vectors in reduced coordinates 
!!  rcut=cutoff radius
!!  ng=number of G vectors
!!  qpt(3,nq)= q-points 
!!  nq=number of q-points 
!!  gmet(3,3)=metric in reciprocal space
!!
!! OUTPUT
!!  vccut(nq,ng)= Fourier components of the effective coulombian interaction 
!!
!! SIDE EFFECTS
!!
!! NOTES
!!  The equation used is 
!!  v(Gxy,Gz) = \frac{4\pi}{G^2} [ 1 + e^{-Gxy*R} |Gz|/Gxy * sin(|Gz|R) -e^{-Gxy*R} cos(|Gz|R) ]
!! 
!!  Should describe singularities
!!
!! PARENTS
!!  
!!
!! CHILDREN
!!  
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine cutoff_surface(nq,qpt,ng,gvec,gprimd,gmet,rcut,vccut)
    
 use defs_basis 
 implicit none
 
!Arguments ------------------------------------
!scalars
 integer,intent(in)  :: nq,ng 
 real(dp),intent(in) :: rcut
!arrays
 integer,intent(in) :: gvec(3,ng)
 real(dp),intent(in) :: gprimd(3,3),gmet(3,3),qpt(3,nq)
 real(dp),intent(out) :: vccut(nq,ng) 

!Local variables-------------------------------
!scalars
 integer :: ig,iq,mu
 real(dp),parameter :: small=1d-4   !should check well this value, because it strongly affects the results
 real(dp) ::qpg2,qpg_xy,qpg_z
!character(len=500) :: message                  
!arrays
 real(dp),allocatable :: gcart(:,:),qcart(:,:)

! *************************************************************************

! From reduced to cartesian coordinates 
 allocate(gcart(3,ng))

 do ig=1,ng
  do mu=1,3
   gcart(mu,ig)=gprimd(mu,1)*gvec(1,ig)+gprimd(mu,2)*gvec(2,ig)+gprimd(mu,3)*gvec(3,ig)
  end do
 end do
 gcart(:,:)=two_pi*gcart(:,:)

 allocate(qcart(3,nq))

 do iq=1,nq
  do mu=1,3
   qcart(mu,iq)=gprimd(mu,1)*qpt(1,iq)+gprimd(mu,2)*qpt(2,iq)+gprimd(mu,3)*qpt(3,iq)
  end do
 end do
 qcart(:,:)=two_pi*qcart(:,:)
 
 do iq=1,nq
  do ig=1,ng

   qpg_xy= (qcart(1,iq)+gcart(1,ig))**2 + (qcart(2,iq)+gcart(2,ig))**2 
   qpg_z=  abs(qcart(3,iq)+gcart(3,ig))
   qpg2= qpg_xy+qpg_z**2
   qpg_xy= sqrt(qpg_xy)

   if (qpg_xy > small) then 
    vccut(iq,ig)= 4*pi/qpg2 * ( 1 + exp(-qpg_xy*rcut) * (qpg_z/qpg_xy * sin(qpg_z*rcut) - cos(qpg_z*rcut)) ) 
   else 
    if (qpg_z > small) then 
     vccut(iq,ig)= 4*pi/qpg_z**2 * ( 1 + cos(qpg_z*rcut) - qpg_z*rcut * sin(qpg_z*rcut) )
    else 
     vccut(iq,ig)= -two_pi*rcut**2
    end if 
   end if 

  end do !ig 
 end do !iq
 
 
 deallocate(qcart)
 deallocate(gcart)

end subroutine cutoff_surface
!!***
