!{\src2tex{textfont=tt}}
!!****f* ABINIT/density
!! NAME
!! density
!!
!! FUNCTION
!!  Compute the real-space density rho2, as well as
!!  the average density to initialize omegaplasma
!!
!!
!! COPYRIGHT
!! Copyright (C) 1999-2007 ABINIT group (GMR, VO, LR, RWG, MG, RShaltaf)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!  irottb(nr,nop) = symmetry operations on the FFT grid.
!!   irottb(r,S)=index of (R^-1)(r-t)  in the FFT array where S is one of the nop
!!   symmetry operation in reciprocal space and S=\transpose R^-1
!!  localrdwf= input variable (for parallel case) if 1, the KSS file is local to each machine
!!  mpi_enreg= datatype gathering information on parallelism, variables used 
!!   !gwpara= if 2 bands are spread btw processors
!!  nbnds=number of bands
!!  nkibz=number of k point in IBZ
!!  nkbz=number of k point in BZ
!!  ngfft1,ngfft1a,ngfft2,ngfft3=FFT grid dimensions
!!  ninv=2 if time-reversal symmetry is used, 1 if not
!!  nop=number of symmetry operations
!!  nr=number of point in FFT grid
!!  nsppol=1 for unpolarized, 2 for spin-polarized
!!  occ(nkibz,nbnds,nsppol)=occupation numbers, for each k point in IBZ, each band and spin
!!  ucvol=unit cell volume
!!  wtk(nkibz)=weights for k points (input variable)
!!
!! OUTPUT
!!  omegaplasma=Drude plasma frequency
!!  rho2(nr,nsppol) = the density on the FFT grid.
!!   (total in first half and spin-up in second half if nsppol=2)
!!
!! PARENTS
!!      screening
!!
!! CHILDREN
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine density(nbnds,nkibz,nkbz,nsppol,nop,ninv,nr,ngfft1,ngfft1a,ngfft2,ngfft3,&
& irottb,ucvol,wtk,occ,rho2,omegaplasma,mpi_enreg,min_band_proc,max_band_proc,localrdwf,&
& wfr) !optional

 use defs_basis
 use defs_datatypes

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01manage_mpi
 use interfaces_11util
 use interfaces_lib01hidempi
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: localrdwf,max_band_proc,min_band_proc,nbnds,ngfft1,ngfft1a,ngfft2
 integer,intent(in) :: ngfft3,ninv,nkbz,nkibz,nop,nr,nsppol
 real(dp),intent(in) :: ucvol
 real(dp),intent(out) :: omegaplasma
 type(MPI_type),intent(inout) :: mpi_enreg
!arrays
 integer,intent(in) :: irottb(nr,nop)
 real(dp),intent(in) :: occ(nkibz,nbnds,nsppol)
 real(dp),intent(in) :: wtk(nkibz)
 real(dp),intent(out) :: rho2(nr,nsppol)
 complex,intent(in),optional :: wfr(nr,min_band_proc:max_band_proc,nkibz,nsppol)

!Local variables-------------------------------
!scalars
 integer,parameter :: unitwfr=26
 integer :: ib,ierr,ik,iop,ir,is,istat,master,me,spaceComm
 real(dp) :: fact,rhoav,rs,tnepuc
 character(len=500) :: message
 character(len=fnlen) :: filnam
 logical:: i_can_read,master_must_cast_data,wfr_in_mem
 logical :: DEBUG
!arrays
 real(dp),allocatable :: rho(:)
 complex,allocatable :: wfnk(:,:)

! *************************************************************************

 ! Initialize MPI related quantities
 call xcomm_init(mpi_enreg,spaceComm)  ! Init me
 call xme_init(mpi_enreg,me)           ! Init master
 call xmaster_init(mpi_enreg,master)

 write(message,'(a)')' density: calculating charge density'
 call wrtout(6,message,'COLL')

 i_can_read=.false. ; master_must_cast_data=.false.
 wfr_in_mem=.true.
 if (.not.(present(wfr))) then
  wfr_in_mem=.false.
  write(message,'(a)')' reading wavefunctions from file '
  call wrtout(06,message,'COLL')
  if (mpi_enreg%nproc==1 .or. localrdwf==1 .or. me==0) i_can_read=.true.
  if (mpi_enreg%nproc>1 .and. localrdwf==0) master_must_cast_data=.true.
  allocate(wfnk(nr,nbnds),stat=istat) 
   if(istat/=0) call memerr('density','wfnk',nr*nbnds,'spc')
 end if

 allocate(rho(nr),stat=istat)
  if(istat/=0) stop 'out of memory'
 ! rho2 is the output density while rho is the temporary array for symmetrization
 rho2(:,:)= zero

 do is=1,nsppol
  rho(:)=zero
  !
  ! Loop over k-points in IBZ 
  ! FIXME we are reading all the bands, it is a waste we can calculate 
  ! the maximum band index for this point and then read only the portion of the record we need
  !
  do ik=1,nkibz
   !
   ! Here, if required, we read wfr from file
   !
   if (.not.(wfr_in_mem)) then
    if (i_can_read) read(unitwfr,rec=ik+nkibz*(is-1)) ((wfnk(ir,ib),ir=1,nr),ib=1,nbnds)
    if (master_must_cast_data) call xcast_mpi(wfnk,master,spaceComm,ierr)
   end if
   !
   ! Loop over bands
   !
   do ib=1,nbnds
    !MG FIXME parallelism has been temporarily disabled
    ! in the new method (that should become the default one) each proc has all the valence states
    !    if (mpi_enreg%nproc>1 .and. mpi_enreg%gwpara==2) then
    !     if (minval(abs(mpi_enreg%proc_distrb(ib,:,:)-mpi_enreg%me))/=0) cycle
    !    end if
    if (abs(occ(ik,ib,is))<tol8) cycle
    !
    do ir=1,nr
     if (.not.(wfr_in_mem)) then
      rho(ir)=rho(ir)+occ(ik,ib,is)*conjg(wfnk(ir,ib))*wfnk(ir,ib)*wtk(ik)
     else
      rho(ir)=rho(ir)+occ(ik,ib,is)*conjg(wfr(ir,ib,ik,is))*wfr(ir,ib,ik,is)*wtk(ik)
     end if
    end do !ir
    !
   end do !ib
  end do !ik
  !
  ! Loop over symmetry operations, symmetrising rho, factor 2 is for inversion
  !
  fact=real(ninv)/(nkbz*ucvol)
  do ir=1,nr
   do iop=1,nop
    rho2(ir,is)=rho2(ir,is)+fact*rho(irottb(ir,iop))
   end do 
  end do
 end do !is

 ! Store the total charge in the first half only if nsspol==2 
 if (nsppol==2) then 
  rho(:)=rho2(:,1)
  rho2(:,1)=rho2(:,1)+rho2(:,2)
  rho2(:,2)=rho(:)
 end if 

 !MG FIXME parallelism has been temporarily disabled
 ! in the new method each proc has all the valence states
 ! if (mpi_enreg%gwpara==2) call xsum_mpi(rho2,spaceComm,ierr)

 deallocate(rho)
 if (allocated(wfnk)) deallocate(wfnk)

 ! Calculate total number of electrons as a check
 tnepuc=0.0
 do ir=1,nr
  tnepuc=tnepuc+rho2(ir,1)
 end do
 tnepuc=tnepuc*ucvol/(ngfft1*ngfft2*ngfft3)

 write(message,'(a,f7.4)')' total number of electrons per unit cell = ',tnepuc
 call wrtout(6,message,'COLL') ; call wrtout(ab_out,message,'COLL')

 rhoav=tnepuc/ucvol
 write(message,'(a,f7.4)')' average of density, n = ',rhoav
 call wrtout(6,message,'COLL') ; call wrtout(ab_out,message,'COLL')

 rs=(3.0/(4.0*(4.0*atan(1.0))*rhoav))**(1.0/3.0)
 write(message,'(a,f7.4)')' r_s = ',rs
 call wrtout(6,message,'COLL') ; call wrtout(ab_out,message,'COLL')

 omegaplasma=sqrt(4.0*(4.0*atan(1.0))*rhoav)
 write(message,'(a,f7.4,2a)')' omega_plasma = ',omegaplasma*Ha_eV,' [eV]',ch10
 call wrtout(6,message,'COLL') ; call wrtout(ab_out,message,'COLL')

 ! Write total charge as a check
 DEBUG=.false.
 if (DEBUG .and. mpi_enreg%me==0) then
  filnam='density.dat'
  call isfile(filnam,'new')
  open(unit=873,file=filnam)
  do ir=1,nr
   write(873,'(2x,f22.15)') (rho2(ir,is),is=1,nsppol)
  end do
  close(873)
 end if 

end subroutine density
!!***
