!{\src2tex{textfont=tt}}
!!****f* ABINIT/fermi
!! NAME
!! fermi
!!
!! FUNCTION
!! Calculate the Fermi level and occupation numbers
!! (This routine should be cleaned !!)
!!
!! COPYRIGHT
!! Copyright (C) 1999-2007 ABINIT group (GMR, VO, LR, RWG, MG, RShaltaf)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!  en(nk,nb,ns)=KS energies, for different k points, bands and spins
!!  fixmom=if differ from -99.99d0, fix the magnetic moment (in Bohr magneton)
!!  hdr=header of previously read file, containing many variables
!!  nb=number of bands
!!  nk=number of k points
!!  ns=number of spins (should be changed to nsppol)
!!  wtk(nk)=weights for k points (input variable)
!!  occ(nk,nb,ns)= occupations numbers for each k-point, band and spin
!!
!! OUTPUT
!!  nbv(ns)=number of valence bands (for each spin)
!!  fermie=Fermi energy
!!
!! SIDE EFFECTS
!!  nel= at input, if nel==0, recalculate nelect from occupation numbers
!!  occ(nk,nb,ns)=occupation numbers, for each k point, each band and spin
!!
!! PARENTS
!!      screening,sigma
!!
!! CHILDREN
!!      newocc
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine fermi(hdr,nb,nk,fixmom,ns,wtk,en,occ,nel,nbv,fermie)

 use defs_basis
 use defs_datatypes

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01manage_mpi
 use interfaces_14occeig
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: nb,nk,ns
 integer,intent(inout) :: nel
 real(dp),intent(in) :: fixmom
 real(dp),intent(out) :: fermie
 type(hdr_type),intent(in) :: hdr
!arrays
 integer,intent(out) :: nbv(ns)
 real(dp),intent(in) :: en(nk,nb,ns),wtk(nk)
 real(dp),intent(inout) :: occ(nk,nb,ns)

!Local variables-------------------------------
!scalars
 integer :: ib,ier,ik,index,is
 real(dp) :: cbot,cnorm,diff,entropy,nelect,vtop
 real(dp),parameter :: eps5=5*1.d-2
 character(len=500) :: message
!arrays
 integer,allocatable :: nbandt(:) 
 real(dp) :: condbottom(ns)
 real(dp) :: n(ns),valencetop(ns),wtk_norm(nk)
 real(dp),allocatable :: eigent(:),occt(:),doccdet(:)

! *************************************************************************

 cnorm=1.0/sum(wtk(:))
 wtk_norm(:)=wtk(:)*cnorm

 ! Calculate the number of electrons for each channel
 n(:)=zero
 do ik=1,nk
  do ib=1,nb
   do is=1,ns
    n(is)= n(is)+wtk_norm(ik)*occ(ik,ib,is)
   end do
  end do
 end do

 if (nel==0) then 
  ! The following is to avoid problem in newocc, 
  ! if fermi is (unlikely) called with nel==0 
  nel=nint(sum(n))
  nelect=nel
  write(message,'(2a,i5)')ch10,&
&  ' fermi : total number of electrons = ',nel
  call wrtout(6,message,'COLL')
 else
  nelect=nel
  write(message,'(2a,i5)')ch10,&
&  ' fermi : total number of electrons = ',nel
  call wrtout(6,message,'COLL')
  !
  ! Check the input value nel with the calculated one.
  ! Warn if the difference is small, stop if too large 
  !
  diff=(nelect-sum(n))
  write(message,'(2a,f12.8)')ch10,&
&  ' fermi : input and calculated number of electrons differ by ',diff 
  call wrtout(6,message,'COLL')
  if (abs(diff)>eps5*nelect) then 
   write(message,'(4a)')ch10,&
&  ' fermi : ERROR - ',ch10,&
&  '  Found too large difference in the number of electrons'
   call wrtout(6,message,'COLL') ; call leave_new('COLL')
  end if
 end if  
 !
 if (ns==2) then 
  write(message,'(2a,5x,2(f8.4,2x))')&
& ' number of electrons for each spin channel = ',ch10,(n(is),is=1,ns)
  call wrtout(6,message,'COLL')
 end if

 if (ns==1) then 
  nbv=nel/2
 else 
  ! Take into account the case in which there are two bands with 
  ! the same spin index close to the gap
  nbv=nint(n)
  write(message,'(a,2i8)')&
&  ' maximum occupied band index for spin up and down= ',nbv  
  call wrtout(6,message,'COLL')
 end if

 if (hdr%occopt<3 .or. hdr%occopt>7) then
  !
  ! Semiconductor or Insulator
  ! Calculate valence index for each channel
  !
  do is=1,ns
   valencetop(is)= -huge(0.0_dp)
   condbottom(is)=  huge(0.0_dp)
   do ik=1,nk
    do ib=1,nbv(is)
     if (valencetop(is)<en(ik,ib,is)) valencetop(is)= en(ik,ib,is)
    end do
    do ib=nbv(is)+1,nb
     if (condbottom(is)>en(ik,ib,is)) condbottom(is)= en(ik,ib,is)
    end do
   end do !ib
  end do !ik
  !
  vtop= maxval(valencetop)
  cbot= minval(condbottom)
  !
  write(message,'(a,f6.2,2a,f6.2)')&
&  ' top of valence       [eV] ',vtop*Ha_eV,ch10,&
&  ' bottom of conduction [eV] ',cbot*Ha_eV
  call wrtout(6,message,'COLL')
  if (ns==2) then 
   if (abs(vtop-minval(valencetop))>tol6) then 
    write(message,'(a,i2)')' top of valence is spin ',maxloc(valencetop)
    call wrtout(6,message,'COLL')
   end if
   if (abs(cbot-maxval(condbottom))>tol6) then 
    write(message,'(a,i2)')' bottom of conduction is spin ',minloc(condbottom)
    call wrtout(6,message,'COLL')
   end if
  end if
  !
  fermie=(vtop+cbot)/2 
  if (abs(cbot-vtop)<1.d-4) fermie= vtop !to avoid error on the last digit
 else 
  !
  ! Metallic system
  ! Re-calculate occupations factors
  !
  allocate(eigent(nb*nk*ns),nbandt(nk*ns))
  eigent(:)=zero
  nbandt(:)=nb
  index=0
  do is=1,ns
   do ik=1,nk
    do ib=1,nb
     index= index+1
     eigent(index)= en(ik,ib,is)
    end do
   end do
  end do
  !
  write(message,'(a,f9.5)')' fermi : calling newocc with fixmom = ',fixmom
  call wrtout(6,message,'COLL')
  allocate(occt(nb*nk*ns),doccdet(nb*nk*ns))
  call newocc(doccdet,eigent,entropy,fermie,fixmom,nb,nbandt,nelect,nk,&
&  hdr%nspinor,ns,occt,hdr%occopt,1,hdr%stmbias,hdr%tphysel,hdr%tsmear,wtk_norm)
  !
  index=0
  do is=1,ns
   do ik=1,nk
    do ib=1,nb
     index= index+1
     if (abs(occt(index)-occ(ik,ib,is))>1.d-2) then
      write(message,'(3a,i5,a,i5,a,i3,2a,f6.3)')&
&      ' fermi: WARNING - ',ch10,&
&      '  Occupation number of band ',ib,' for kpt ',ik,' spin ',is,ch10,&
       '  is changed with respect to KSS file : ',occt(index)-occ(ik,ib,is)
      call wrtout(6,message,'COLL')
     end if
     occ(ik,ib,is)=occt(index)
    end do
   end do
  end do
  deallocate(eigent,nbandt,occt,doccdet)
 end if

 write(message,'(a,f6.2,a)')' Fermi energy         [eV] ',fermie*Ha_eV,ch10
 call wrtout(6,message,'COLL')
 
end subroutine fermi
!!***
