!{\src2tex{textfont=tt}}
!!****f* ABINIT/findggp
!! NAME
!! findggp
!!
!! FUNCTION
!!  Fing the independent (G,Gp) pairs that are sufficient to reconstruct 
!!   chi_o(q), \tilde\epsilon(q) or \tilde\epsilon^{-1}(q) 
!!  using symmetry properties
!!
!! COPYRIGHT
!!  Copyright (C) 2007 ABINIT group (MG)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!  qpt(3)=q-point 
!!  nop= number of symmetry operations
!!  npwc=number of G vectors in the matrix
!!  op(3,3,nop)= symmetry operations in reciprocal space
!!  gvec(3,npwc) = G vectors in reduced coordinates
!!  gmet(3,3)=metric tensor in reciprocal space
!!
!! OUTPUT
!!  ngpi=nuber of independent (G1,G1) pairs   
!!  gptab(npwc,npwc)= for given T1 and T1 reports the sequential index of the indendent pair (G1,G2) 
!!   such as (T1,T2)= S (G1,G2), see also ip2fp 
!!  gptabo(npwc,npwc)= index of the symmetry operation S in the array op such as (T1,T2)= S (G1,G2)
!!  ip2fp(2,npwc**2)= index of G1 and G2 in the array gvec for each ngpi independent pair. 
!!   Note that in input the second dimension of ip2fp is equal to npwc**2 but only the first ngpi element
!!   are used since ngpi is calculated inside this subroutine. 
!!   For this reason it is better to call findggp using a dummy array for ip2fp whose dimension is (2,npwc**2), 
!!   and then copy only the slice needed 
!!
!! SIDE EFFECTS
!!
!! NOTES
!!  This implementation could not work properly for particular KSS file generated with Abinit version < 5.3  
!!  In particular for KSS files reporting only the inversion in the set of symmetries operations
!!
!! TODO
!!  Write an interface for vnorm and dosym (also vdotw) in order to deal with integer vectors 
!!
!! PARENTS
!! 
!!
!! CHILDREN
!! 
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine findggp(qpt,nop,op,npwc,gvec,gmet,ngpi,gptab,gptabo,ip2fp)
    
 use defs_basis

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01manage_mpi
 use interfaces_12geometry
 use interfaces_13recipspace
 use interfaces_15gw, except_this_one => findggp
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: nop,npwc
 integer,intent(out) :: ngpi
!arrays
 integer,intent(in) :: gvec(3,npwc)
 integer,intent(out) :: gptab(npwc,npwc) 
 integer,intent(out) :: gptabo(npwc,npwc)
 integer,intent(out) :: ip2fp(2,npwc**2)
 real(dp),intent(in) :: qpt(3)
 real(dp),intent(in) :: op(3,3,nop),gmet(3,3)

!Local variables-------------------------------
!scalars
 integer :: ii,jj,iop,iinv,ig,ig1,ig2,igpi,timrev,i2f,iid,ic,istat
 real(dp) :: norm,norm1,norm2,eps,eps1,eps2
 character(len=500) :: message           
 logical :: DEBUG=.true.
 logical :: found,no_identity
!arrays
 integer :: identity(3,3)
 integer :: symqpt(4,2,nop),op_int(3,3,nop),ltg(nop)
 real(dp) :: gposs1(3),gposs2(3) 
 real(dp) :: gxx1(3),gxx2(3)
 real(dp),allocatable :: gnorm(:),gnormp(:,:)
 real(dp),allocatable :: gpi(:,:)  !FIXME this could be integer

! *************************************************************************
!This routine is not used in the present v5.4.3 - still it seems to be very time-consuming
!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifndef HAVE_FORTRAN_INTERFACES
 real(dp) :: normv
#endif
!End of the abilint section

 return

 identity(:,:)=reshape((/1,0,0,0,1,0,0,0,1/),(/3,3/))

 write(message,'(2a,3f8.5,a)')ch10,&
& ' findggp : Analyzing symmetries at q point : ',qpt,ch10 
 call wrtout(6,message,'COLL')

 ! symq3 requires an integer argument while in the GW code operations are real
 op_int=nint(op)

 ! symq3 assumes that the identity operation is present
 ! but in the ABINIT < 5.3  we were removing symmetries related by the inversion 
 ! and sometimes it happened that only the inversion was reported in the KSS file (see outkss.F90) 
 ! in this case the code will stop
 no_identity=.true. 
 do iop=1,nop
  if (all(op_int(:,:,iop)==identity)) then 
   no_identity=.false.
   iid=iop
   exit
  end if
 end do 
 
 if (no_identity) then 
  write(message,'(4a)')&
&  ' findggp : COMMENT -',&
&  '  Only the inversion was found in the set of symmetries read from the KSS file ',ch10,&
&  '  Maybe old version of KSS file? '
  call wrtout(6,message,'COLL') ;call leave_new('COLL')
  !op_int=-op_int
  else 
  write(message,'(a,i4)')' findggp : found identity with index : ',iid
  call wrtout(6,message,'COLL')
 end if 

 ! Find operations in the little group, timrev is not used 
 call symq3(nop,qpt,symqpt,op_int,timrev)

 ltg(:)=0 

 ! Non-zero umklapp G0 vectors are not allowed 
 ! ltg contains the index of all the symmetries (time reversal excluded) that preserve q with G0==0
 do iop=1,nop
  if (symqpt(4,1,iop)==1 .and. all(symqpt(1:3,1,iop)==0)) then !no time reversal and G==0
   ltg(iop)=1
  end if
 end do !iop

 if (sum(ltg)==1) then 
  write(message,'(a)')' findggp : not enough symmetries to reduce the number of pair'
  call wrtout(06,message,'COLL')
  ngpi=npwc**2
  gptabo(:,:)=iid
  ic=1 
  do ig1=1,npwc
   do ig2=1,npwc
    gptab(ig1,ig2)=ic
    ip2fp(1,ic)=ig1
    ip2fp(2,ic)=ig2
    ic=ic+1
   end do
  end do 
  return
 end if 

 ! Precalculate G norm to speed the loop over G-pairs
 allocate(gnorm(npwc))
 do ig=1,npwc
  gnorm(ig)=normv(dble(gvec(:,ig)),gmet,'g')
 end do

 ! Warning the following arrays might be quite large,
 allocate(gnormp(2,npwc**2),stat=istat)
  if(istat/=0) call memerr('findggp','gnormp',npwc*2,'dp')
 gnormp(:,:)=zero
 allocate(gpi(6,npwc**2),stat=istat)
  if(istat/=0) call memerr('findggp','gpi',6*npwc*2,'dp')
 gpi=zero

 ! Zero no. of pairs found
 ngpi=0
 !
 ! Loop over all all possible pairs (G1,G2) 
 !
 do ig1=1,npwc
  do ig2=1,npwc
   gposs1(:)=dble(gvec(:,ig1))
   gposs2(:)=dble(gvec(:,ig2))
   norm1=gnorm(ig1)
   norm2=gnorm(ig2)
   eps1=1.d-8*norm1
   eps2=1.d-8*norm2
   !
   ! Check whether this pair (G1,G2) is the image through the same operation of a pair already found 
   ! Consider only vectors with the same lenght
   !
   found=.false.
ip: do igpi=ngpi,1,-1
    !
    if (abs(gnormp(1,igpi)-norm1)>eps1) cycle
    if (abs(gnormp(2,igpi)-norm2)>eps2) cycle
    !
    do iop=1,nop
     ! Only operations in the little group such as Sq=q
     ! S G1 = T1 and  S G2= T2
     ! TODO Calculate SG outside the loop, requires more memory but should be faster!
     ! avoid check on ltg, could make a table full==> little_group
     if (ltg(iop)==1) then 
      call dosym(op(1,1,iop),1,gpi(1:3,igpi),gxx1)
      call dosym(op(1,1,iop),1,gpi(4:6,igpi),gxx2)
      if( all(abs(gxx1-gposs1)<1.0e-3) .and.  &
&         all(abs(gxx2-gposs2)<1.0e-3) )  then 
       found=.true.
       gptab(ig1,ig2)=igpi 
       gptabo(ig1,ig2)=iop
       exit ip
      end if 
     end if 
    end do !iop 
   end do ip 
   !
   if (.not.found) then
    ngpi=ngpi+1
    gpi(1:3,ngpi)=gposs1(:)
    gpi(4:6,ngpi)=gposs2(:) 
    gnormp(1,ngpi)=norm1
    gnormp(2,ngpi)=norm2
    ip2fp(1,ngpi)=ig1
    ip2fp(2,ngpi)=ig2
    gptab(ig1,ig2)=ngpi
    gptabo(ig1,ig2)=iid  !this irreducible pair comes from the identity!
   end if
   !
  end do !ig2
 end do !ig1
 
 deallocate(gnorm,gnormp,gpi)

 if (ngpi>npwc**2) then 
  write(message,'(3a)')ch10,' findggp : BUG - ngpi>npwc**2 ',ch10
  call wrtout(6,message,'COLL') ; call leave_new('COLL')
 end if 

 write(message,'(2a,i8,a,i8,a)')ch10,&
& ' findggp : number of independent (G,G'') pairs found = ',ngpi,' / ',npwc**2,ch10
 call wrtout(6,message,'COLL')

!DEBUG 
!For the time being do not comment this part
 DEBUG=.false.
 if (DEBUG) then
  !open(file='Gpairs.dat',unit=88)
  !write(88,*)'checking independent pairs at',qpt
  !do iop=1,nop
  ! write(88,'(i3,a,9i3)')iop,')',((nint(op(ii,jj,iop)),jj=1,3),ii=1,3)
  !end do 
  do ig1=1,npwc
   do ig2=1,npwc
    i2f=gptab(ig1,ig2)
    gposs1=gvec(:,ip2fp(1,i2f))
    gposs2=gvec(:,ip2fp(2,i2f))
    iop=gptabo(ig1,ig2)
    !write(88,'(2(i4,a,3i4),a,6i4,a,i3)')&
    !&    ig1,')',gvec(:,ig1),ig2,')',gvec(:,ig2),'==>',gvec(:,ip2fp(1,i2f)),gvec(:,ip2fp(2,i2f)),'sym',iop
    call dosym(op(1,1,iop),1,gposs1,gxx1)
    call dosym(op(1,1,iop),1,gposs2,gxx2)
    if ( any(abs(gvec(:,ig1)-gxx1(:))>1.0d-3) .or. &
&        any(abs(gvec(:,ig2)-gxx2(:))>1.0d-3) ) then 
     write(message,'(3a,2i8,a)')' findggp : BUG -',ch10,&
&    ' G-G'' pair',ig1,ig2,' has no corresponding pair' 
     call wrtout(6,message,'COLL')
     write(*,*)'G-G''',gvec(:,ig1),gvec(:,ig2)
     write(*,*)'independent pair',gposs1(:),gposs2(:)
     write(*,*)'operation',iop,op(:,:,iop)
     call leave_new('COLL')
    end if 
   end do
  end do 
  !close(88)
 end if 
!ENDDEBUG

end subroutine findggp
!!***
