!{\src2tex{textfont=tt}}
!!****f* ABINIT/findq
!! NAME
!! findq
!!
!! FUNCTION
!! Identify the q-points by which the k-points in BZ differ
!!
!! COPYRIGHT
!! Copyright (C) 1999-2007 ABINIT group (GMR, VO, LR, RWG, MT, MG)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!  b1(3),b2(3),b3(3)=the three primitive vectors in reciprocal space
!!  kbz(3,nkbz)=coordinates of k points in BZ
!!  ninv=1 if inversion is not to be taken into account; 2 if inversion is to be taken into account
!!  nkbz=number of k points in Brillouin zone
!!  nop=number of symmetry operations
!!  nq=number of q points by which k points differ (computed in findnq)
!!  op(3,3,nop)=symmetry operations in reciprocal space
!!
!! OUTPUT
!!  q(3,nq)=coordinates of q points by which k points differ
!!
!! PARENTS
!!      mrgscr,screening
!!
!! CHILDREN
!!      bz1,dosym,leave_new,wrtout
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine findq(nkbz,kbz,nop,op,nq,q,ninv,b1,b2,b3)

 use defs_basis

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01manage_mpi
 use interfaces_15gw, except_this_one => findq
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: ninv,nkbz,nop,nq
!arrays
 real(dp),intent(in) :: b1(3),b2(3),b3(3),kbz(3,nkbz),op(3,3,nop)
 real(dp),intent(out) :: q(3,nq)

!Local variables ------------------------------
!scalars
 integer :: ii,i,iinv,ik,iop,iq,iqp,j
 real(dp) :: shift
 logical :: found
 character(len=500) :: message
!arrays
 integer :: gtemp(3)
 real(dp) :: qposs(3),qrot(3)

! *************************************************************************
!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifndef HAVE_FORTRAN_INTERFACES
 integer :: icmpk2
 integer :: itst0
#endif
!End of the abilint section

 write(message,'(a)')&
& ' find q-points q = k - k1 and translate in first BZ'
 call wrtout(6,message,'COLL') ; call wrtout(ab_out,message,'COLL')

 ! Zero no. of q-points found
 iq=0
 !
 ! Loop over all k-points in BZ, forming k-k1
 !
 do ik=1,nkbz
  qposs(1)=kbz(1,ik)-kbz(1,1)
  qposs(2)=kbz(2,ik)-kbz(2,1)
  qposs(3)=kbz(3,ik)-kbz(3,1)
  ! Check whether this q (or its equivalent) has already been found
  found=.false.
  do iqp=1,iq
   do iinv=1,ninv
    do iop=1,nop
     call dosym(op(1,1,iop),iinv,q(1,iqp),qrot)
     if(icmpk2(qrot,qposs)==1) found=.true.
    end do
   end do
  end do
  if(.not.found) then
   iq=iq+1
   if(iq>nq) then 
    write(message,'(3a)')ch10,&
&    ' findq : BUG - iq>nq ',ch10
    call wrtout(6,message,'COLL') ; call leave_new('COLL')
   end if 
   q(1,iq)=qposs(1)
   q(2,iq)=qposs(2)
   q(3,iq)=qposs(3)
  end if
 end do !ik

 if(iq/=nq) then 
  write(message,'(3a)')ch10,' findq : BUG - iq /=nq ',ch10
  call wrtout(6,message,'COLL') ; call leave_new('COLL')
 end if 

 ! Translate q-points (calculated for info) to 1st BZ
 !
 ! MG it seems that bz1 sometimes does not work, 
 ! in SiO2 for example I got 0.333333   -0.666667    0.000000
 ! it is better to use canon9 also because if an irred qpoint 
 ! lies outside the 1BZ then most probably we have to consider
 ! an umklapp G vector to reconstruct the full BZ and the 
 ! correct treatment of this case is not implemented yet, see csigme.F90
 ! anyway I should check this new method because likely mrscr will complain
 ! the best idea consists in  writing a new subroutine canon10 
 ! wich reduce the q point in the interval [-1/2,1/2[ which was the scope of bz1
 ! only to obtain the same results as the automatic tests
 ! FIXME for the moment use old version, easy for debugging
 ! OLD CODING 
 do iq=1,nq
  call bz1(q(:,iq),gtemp,b1,b2,b3)
 end do
 !END OLD CODING
 !NEW CODING 
 ! do iq=1,nq
 !  do ii=1,3
 !   call canon9(q(ii,iq),q(ii,iq),shift)
 !  end do
 ! end do 
 !NED OLD CODING

 write(message,'(a)')' q-points [reduced coordinates]'
 call wrtout(6,message,'COLL') ; call wrtout(ab_out,message,'COLL')
 do j=1,nq
  if(itst0(q(:,j))==1) then
   q(1,j)=0.000010
   q(2,j)=0.000020
   q(3,j)=0.000030
  end if
  write(message,'(3f12.6)') (q(i,j),i=1,3)
  call wrtout(06,message,'COLL') ; call wrtout(ab_out,message,'COLL')
 end do
 
 !write(message,'(a)')ch10
 !call wrtout(06,message,'COLL') ; call wrtout(ab_out,message,'COLL')
 write(06,*) ; write(ab_out,*)

end subroutine findq
!!***
