!{\src2tex{textfont=tt}}
!!****f* ABINIT/initmpi_gw
!! NAME
!! initmpi_gw
!!
!! FUNCTION
!! Initialize mpi information for GW calculations
!!
!! COPYRIGHT
!!  Copyright (C) 2007 ABINIT group (MG, RS)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!  ibocc(dtset%nsppol)= for each spin this array defines the band index after which the 
!!   occupations numbers are smaller that tol8 (required in case of gwpara==2)
!!  nbcw=  
!!  nbcw=  
!!  dtset : dataset datatype, variables used 
!!  |mkmem   0  write wfs on an external file
!!           1  wavefunctions are in memory
!!  nbnds=number of bands considered in the GW calculation 
!!  nkibz=number of irreducible k-points
!!  nsppol=number of spin
!!
!!
!! OUTPUT
!!  FIXME should depend on spin
!!  Only in case of gwpara==2 
!!  min_band_proc,max_band_proc=min and max conduction band index for this processor 
!!  nbnds_per_proc=number of conduction bands for this processor
!!
!! SIDE EFFECTS
!!  mpi_enreg=informations about MPI parallelization
!!   mpi_enreg%proc_distrb(nkpt,mband,nsppol)  array that  describes the work of each processor 
!!   allocated and defined here only in case of parallelism over bands
!!   in case of parallelism over k-points mpi_enreg%proc_distrb is redefined on the fly for each q-point  
!!
!! NOTES
!!
!! TODO
!! This is only a first coding
!!
!! PARENTS
!! 
!!
!! CHILDREN
!!  
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine initmpi_gw(mpi_enreg,dtset,nkibz,nbnds,nsppol,ibocc,nbvw,nbcw,&
& min_band_proc,max_band_proc,nbnds_per_proc)
    
 use defs_basis
 use defs_datatypes

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01manage_mpi
#endif
!End of the abilint section

 implicit none

#if defined MPI 
           include 'mpif.h'
#endif

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: nbnds,nsppol,nkibz
 integer,intent(in) :: nbcw,nbvw
 integer,intent(out) :: min_band_proc,max_band_proc,nbnds_per_proc
 type(dataset_type),intent(in) :: dtset
 type(MPI_type),intent(inout) :: mpi_enreg
!arrays 
 integer,intent(in) :: ibocc(nsppol)

!Local variables-------------------------------
!scalars
 integer :: ib,iproc,ierr,ind,is,nbsteps,nbw
 character(len=500) :: message       
!arrays
 integer,allocatable :: nband_t(:)
 
! *************************************************************************

!DEBUG
!write(message,'(a)')' enter initmpi_gw'
!call wrtout(6,message,'PERS')
!ENDDEBUG

 mpi_enreg%parareel=0
 mpi_enreg%paralbd=0
 mpi_enreg%paral_level=2   !dont understand, this means k-points?

 min_band_proc=0 
 max_band_proc=0

 if (mpi_enreg%gwpara==0) then 
  ! Sequential run
  write(message,'(2a)')ch10,&
&  ' initmpi_gw : sequential run'
  call wrtout(06,message,'COLL')
  min_band_proc=1 ; max_band_proc=nbnds
  nbnds_per_proc=nbnds
  ! This is to avoid problems in statements like : if (gwpara==2 .and. mpi_enreg%proc_distr)
  allocate(mpi_enreg%proc_distrb(nkibz,nbnds,nsppol))  
  mpi_enreg%proc_distrb=mpi_enreg%me
 else if (mpi_enreg%gwpara==1) then 
  ! Parallelization over k-points and spin, Memory is not yet parallelized for this option
  write(message,'(3a)')ch10,' initmpi_gw : loop on k-points and spin done in parallel',ch10
  call wrtout(06,message,'COLL')
  min_band_proc=1 ;  max_band_proc=nbnds
  nbnds_per_proc=nbnds
 else if (mpi_enreg%gwpara==2) then 
  ! Parallelization over bands and spin
  write(message,'(2a)')ch10,&
&  ' initmpi_gw : loop over bands done in parallel'
  call wrtout(06,message,'COLL')
  if (dtset%mkmem==0) then 
   write(*,*)'not implemented yet'
   call leave_new('COLL')
  end if 
  if (dtset%optdriver==3) then 
   ! In a screening calculation we have a double loop over bands 
   ! Fully and partially occupied states are allocated by each proc while
   ! unoccupied states are spreaded (this is possible if time reversal can be used) 
   nbw=nbcw+nbvw
   if (nbw/=nbnds) then 
    write(*,*) 'ERROR nbw/=nbnds'
    call leave_new('COLL')
   end if 
   allocate(mpi_enreg%proc_distrb(nkibz,nbw,nsppol))  
   mpi_enreg%proc_distrb=-999
   nbsteps=nbcw/mpi_enreg%nproc   
   if (mod(nbcw,mpi_enreg%nproc)/=0) then 
    nbsteps=nbsteps+1
    write(message,'(7a)')ch10,&
&    ' initmpi_gw : WARNING -',ch10,&
&    '  number of unoccupied states is not divisible by the number of processors',ch10,&
&    '  parallelization is not efficient',ch10
    call wrtout(6,message,'COLL')
   end if 
   ind=0
   do ib=nbvw+1,nbw
    mpi_enreg%proc_distrb(:,ib,:)=ind/nbsteps
    ind= ind + 1
   end do
   ! Find min and max conduction band at each spin for each proc 
   do ib=nbvw+1,nbw
    if (mpi_enreg%proc_distrb(1,ib,1)==mpi_enreg%me) then 
     min_band_proc=ib
     exit
    end if 
   end do 
   do ib=nbw,1,-1
    if (mpi_enreg%proc_distrb(1,ib,1)==mpi_enreg%me) then 
     max_band_proc=ib
     exit
    end if 
   end do 
   nbnds_per_proc=max_band_proc-min_band_proc+1
   ! Each proc has fully and partially occupied states 
   do ib=1,nbvw
    mpi_enreg%proc_distrb(:,ib,:)=mpi_enreg%me
   end do
  else if (dtset%optdriver==4) then 
   ! In sigma we have a single loop over bands, so there is no 
   ! advantage in dividing the states in valence and conduction
   allocate(mpi_enreg%proc_distrb(nkibz,nbnds,nsppol))  
   mpi_enreg%proc_distrb=-999
   nbsteps=nbnds/mpi_enreg%nproc   
   if (mod(nbnds,mpi_enreg%nproc)/=0) then 
    nbsteps=nbsteps+1
    write(message,'(7a)')ch10,&
&    ' initmpi_gw : WARNING -',ch10,&
&    '  number of bands is not divisible by the number of processors',ch10,&
&    '  parallelization is not efficient',ch10
    call wrtout(6,message,'COLL')
   end if 
   ind=0
   do ib=1,nbnds
    mpi_enreg%proc_distrb(:,ib,:)= ind/nbsteps
    ind= ind + 1
   end do
   ! Now find min and max conduction band at each spin for each proc 
   do ib=1,nbnds
    if (mpi_enreg%proc_distrb(1,ib,1)==mpi_enreg%me) then 
     min_band_proc= ib
     exit
    end if 
   end do 
   do ib=nbnds,1,-1
    if (mpi_enreg%proc_distrb(1,ib,1)==mpi_enreg%me) then 
     max_band_proc= ib
     exit
    end if 
   end do 
   nbnds_per_proc=max_band_proc-min_band_proc+1
  else 
   write(message,'(4a,i8)')ch10,&
&   ' initmpi_gw : BUG - ',ch10,&
&   '  called with wrong value of optdriver = ',dtset%optdriver
    call wrtout(6,message,'COLL') ; call leave_new('COLL')
  end if 
   ! Announce the treatment of bands by each proc
   do iproc=0,mpi_enreg%nproc-1
    if (mpi_enreg%me==iproc) then 
     write(message,'(a,i4,a,i4,a,i4,a,i3)')&
&    'P initmpi_gw : treating ',nbnds_per_proc,' bands from ',min_band_proc,&
&    ' up to ',max_band_proc,' by node ',iproc
     call wrtout(6,message,'PERS')
    end if 
   end do 
 else 
  write(message,'(4a,i4)')ch10,' initmpi_gw : BUG- ',ch10,&
&  ' called with wrong value for gwpara = ',mpi_enreg%gwpara
  call wrtout(6,message,'COLL') ; call leave_new('COLL')
 end if
 if (min_band_proc==0 .or. max_band_proc==0) then 
  write(message,'(6a,i6,a,i6,3a)')ch10,&
&  ' initmpi_gw : ERROR - ',ch10,&
&  '  one or more processors has zero number of bands ',ch10,&
&  '  min_band_proc = ',min_band_proc,' max_band_proc = ',max_band_proc,ch10,&
&  '  try to decrease the number of processors ',ch10
  call wrtout(06,message,'PERS') ; call leave_new('COLL')
 end if 

!DEBUG !for the time being do not comment
! if (any(mpi_enreg%proc_distrb(:,:,:)==-999)) then 
!  write(message,'(a)')' initmpi_gw : BUG in distribution'
!  call wrtout(06,message,'PERS') ; call leave_new('COLL')
! end if 
! write(message,'(a)')' exit initmpi_gw'
! call wrtout(06,message,'PERS') 
! call leave_test(mpi_enreg) 
!ENDDEBUG

end subroutine initmpi_gw
!!***
