!{\src2tex{textfont=tt}}
!!****f* ABINIT/rdkss
!! NAME
!! rdkss
!!
!! FUNCTION
!!  read a _KSS file
!!
!! COPYRIGHT
!! Copyright (C) 1999-2007 ABINIT group (GMR, VO, LR, RWG, MM, XG, MG, RShaltaf)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!  dtfil=filenames (here, only dtfil%filkss is used)
!!  mpsangx= 1+maximum angular momentum for nonlocal pseudopotential
!!  natomx=maximum number of atoms
!!  nbx=maximum number of bands
!!  ngx=maximum number of G vectors
!!  nkx=maximum number of k points
!!  nopx=maximum number of symmetry operations
!!  nsx=1 for unpolarized, 2 for spin-polarized
!!  ntypatx=maximum number of types of atoms
!!  nwx=maximum number of planewaves
!!  localrdwf= input variable (for parallel case) if 1, the KSS file is local to each machine
!!  min_band_proc,max_band_proc=index of the min and max band that will be kept in memory by this proc
!!  nbvw= maximum number of fully and partially occupied states over spin  (used for the new algorithm)
!!  prtvol=input variable defining the verbosity
!!
!! OUTPUT
!!  a1(3),a2(3),a3(3)=real-space lattice vectors in au
!!  en(nkx,nbx,nsx)=KS energies for each k point and each band
!!  gvec(3,ngx)=integer coordinates of G vectors
!!  hdr=header of the KSS file
!!  kibz(3,nkx)=coordinates of k points in IBZ
!!  mpi_enreg=informations about MPI parallelization
!!  nel=number of electrons
!!  oc(nkx,nbx,nsx)=occupation numbers for each k point, each band and each spin
!!  op(3,3,nop)=symmetry operations in reciprocal space
!!  tit(2)(len=80)= titles in the KSS file
!!  typat(natomx)=types of atoms
!!  tnons(3,nopx)=fractional translations
!!  vkb(nwx,ntypatx,mpsangx,nkx)=KB projector function
!!  vkbd(nwx,ntypatx,mpsangx,nkx)=derivative of the KB projector function in reciprocal space
!!  vkbsign(mpsangx,ntypatx)=sign of each KB dyadic product
!!  wf(nwx,min_band_proc:max_band_proc,nkx,nsx)=wavefunctions in G space, for each band, each k point and spin
!!  xred(3,natomx)=coordinates of atoms in real space
!!  wf_val(nwx,nbvw,nkx,nsx)= (OPTIONAL) array containing all the states whose occupation is less than tol8 
!!                             used only if called from screening
!! 
!! NOTES 
!! For historical reasons symmetry operations and fractional translations are not read from the 
!! hdr but directly from the KSS file. This behavior should be changed but most of the automatic 
!! tests should be updated
!!
!! PARENTS
!!      screening,sigma
!!
!! CHILDREN
!!      hdr_io,hdr_io_netcdf,leave_new,pclock,wrtout,xcast_mpi,xcomm_init
!!      xmaster_init,xmax_mpi,xme_init,xmin_mpi_dpv
!!
!! SOURCE
#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine rdkss(dtfil,hdr,nopx,nbx,nbvw,nkx,ngx,nsx,nwx,tit,a1,a2,a3,op,&
& gvec,kibz,en,oc,wf,ntypatx,natomx,mpsangx,prtvol,typat,tnons,xred,vkbsign,vkb,vkbd,nel,&
& mpi_enreg,localrdwf,min_band_proc,max_band_proc,&
& wf_val) !optional arguments 

 use defs_basis
 use defs_datatypes

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01manage_mpi
 use interfaces_11util
 use interfaces_13io_mpi
 use interfaces_lib01hidempi
#else
 use defs_interfaces
 use defs_xfuncmpi
#endif
!End of the abilint section

 implicit none

#if defined MPI || defined MPI_FFT
    include 'mpif.h'
#endif
    
!Arguments ------------------------------------
!scalars
 integer,intent(in) :: mpsangx,natomx,nbx,ngx,nkx,nopx,nsx,ntypatx,nwx,min_band_proc,max_band_proc
 integer,intent(in):: localrdwf   
 integer,intent(in)  :: nbvw,prtvol
 integer,intent(out) :: nel
 type(datafiles_type),intent(in) :: dtfil
 type(hdr_type),intent(out) :: hdr
 type(MPI_type),intent(in) :: mpi_enreg
!arrays
 integer,intent(out) :: gvec(3,ngx),typat(natomx)
 real(dp),intent(out) :: a1(3),a2(3),a3(3),en(nkx,nbx,nsx),kibz(3,nkx)
 real(dp),intent(out) :: oc(nkx,nbx,nsx)
 real(dp),intent(out) :: op(3,3,nopx),tnons(3,nopx),vkb(nwx,ntypatx,mpsangx,nkx)
 real(dp),intent(out) :: vkbd(nwx,ntypatx,mpsangx,nkx),vkbsign(mpsangx,ntypatx)
 real(dp),intent(out) :: xred(3,natomx)
 complex,intent(out) :: wf(nwx,min_band_proc:max_band_proc,nkx,nsx)
 complex,intent(out),optional :: wf_val(nwx,nbvw,nkx,nsx)
 character(len=80),intent(out) :: tit(2)
 
!Local variables-------------------------------
!scalars

 integer,parameter :: unitkss=23
 integer :: day,enough,fform,ii,i,ia,ib,ibp,ig,ik,il,ipa,is,ish,isym,isymend,isymin,it,isppol,ios
 integer :: itypat,j,jj,month,mpsang,natom,nbandkss,nela,nkpt,npwkss,nsh,nsym2,nsppol
 integer :: ntypat,osl,rdwr,readnetcdf,year,ierr
 integer :: spaceComm,me,master
 real(dp) :: cinf,csup,dum,einf,esup,cinf1,csup1,einf1,esup1
 complex :: cdum
 logical,parameter :: DEBUG=.false.
 logical :: i_can_read,master_cast,use_tr
 character(len=10) :: stag(2)=(/' UP   ',' DOWN '/)
 character(len=500) :: message
!arrays
 integer,allocatable :: shlim(:),symrel2(:,:,:),symrec2(:,:,:)
 real(dp),allocatable :: energyd(:),vkbdb(:,:,:),vkbdd(:,:,:)
 real(dp),allocatable :: tnons2(:,:),vkbsignd(:,:)
 real(dp),allocatable :: temp_en(:,:)
 real,allocatable :: vkbsigns(:,:)
 real,allocatable :: vkbds(:,:,:),vkbs(:,:,:)
 real,allocatable :: energys(:)
 complex,allocatable :: wfgs(:)
 complex,allocatable :: temp_wf(:,:,:)
 complex(dpc),allocatable :: wfgd(:)

! *************************************************************************

!DEBUG
!write(6,*)' rdkss : enter '
!ENDDEBUG

 ! Initialize MPI related quantities
 call xcomm_init(mpi_enreg,spaceComm) ! Init mpi_comm
 call xme_init(mpi_enreg,me)          ! Init me
 call xmaster_init(mpi_enreg,master)  ! Init master

 write(message,'(3a)')ch10,&
& ' rdkss : reading Kohn-Sham structure file ',trim(dtfil%filkss)
 call wrtout(6,message,'COLL')

 use_tr=.false.
 if (present(wf_val)) then 
  use_tr=.true.
  write(message,'(5a)')ch10,&
&  ' rdkss : assuming time reversal symmetry ',ch10,&
&  '         occupied and unoccupied states are stored in different arrays ',ch10
  call wrtout(06,message,'COLL')
 end if 

 i_can_read=.false. ; master_cast=.false.
 if (localrdwf==1 .or. me==0) i_can_read=.true.
 if (localrdwf==0 .and. mpi_enreg%nproc>1) master_cast=.true.
 
 if (i_can_read) then
  open(unit=unitkss,file=dtfil%filkss,form='unformatted',status='old',iostat=ios)
  if (ios/=0) then 
   write(message,'(6a)')ch10,&
&   ' rdkss: ERROR- ',ch10,  &
&   '  opening file: ',trim(dtfil%filkss),' as old'
   call wrtout(6,message,'COLL') ; call leave_new('COLL')
  end if
  ! Read the header of the GS wavefunction file
  rdwr=1
  readnetcdf=0 ! should become input parameter and make KSS a netcdf file as well
  if (readnetcdf==0) call hdr_io(fform,hdr,rdwr,unitkss)
  if (readnetcdf==1) call hdr_io_netcdf(fform,hdr,rdwr,unitkss)
  if (fform/=502 .and. fform/=602) then
   write(message,'(5a,i8)')ch10,&
&   ' rdkss : ERROR-  ',ch10,   &
&   trim(dtfil%filkss),' has fform/=502,602 ',fform
   call wrtout(6,message,'COLL') ; call leave_new('COLL')
  end if
  if (fform==502) write(message,'(2a)')' Kohn Sham Structure double precision file',ch10
  if (fform==602) write(message,'(2a)')' Kohn Sham Structure single precision file',ch10
  call wrtout(6,message,'COLL')
  ! Write the header of the GS wavefunction file
  rdwr=4
  call hdr_io(fform,hdr,rdwr,6)
  !
  natom=hdr%natom
  nsppol=hdr%nsppol
  ntypat=hdr%ntypat
  nkpt=hdr%nkpt
  a1(:)=hdr%rprimd(:,1)
  a2(:)=hdr%rprimd(:,2)
  a3(:)=hdr%rprimd(:,3)
  !
  if (ntypat/=ntypatx .or. natom/=natomx .or. nkpt/=nkx .or. nsppol/=nsx) then 
   write(message,'(3a)')ch10,&
&   ' rdkss : BUG - inconsistent data1',ch10
   call wrtout(06,message,'COLL')
   write(*,*) ntypat,ntypatx,natom,natomx,nkpt,nkx,nsppol,nsx 
   call leave_new('COLL')
  end if 
  typat(1:natomx)=hdr%typat(1:natomx)
  kibz(:,1:nkx)=hdr%kptns(:,1:nkx)
  !
  do isppol=1,nsppol
   do ik=1,nkpt
    do ib=1,hdr%nband(ik+nkpt*(isppol-1))
     !NOTE: hdr%nband(nkpt*nsppol)
     ! hdr%occ(bantot) where bantot is the total number of bands 
     ! (i.e sum of nband on all kpts and spins)
     ! in the GW part the number of bands must be the same for each k-point and spin
     i= (ik-1)*hdr%nband(ik) +(isppol-1)*nkpt*hdr%nband(ik) + ib 
     if (ib<=nbx) oc(ik,ib,isppol)=hdr%occ(i)
    end do
   end do
  end do 
  do itypat= 1,ntypat
   if (hdr%pspso(itypat)==2 .or. hdr%pspso(itypat)==3) then
    write(message,'(5a)')ch10,&
&    ' rdkss:  ERROR- ',ch10, &
&    '  pspso value not programmed',ch10
    write(*,*)' pspso = ', hdr%pspso(itypat)
    call wrtout(6,message,'COLL') ; call leave_new('COLL')
   end if
  end do
  !
  nel=0
  do ia=1,hdr%natom
   it=hdr%typat(ia)
   nela=hdr%znucltypat(it)  ! if it is not a pseudoatom, add all electron
   do ipa=1,hdr%npsp
    if (hdr%znuclpsp(ipa)==hdr%znucltypat(it)) nela=hdr%zionpsp(ipa)
   end do
   nel=nel + nela
  end do
  xred(:,1:natomx)=hdr%xred(:,1:natomx)
 end if !i_can_read
 !
 ! End of treatment of the header, in case of parallel execution transmit data
 !
 if (master_cast) then 
  write(message,'(2a)')ch10,&
&  ' rdkss : master is sending first set of data...'
  call wrtout(06,message,'COLL')
  call xcast_mpi(a1,master,spaceComm,ierr)
  call xcast_mpi(a2,master,spaceComm,ierr)
  call xcast_mpi(a3,master,spaceComm,ierr)
  call xcast_mpi(kibz,master,spaceComm,ierr)
  call xcast_mpi(oc,master,spaceComm,ierr)
  call xcast_mpi(typat,master,spaceComm,ierr)
  call xcast_mpi(xred,master,spaceComm,ierr)
  call xcast_mpi(nel,master,spaceComm,ierr)
  call xcast_mpi(nkpt,master,spaceComm,ierr)
  call xcast_mpi(ntypat,master,spaceComm,ierr)
  call xcast_mpi(fform,master,spaceComm,ierr)
  call xcast_mpi(nsppol,master,spaceComm,ierr)
  call xcast_mpi(hdr%occopt,master,spaceComm,ierr)
  call xcast_mpi(hdr%nspinor,master,spaceComm,ierr)
  call xcast_mpi(hdr%stmbias,master,spaceComm,ierr)
  call xcast_mpi(hdr%tphysel,master,spaceComm,ierr)
  call xcast_mpi(hdr%tsmear,master,spaceComm,ierr)
  call xcast_mpi(hdr%nsppol,master,spaceComm,ierr)

  !MG to treat non simmorphic operations with the new KSS version, here we have to bcast hdr%tnons
  ! but first of all I have to update fform
  call leave_test(mpi_enreg)
 end if
 if (i_can_read) then 
  read(unitkss) tit(1)
  read(unitkss) tit(2)
  write(message,'(a,1x,a79,a,1x,a79)')ch10,tit(1)(:79),ch10,tit(2)(:79)
  call wrtout(6,message,'COLL')
 end if
 ! The following string data is not important to be known by every proc
 if (me>0) then
  tit(1)='only master proc knows'   
  tit(2)='only master proc knows'
 end if
 !
 if (i_can_read) then
  read(unitkss) nsym2,nbandkss,npwkss,nsh,mpsang
  write(message,'(a,i8)')' number of electrons                    ',nel
   call wrtout(6,message,'COLL')
  write(message,'(a,i8)')' number of symmetries without inversion ',nsym2
   call wrtout(6,message,'COLL')
  write(message,'(a,i8)')' number of bands                        ',nbandkss
   call wrtout(6,message,'COLL')
  write(message,'(a,i8)')' number of plane waves                  ',npwkss
   call wrtout(6,message,'COLL')
  write(message,'(a,i8)')' number of shells                       ',nsh
   call wrtout(6,message,'COLL')
  write(message,'(a,i8,a)')' maximum angular momentum for PSPs+1    ',mpsang,ch10
   call wrtout(6,message,'COLL')
  ! enforcing equal values for nsym is better 
  if (mpsang/=mpsangx .or. nopx>nsym2 .or. ngx>npwkss .or. nwx>npwkss .or. nbx>nbandkss) then
   write(message,'(a)')' rdkss : BUG - inconsistent data2'
   call wrtout(6,message,'COLL')
   write(*,*)mpsang,mpsangx,nopx,nsym2,ngx,npwkss,nwx,npwkss,nbx,nbandkss
   call leave_new('COLL')
  end if 

  allocate(symrel2(3,3,nsym2))
  allocate(tnons2(3,nsym2))
  !OLD CODING
  ! read(unitkss) (((symrel2(jj,ii,isym),ii=1,3),jj=1,3),isym=1,nsym2)
  !ENDOLD CODING
  read(unitkss) (((symrel2(jj,ii,isym),jj=1,3),ii=1,3),isym=1,nsym2)
  read(unitkss) ((tnons2(ii,isym),ii=1,3),isym=1,nsym2)
  if (any(abs(tnons2(:,1:nsym2))>tol8) ) then 
   write(message,'(2a)')' Non-Symmorphic Space group :',ch10
   call wrtout(06,message,'COLL')
   ! Calculate symmetry operations in reciprocal space (inverse-transpose)
   allocate(symrec2(3,3,nsym2))
   do isym=1,nsym2
    call mati3inv(symrel2(:,:,isym),symrec2(:,:,isym))
   end do 
   op(:,:,1:nopx)=symrec2(:,:,1:nopx)
   tnons(:,1:nopx)=tnons2(:,1:nopx)
   deallocate(symrec2)
  else 
   ! This coding is needed for historical reasons
   ! op are indeed the symmetry operations in reciprocal space
   ! here we do not store transpose R**-1 but transpose R 
   ! this approach is correct since Rs form a group but in this 
   ! way we lose the correspondence btw an operation in G space
   ! and the phase associated to tnons. No harm is done in case 
   ! of a symmorphic space group since all the translations are zero
   ! If sym ops are not read in such order some automatic tests fail 
   ! due to a different, althought equivalent, IBZ for q-points 
   write(message,'(a)')' Symmorphic Space group :'
   call wrtout(06,message,'COLL')
   do isym=1,nopx
    op(:,:,isym)=transpose(symrel2(:,:,isym))
   end do 
   tnons(:,1:nopx)=tnons2(:,1:nopx)
  end if 
  deallocate(tnons2)
  write(message,'(2a)')ch10,&
&  ' Rotations                           Translations '
  call wrtout(6,message,'COLL')
  do isym=1,nopx
   write(message,'(1x,(3(3i3,1x),4x,3(f11.7,1x)))' )&
&   symrel2(1:3,1:3,isym),tnons(1:3,isym)
   call wrtout(6,message,'COLL')
  end do 
 end if !i_can_read

 if (master_cast) then
  write(message,'(2a)')ch10,&
&  ' rdkss : master is sending second set of data...'
  call wrtout(06,message,'COLL')
  call xcast_mpi(nsym2,master,spaceComm,ierr)
  call xcast_mpi(nbandkss,master,spaceComm,ierr)
  call xcast_mpi(npwkss,master,spaceComm,ierr)
  call xcast_mpi(nsh,master,spaceComm,ierr)
  call xcast_mpi(mpsang,master,spaceComm,ierr)
  call xcast_mpi(op,master,spaceComm,ierr)
  call xcast_mpi(tnons,master,spaceComm,ierr)
  call leave_test(mpi_enreg) 
 end if

 if (i_can_read) then
  allocate(shlim(nsh))
  read(unitkss) ((gvec(i,ig),i=1,3),ig=1,ngx)
  read(unitkss) (shlim(i),i=1,nsh)
  write(message,'(3a)')ch10,&
&  '  shell limit G-vectors [reduced coordinates] ',ch10
  call wrtout(6,message,'COLL')
  if (me==0) then !only master writes
   enough=0
   osl=1
   do ish=1,nsh
    enough=enough+1 
    if (enough>100 .and. prtvol==0) then 
     write(message,'(a)')' rdkss : prtvol=0, stop printing more G-vector information'
     call wrtout(6,message,'COLL')
     exit
    end if
    if (shlim(ish)>ngx) osl= shlim(ish)+1
    write(*,'(i6,i6,4(i5," (",3i3,")"),20(/12x,4(i5," (",3i3,")")))')&
&   ish,shlim(ish),(ig,(gvec(i,ig),i=1,3),ig=osl,shlim(ish))
    osl= shlim(ish)+1
   end do
   write(*,*)
  end if
  !
  ! Read Kleynmann-Bylander sign and matrix elements
  !
  if (fform==502) then
   allocate(vkbsignd(mpsang,ntypat))
   read(unitkss) ((vkbsignd(il,is),il=1,mpsang),is=1,ntypat)
   vkbsign(1:mpsang,1:ntypat)=vkbsignd(1:mpsang,1:ntypat)
   deallocate(vkbsignd)
  else if(fform==602) then
   allocate(vkbsigns(mpsang,ntypat))
   read(unitkss) ((vkbsigns(il,is),il=1,mpsang),is=1,ntypat)
   vkbsign(1:mpsang,1:ntypat)=vkbsigns(1:mpsang,1:ntypat)
   deallocate(vkbsigns)
  end if
 end if !i_can_read
 if (master_cast) then
  write(message,'(2a)')ch10,&
&  ' rdkss : master is sending third set of data...'
  call wrtout(06,message,'COLL')
  call xcast_mpi(vkbsign,master,spaceComm,ierr)
  call xcast_mpi(gvec,master,spaceComm,ierr)
  call leave_test(mpi_enreg) 
 end if
 if (me==0) then 
  write(6,'(" vkbsign: ",8(1x,f4.1))') vkbsign(:,:)
  !call wrtout(06,message,'COLL')
 end if 
 if (fform==502) then
  allocate(vkbdb(npwkss,ntypat,mpsang))
  allocate(vkbdd(npwkss,ntypat,mpsang))
  allocate(energyd(nbandkss))
  allocate(wfgd(npwkss))
 else if(fform==602) then
  allocate(vkbs(npwkss,ntypat,mpsang))
  allocate(vkbds(npwkss,ntypat,mpsang))
  allocate(energys(nbandkss))
  allocate(wfgs(npwkss))
 end if
 write(message,'(2a)')ch10,' k       eigenvalues [eV]'
 call wrtout(ab_out,message,'COLL') ; call wrtout(6,message,'COLL')
 ! MG here it is possible to reduce the number of BCAST
 do isppol=1,nsppol
  do ik=1,nkpt
   !
   ! Read Kleynmann-Bylander matrix elements and corresponding derivatives for this k-point
   !
   do is=1,ntypat
    do il=1,mpsang
     if (fform==502) then
      if (i_can_read) then
       read(unitkss) vkbdb(:,is,il)
       read(unitkss) vkbdd(:,is,il)
      end if
      if (master_cast) then
       call xcast_mpi(vkbdb(:,is,il),master,spaceComm,ierr)
       call xcast_mpi(vkbdd(:,is,il),master,spaceComm,ierr)
      end if
      vkb(1:nwx,is,il,ik)=vkbdb(1:nwx,is,il)
      vkbd(1:nwx,is,il,ik)=vkbdd(1:nwx,is,il)
     else if (fform==602) then
      if (i_can_read) then
       read(unitkss) vkbs(:,is,il)
       read(unitkss) vkbds(:,is,il)
      end if
      if (master_cast) then
       call xcast_mpi(vkbs(:,is,il),master,spaceComm,ierr)
       call xcast_mpi(vkbds(:,is,il),master,spaceComm,ierr)
      end if
      vkb(1:nwx,is,il,ik)=vkbs(1:nwx,is,il)
      vkbd(1:nwx,is,il,ik)=vkbds(1:nwx,is,il)
     end if
     if (DEBUG) write(*,*)'vkb ',vkb(:,:,:,ik)
     if (DEBUG) write(*,*)'vkbd ',vkbd(:,:,:,ik)
    end do !il
   end do !is
   ! Each proc stores only the sub-block of eigenvalues request by the user
   if (fform==502) then
    if (i_can_read) read(unitkss) energyd(1:nbandkss)
    if (master_cast) call xcast_mpi(energyd,master,spaceComm,ierr)
    en(ik,1:nbx,isppol)=energyd(1:nbx)
   else if (fform==602) then
    if (i_can_read) read(unitkss) energys(1:nbandkss)
    if (master_cast) call xcast_mpi(energys,master,spaceComm,ierr)
    en(ik,1:nbx,isppol)=energys(1:nbx)
   end if
   if (me==0) then 
    if (nsppol==2) then 
     write(6,'(i3,a,10f7.2/50(10x,10f7.2/))') ik,stag(isppol),(en(ik,ib,isppol)*Ha_eV,ib=1,nbx)
     write(ab_out,'(i3,a,10f7.2/50(10x,10f7.2/))') ik,stag(isppol),(en(ik,ib,isppol)*Ha_eV,ib=1,nbx)
    else 
     write(6,'(i3,7x,10f7.2/50(10x,10f7.2/))') ik, (Ha_eV*en(ik,ib,isppol),ib=1,nbx)
     write(ab_out,'(i3,7x,10f7.2/50(10x,10f7.2/))') ik, (Ha_eV*en(ik,ib,isppol),ib=1,nbx)
    end if
   end if
   if (fform==502) then
    ! Each proc stores only the sub-block of wavefunction request by the user
    do ib=1,nbx
     if (i_can_read) read(unitkss) wfgd(1:npwkss)
     if (master_cast) call xcast_mpi(wfgd,master,spaceComm,ierr)
     if (ib>=min_band_proc .and. ib<=max_band_proc) wf(1:nwx,ib,ik,isppol)= wfgd(1:nwx)
     ! In the new algorithm each proc stores the valence states in a separate array
     ! FIXME should optimize the communication should check the localrdwf==0 case
     if (use_tr .and. ib<=nbvw) wf_val(1:nwx,ib,ik,isppol)= wfgd(1:nwx)
    end do
    do ib=nbx+1,nbandkss !skip the rest
     if (i_can_read) read(unitkss) wfgd(1:npwkss)
    end do
   else if (fform==602) then
    ! Each proc stores only the sub-block of wavefunction request by the user
    do ib=1,nbx
     if (i_can_read) read(unitkss) wfgs(1:npwkss)
     if (master_cast) call xcast_mpi(wfgs,master,spaceComm,ierr)
     if (ib>=min_band_proc .and. ib<=max_band_proc) wf(1:nwx,ib,ik,isppol)= wfgs(1:nwx)
     ! In the new algorithm each proc stores the valence states in a separate array
     ! FIXME should optimize the communication, should check the localrdwf==0 case
     if (use_tr .and. ib<=nbvw) wf_val(1:nwx,ib,ik,isppol)= wfgs(1:nwx)
    end do
    do ib=nbx+1,nbandkss !skip the rest
     if (i_can_read) read(unitkss) wfgs(1:npwkss)
    end do
   end if !fform
  end do !ik loop over k-points
 end do !isppol

 einf1=1.0e24 ; esup1=0.0
 cinf1=1.0e24 ; csup1=0.0

 do isppol=1,nsppol
  do ik=1,nkx
   ! Test on the normalization of wavefunctions
   do ib=min_band_proc,max_band_proc
    dum= 0.0
    do ig=1,nwx
     dum= dum + conjg(wf(ig,ib,ik,isppol))*wf(ig,ib,ik,isppol)
    end do
    if (dum<einf1) einf1= dum
    if (dum>esup1) esup1= dum
   end do
   if (mpi_enreg%nproc>1) then
    call xmin_mpi(einf1,einf,spaceComm,ierr)
    call xmax_mpi(esup1,esup,spaceComm,ierr)
   else
    einf=einf1
    esup=esup1
   end if
   ! Test on the orthogonalization of wavefunctions
   do ib=min_band_proc,max_band_proc
    do ibp=ib+1,max_band_proc
     cdum= (0.0,0.0)
     do ig=1,nwx
      cdum= cdum + conjg(wf(ig,ib,ik,isppol))*wf(ig,ibp,ik,isppol)
     end do
     if (abs(cdum)<cinf1) cinf1= abs(cdum)
     if (abs(cdum)>csup1) csup1= abs(cdum)
    end do
   end do
   if (mpi_enreg%nproc>1) then
    call xmin_mpi(cinf1,cinf,spaceComm,ierr)
    call xmax_mpi(csup1,csup,spaceComm,ierr)
   else
    csup= csup1
    cinf= cinf1
   end if
  end do !ik
 end do !isppol

 !FIXME add a label for the spin 
 !this part should be modified to take into account the mew implementation but Im lazy now
 write(message,'(2a)')ch10,' test on the normalization of the wavefunctions'
 call wrtout(6,message,'COLL') ; call wrtout(ab_out,message,'COLL')

 write(message,'(a,f9.6,a,a,f9.6)')&
& ' min sum_G |a(n,k,G)| = ',einf,ch10,&
& ' max sum_G |a(n,k,G)| = ',esup
 call wrtout(6,message,'COLL') ; call wrtout(ab_out,message,'COLL')

 write(message,'(a)')' test on the orthogonalization of the wavefunctions'
  call wrtout(6,message,'COLL') ; call wrtout(ab_out,message,'COLL')

 write(message,'(a,f9.6,a,a,f9.6,a)')&
& ' min sum_G a(n,k,G)* a(n",k,G) = ',cinf,ch10,&
& ' max sum_G a(n,k,G)* a(n",k,G) = ',csup,ch10
  call wrtout(6,message,'COLL') ; call wrtout(ab_out,message,'COLL')

 ! In case of parallelization over bands we are evaluating
 ! the scalar product only btw bands belonging to the same sub-block
 ! write a warning on ab_out in case of para on bands 
 if (mpi_enreg%gwpara==2) then 
  write(message,'(6a)')&
&  ' rdkss : COMMENT -',ch10,&
&  '  Note that the test on the orthogonalization is not complete ',ch10,&
&  '  since bands are spread between different processors',ch10
  call wrtout(06,message,'COLL') ; call wrtout(ab_out,message,'COLL')
 end if

 if (i_can_read) close(unitkss)

 ! Deallocate arrays
 if (allocated(symrel2)) deallocate(symrel2)
 if (allocated(symrec2)) deallocate(symrec2)
 if (allocated(shlim  )) deallocate(shlim)
 if (allocated(vkbdb  )) deallocate(vkbdb)
 if (allocated(vkbdb  )) deallocate(vkbdb)
 if (allocated(vkbdd  )) deallocate(vkbdd)
 if (allocated(energys)) deallocate(energys)
 if (allocated(energyd)) deallocate(energyd)
 if (allocated(wfgs   )) deallocate(wfgs)
 if (allocated(wfgd   )) deallocate(wfgd)
 if (allocated(vkbs   )) deallocate(vkbs)
 if (allocated(vkbds  )) deallocate(vkbds)

!DEBUG
!write(message,'(a)')' rdkss : exit '
!call wrtout(06,message,'PERS')
!call leave_test(mpi_enreg) 
!ENDDEBUG

 end subroutine rdkss
!!***
