!{\src2tex{textfont=tt}}
!!****f* ABINIT/rdqps
!! NAME
!! rdqps
!!
!! FUNCTION
!!  Read a _QPS file
!!
!! COPYRIGHT
!!  Copyright (C) 2005-2007 ABINIT group (FBruneval, MG)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!  ns= 1 for unpolarized, 2 for spin-polarized
!!  nb= number of bands considered in the calculation 
!!  nk= number of irreducible k-points
!!  gwcalctyp= type of GW calculation (see related input variable) 
!!  dtfil=filenames 
!!  en(nk,nb,ns)= energies for each k-point, band and spin 
!!  kibz(3,nk)= reduced coordinates of each irreducible k-point
!!
!! OUTPUT
!!  nbsc=number of bands used to describe the QP amplitudes 
!!  nscf= number of iterations that have been performed (==0 if we start from a KS calculation)  
!!  en_qp(nk,nb,ns)=  QP energies at iteration nscf 
!!  m_lda_to_qp(nb,nb,nk,ns)= matrix giving the decomposition of the QP
!!   wavefunction along the mainfold generated by the KS wavefunctions 
!!   (i.e $ m_lda_to_qp(ib,jb,k,s) := \langle \psi_{ib,k,s}^{KS}| \psi_{jb,k,s}^{QP}\rangle $
!!  rho_p(nr,ns)=quasi-particle density
!!
!! SIDE EFFECTS
!!
!! NOTES
!!
!! PARENTS
!!      screening,sigma
!!
!! CHILDREN
!!      leave_new,wrtout
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine rdqps(dtfil,gwcalctyp,nk,nb,ns,kibz,nscf,nr,en,&
& nbsc,en_qp,m_lda_to_qp,rho_p)

 use defs_basis
 use defs_datatypes

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01manage_mpi
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: gwcalctyp,nb,nk,nr,ns
 integer,intent(out) :: nscf,nbsc
 type(datafiles_type),intent(in) :: dtfil
!arrays
 real(dp),intent(in) :: en(nk,nb,ns),kibz(3,nk)
 real(dp),intent(inout) :: rho_p(nr,ns)
 real(dp),intent(out) :: en_qp(nk,nb,ns)
 complex,intent(out) :: m_lda_to_qp(nb,nb,nk,ns)

!Local variables-------------------------------
!scalars
 integer,parameter :: unitfile=999
 integer :: ib,ik,isppol,nbmin,nbr,nkr,nsr
 logical :: lfile
 character(len=500) :: message
!arrays
 real(dp) :: k(3)
 real(dp),allocatable :: en_tmp(:)
 complex,allocatable :: mtmp(:,:)

! *************************************************************************

!FIXME this should be done inside sigma.F90 and screening.F90 
! only to make the code clearer and skip the other stuff related to SCGW

 ! Initialize with KS eigenvalues
 m_lda_to_qp(:,:,:,:)=(0.0,0.0)
 do ib=1,nb
  m_lda_to_qp(ib,ib,:,:)= (1.0,0.0)
 end do

 ! Initialize with KS eigenfunctions
 en_qp(:,:,:)=en(:,:,:)

 ! In case of non self-consistent calculation, exit immediately
 if (gwcalctyp<10) then 
  write(message,'(3a)')ch10,&
& ' rdqps : Non self-consistent case, return',ch10
  call wrtout(6,message,'COLL')
  return
 end if

 ! Self-consistency: check whether file exists or not
 write(message,'(3a)')ch10,' rdqps: looking for file ',trim(dtfil%filqps)
 call wrtout(6,message,'COLL')
 !FIXME It should be written to ab_out, but the automatic test fails
 !call wrtout(ab_out,message,'COLL')
 inquire(file=trim(dtfil%filqps),exist=lfile)
 if (.not.(lfile)) then
  write(message,'(5a)')ch10,&
& ' file not found',ch10,&
& ' 1st iteration initialized with KS eigenelements',ch10
  call wrtout(6,message,'COLL')
  !FIXME It should be written to ab_out, but the automatic test fails
  !call wrtout(ab_out,message,'COLL')
  nscf=0
  return
 else
  open(unit=unitfile,file=trim(dtfil%filqps),form='formatted',status='unknown')
  !FIXME the _QPS file should contain the Abinit header 
  read(unitfile,*) nscf
  write(message,'(2a,i4,a)')ch10,&
&  ' Number of iteration(s) already performed: ',nscf,ch10
  call wrtout(6,message,'COLL')
   !FIXME It should be written to ab_out, but the automatic test fails
   !call wrtout(ab_out,message,'COLL')

  read(unitfile,*) nkr
  if (nkr/=nk) then 
   write(message,'(4a)')&
&   ' rdqps: ERROR -',ch10,&
&   '  wrong number of kpoints in ',trim(dtfil%filqps)
   call wrtout(6,message,'COLL') ; call leave_new('COLL')
  end if
  read(unitfile,*) nbr
  nbmin=min(nbr,nb)
  nbsc=nbmin
  if (nbsc/=nb) then 
   write(message,'(4a)')ch10,&
&   ' rdqps : WARNING - ',ch10,&
&   ' the QPS file contains more bands than that used in the present calculation'
   call wrtout(06,message,'COLL')
  end if
  allocate(mtmp(nbr,nbr))
  allocate(en_tmp(nbr))
  !
  read(unitfile,*) nsr

!FIXME this does not work !!!!
!  if (nsr/=nr) then 
!   write(message,'(4a,i2,2a,i2)')ch10,&
!&   ' rdqps : ERROR - ',ch10,&
!&   ' the _QPS file has been generated with nsppol= ',nsr,ch10,&
!&   ' while in the present calculation nsppol = ',ns
!   write(*,*)ns,nsr
!   call wrtout(06,message,'COLL')
!   call leave_new('COLL')
!  end if 

  !FIXME  The format of the QPS file must be standardized!
  do isppol=1,ns 
   do ik=1,nk
    read(unitfile,*) k(:)
    write(message,'(a,i5,a,3f6.3,4x,a,i2)')&
&    ' rdqps: reading ik ',ik,')  k = ',k(:),' is= ',isppol
    call wrtout(6,message,'COLL')
    if (any(abs(k(:)-kibz(:,ik))>0.001)) then 
     write(message,'(4a)')&
&     ' rdqps: ERROR-  ',ch10,&
&     '  wrong k-point read in ',trim(dtfil%filqps)
     call wrtout(6,message,'COLL') ; call leave_new('COLL')
    end if 
    do ib=1,nbr
     read(unitfile,*) en_tmp(ib)
     read(unitfile,*) mtmp(:,ib)
    end do !ib
    m_lda_to_qp(1:nbmin,1:nbmin,ik,isppol)=mtmp(1:nbmin,1:nbmin)
    en_qp(ik,1:nbmin,isppol)=en_tmp(1:nbmin)
   end do !ik
  end do !isppol 
  !
  read(unitfile,*) rho_p(:,:)
  !
  deallocate(mtmp,en_tmp)
  close(unitfile)
 end if

  write(message,'(3a)')ch10,&
& ' rdqps : File successfully read',ch10
  call wrtout(6,message,'COLL')

end subroutine rdqps
!***
!!***
