!{\src2tex{textfont=tt}}
!!****f* ABINIT/rdscr
!! NAME
!! rdscr
!!
!! FUNCTION
!! Read screening (epsilon-twiddle**-1) file in the SCR format
!!
!! COPYRIGHT
!! Copyright (C) 1999-2007 ABINIT group (GMR, VO, LR, RWG, XG, MG, RShaltaf)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!  dtfil=filenames (only dtfil%filsrc is used)
!!  iqa= (optional) used if only a particular q-point is required. In this case iqa define the index
!!   of the required q-point in the array q(3,nq)
!!  nqa=number of asked q-points (used to dimension the output arrays) it is equal to nq if the full
!!  matrix is required
!!  FIXME I should modify testscr in order to output the full set of irred q-points
!!  localrdwf= input variable (for parallel case) if 1, the SCR file is local to each machine
!!  mpi_enreg= datatype gathering information on parallelism
!!  npwvec=number of planewaves
!!  nomega=number of frequencies
!!  nq=number of q points
!!  verbose= if .true. output more
!!
!! OUTPUT
!!  q(3,nq)=coordinates of q points
!!  epsm1(npwvec,npwvec,nomega,nq)=epsilon tilde minus 1 matrix, for each frequency and q point
!!  omega(nomega)=frequencies
!!
!! NOTES
!!  If the epsilon matrix read is bigger than NPWVEC x NPWVEC,
!!  it will be truncated; if it is smaller, an error will occur
!!
!! PARENTS
!!      csigme,mrgscr,sigma,sigma_OLDPARA
!!
!! CHILDREN
!!      hdr_clean,hdr_io,hdr_io_netcdf,leave_new,memerr,wrtout,xcast_mpi
!!      xcomm_init,xmaster_init,xme_init
!!
!! SOURCE
#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine rdscr(dtfil,npwvec,nq,nqa,nomega,q,omega,epsm1,mpi_enreg,localrdwf,verbose,iqa)

 use defs_basis
 use defs_datatypes

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01manage_mpi
 use interfaces_13io_mpi
 use interfaces_15gw, except_this_one => rdscr
 use interfaces_lib01hidempi
#else
 use defs_interfaces
 use defs_xfuncmpi
#endif
!End of the abilint section

 implicit none
#if defined MPI || defined MPI_FFT
    include 'mpif.h'
#endif
    
!Arguments ------------------------------------
!scalars
 integer,intent(in) :: localrdwf,nomega,npwvec,nq,nqa
 integer,optional,intent(in) :: iqa
 logical,intent(in) :: verbose
 type(datafiles_type),intent(in) :: dtfil
 type(MPI_type),intent(in) :: mpi_enreg
!arrays
 real(dp),intent(out) :: q(3,nq)
 complex,intent(out) :: epsm1(npwvec,npwvec,nomega,nqa),omega(nomega)
  
!Local variables-------------------------------
!scalars
 integer,parameter :: unitem1=24
 integer :: fform,headform,ii,io,ios,iq,istat,nbndse,nomegae,npwvece,npwwfne,nqe,rdwr
 integer :: readnetcdf
 real(dp) :: dielconst
 type(hdr_type) :: hdr
 character(len=500) :: message
!arrays
 real(dp) :: qd(3,nq)
 character(len=80) :: titem1(2)
 complex(dpc),allocatable :: epsm1d(:,:),omegad(:)
 integer :: spaceComm,me,master,ierr
 logical :: i_read,master_cast,single_qpoint

! *************************************************************************

 call xcomm_init(mpi_enreg,spaceComm)  !Init mpi_comm
 call xme_init(mpi_enreg,me)           !Init me
 call xmaster_init(mpi_enreg,master)   !Init master

 single_qpoint=.false.
 if (present(iqa)) then 
  single_qpoint=.true.
  write(message,'(a,i5,2a)')&
&  ' rdscr : reading the slice of the full array corresponding to iq = ',iqa,&
&  ' from file : ',trim(dtfil%filscr)
  call wrtout(6,message,'COLL')
  if (iqa>nq .or. iqa<=0) then 
   write(message,'(4a,i5)')ch10,&
&   ' rdscr : BUG - ',ch10,&
&   '  called with wrong iq = ',iqa
   call wrtout(6,message,'COLL')
   call leave_new('COLL')
  end if 
 end if 
    
 allocate(omegad(nomega))

 i_read=.false. ; master_cast=.false.
 if (localrdwf==1 .or. me==0) i_read=.true.
 if (localrdwf==0 .and. mpi_enreg%nproc>1) master_cast=.true.
   
 if (i_read) then
  if (verbose) then 
   write(message,'(4a)')ch10,&
&   ' rdscr: reading screening (epsilon-twiddle^-1) file in the SCR format from : ',trim(dtfil%filscr),ch10
   call wrtout(6,message,'COLL')
  end if

  open(unit=unitem1,file=dtfil%filscr,status='unknown',form='unformatted',iostat=ios)
  if (ios/=0) then 
   write(message,'(4a)')&
&   ' rdscr : ERROR -',ch10,&
&   ' opening file ',trim(dtfil%filscr)
   call wrtout(6,message,'COLL')
   call leave_new('COLL')
  end if  

! Read the header of the file
  rdwr=1
  readnetcdf=0 ! should become input parameter and make EM1 a netcdf file as well

  if (readnetcdf==0) then 
   call hdr_io(fform,hdr,rdwr,unitem1)
  else if (readnetcdf==1) then 
   call hdr_io_netcdf(fform,hdr,rdwr,unitem1)
  end if 

  read(unitem1) titem1
  read(unitem1) npwvece,npwwfne,nbndse,nqe,nomegae

  if (npwvece>npwvec) then
   write(message,'(3a,i8,2a,i8)')              &
&   ' rdscr : WARNING - ',ch10,                &
&   ' dimension of epsm1 matrix ',npwvece,ch10,&
&   ' reading a smaller matrix  ',npwvec
   call wrtout(6,message,'COLL')
  else if (npwvece<npwvec) then
   write(message,'(3a,i8,2a,i8)')&
&   ' rdscr : ERROR - ',ch10,&
&   ' dimension of epsm1 matrix  ',npwvece,ch10,&
&   ' requiring a too big matrix ',npwvec
   call wrtout(6,message,'COLL')
   call leave_new('COLL')
  end if

  if (nqe<nq .or. nqe<nqa) then 
   write(message,'(4a)')ch10,&
&   ' rdscr : ERROR - ',ch10,&
&   ' requiring too much q-points'
   call wrtout(6,message,'COLL')
   call leave_new('COLL')
  end if 

  if (nomegae<nomega) then 
   write(message,'(4a)')ch10,&
&   ' rdscr : ERROR - ',ch10,&
&   ' requiring too much omega-points'
   call wrtout(6,message,'COLL')
   call leave_new('COLL')
  end if 

  read(unitem1) !skip gvec
  read(unitem1) qd(1:3,1:nq)
  read(unitem1) omegad(1:nomega)
  q(:,:)= qd(:,:)
  omega(:)= omegad(:)

  if (me==0 .and. verbose) then 
   write(6,'(a,/,(i3,3f14.6))')&
&   ' q-points [reciprocal lattice units]:',(iq,(q(ii,iq),ii=1,3),iq=1,nq)
   write(6,'(a,/,(i3,2f7.2))')&
&   ' frequencies omega [eV]:',(io,omega(io)*Ha_eV,io=1,nomega)
  end if 
 end if ! i_read

 if (master_cast) then
  call xcast_mpi(q,master,spaceComm,ierr)
  call xcast_mpi(npwvece,master,spaceComm,ierr)
  call xcast_mpi(omega,master,spaceComm,ierr)
  call xcast_mpi(nomegae,master,spaceComm,ierr)
 end if

!Read epsilon^-1
 allocate(epsm1d(npwvece,npwvece),stat=istat)
 if(istat/=0)then
  call memerr('rdscr','epsm1d',npwvece**2,'dpc')
 endif

 if (single_qpoint) then
! Read only a slice of the full array (supposed to be huge)
  do iq=1,nq 
   if (iq==iqa) then 
    do io=1,nomega
     if (i_read) read(unitem1) epsm1d(1:npwvece,1:npwvece)
     if (master_cast) call xcast_mpi(epsm1d,master,spaceComm,ierr)
     epsm1(1:npwvec,1:npwvec,io,1)= epsm1d(1:npwvec,1:npwvec)
    end do
    do io=nomega+1,nomegae
     if (i_read) read(unitem1) ! skip other omegas
    end do
   else 
    do io=1,nomegae
     if (i_read) read(unitem1) ! skip epsm1d(1:npwvece,1:npwvece)
    end do
   end if 
  end do
 else  
! Read full array
  do iq=1,nq
   do io=1,nomega
    if (i_read) read(unitem1) epsm1d(1:npwvece,1:npwvece)
    if (master_cast) call xcast_mpi(epsm1d,master,spaceComm,ierr)
    epsm1(1:npwvec,1:npwvec,io,iq)= epsm1d(1:npwvec,1:npwvec)
   end do
   do io=nomega+1,nomegae
    if (i_read) read(unitem1) ! skip other omegas
   end do
  end do !iq
 end if 

 if (master_cast) call leave_test(mpi_enreg) 

 close(unitem1)

 if (allocated(epsm1d)) deallocate(epsm1d)
 if (allocated(omegad)) deallocate(omegad)

 if (i_read) call hdr_clean(hdr)

 if (verbose) then
  if (abs(real(omega(1)))<0.01 .and. abs(aimag(omega(1)))<0.01) then
   do iq=1,nq
    if (all(abs(q(:,iq))<0.001)) then
     dielconst = 1.0/real(epsm1(1,1,1,iq))
     write(message,'(a,f6.2)')' dielectric constant with LF = ', dielconst
     call wrtout(6,message,'COLL')
    end if
   end do
  end if
  write(message,'(a)')' epsilon-twiddle^-1 read '
  call wrtout(6,message,'COLL')
 end if 

 end subroutine rdscr
!!***
