!{\src2tex{textfont=tt}}
!!****f* ABINIT/setmesh
!!
!! NAME
!! setmesh
!!
!! FUNCTION
!! Calculate the size of the FFT grid for the GW calculation.
!!
!! COPYRIGHT
!! Copyright (C) 1999-2007 ABINIT group (GMR, VO, LR, RWG, YMN, MG)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!  addshell= number of shell that must be added to take into account umklapp processes
!!  enforce_sym = flag to enforce a FFT compatible with all the symmetry operations, both rotations and 
!!   fractional translations (should be default)
!!  gmet(3,3) = reciprocal space metric.
!!  gvec(3,npwvec) = G-vectors array.
!!  method = integer flag for FFT grid (see below)
!!  npwvec= number of G vectors in the array gvec max(npwwfn,npwsigx)
!!  npwsigx = size of the dielectric or self-energy matrix.
!!  npwwfn = number of G-vectors in the wavefunctions.
!!  nsym = number of symmetry operations 
!!  symrel(3,3,nsym) = symmetry operations in real space
!!  tnons(3,nsym) = fractional translations
!!  verbose= if set to 1 then warnings and comments will be reported also in the main output file 
!!
!! OUTPUT
!!  n1,n2,n3  = size of FFT grid.
!!  n1d,n2d,n3d = size of the augmented FFT grid (actually equal to normal the normal FFT).
!!  nr = n1d*n2d*n3d = number of points on the augmented FFT grid.
!!
!! NOTE
!!  In principle, the FFT grid can be augmented to improve the efficiency of the FFT
!!  (see ~abinit/doc/input_variables/vargs.htm#ngfft).
!!  This is now actually done "on the fly" in the GW code (see e.g. fftwfn.f), 
!!  so that setmesh always returns n1d=n1, n2d=n2, and n3d=n3.
!!
!!  Four methods have been coded for the calculation of the mesh (see parameter "method" below):
!!
!!   method=0 FFT mesh defined by the user
!!
!!   method=1 roughly takes the FFT box that encloses the larger of the two spheres of radius
!!            aliasing_factor*rwfn and rsigx, where rwfn and rsigx are the radius of the spheres
!!            with npwwfn and npwsigx planewaves respectively.
!!
!!   method=2 calculates the optimal FFT grid that allows aliasing only outside the sphere of the
!!            npwsigx planewaves (finer than method=1 with aliasing_factor=1).
!!
!!   method=3 calculates the FFT grid needed to expand the density.
!!            (even finer than method=2, roughly corresponds to method=1 with aliasing_factor=2).
!!
!!  At the present time method=1 is enforced. The use of method=2 or method=3 requires a change
!!  in the call to setmesh in screening.F90 and sigma.F90 (see these routines).
!!
!! PARENTS
!!      mrgscr,screening,sigma
!!
!! CHILDREN
!!      sizefft,wrtout
!!
!! SOURCE
#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine setmesh(gmet,gvec,n1,n2,n3,n1d,n2d,n3d,npwvec,npwsigx,npwwfn,nr,&
& method,addshell,aliasing_factor,nsym,symrel,tnons,enforce_sym,verbose)

 use defs_basis

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01manage_mpi
 use interfaces_15gw, except_this_one => setmesh
#endif
!End of the abilint section

 implicit none
 
!Arguments ------------------------------------
!scalars
 integer,intent(in) :: addshell,aliasing_factor,enforce_sym,npwvec,npwsigx,npwwfn,nsym 
 integer,intent(in) :: method,verbose
 integer,intent(out) :: n1,n1d,n2,n2d,n3,n3d,nr
!arrays
 integer,intent(in) :: gvec(3,npwvec)
 integer,intent(in) :: symrel(3,3,nsym)
 real(dp),intent(in) :: tnons(3,nsym)
 real(dp),intent(in) :: gmet(3,3)
 
!Local variables ------------------------------
!scalars
 integer :: ig,ig1,ig1max,ig2,ig2max,ig3,ig3max,ii,index,is,m1,m2,m3,mm1,mm2,mm3,nt
 real(dp) :: ecuteff,ecutsigx,ecutwfn,g1,g2,g3,gsq,gsqmax,reff,rsigx,rwfn
 character(len=500) :: message
 logical :: fft_ok,fftfile
!arrays 
 integer :: fftsym(3),fftnons(3),mdum(3)
!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifndef HAVE_FORTRAN_INTERFACES
 logical :: check_rot_fft
 integer :: divisor_tns
 integer :: mcm
#endif
!End of the abilint section

!************************************************************************

 ! Calculate the limits of the sphere of npwwfn G-vectors in each direction
 m1=maxval(abs(gvec(1,1:npwwfn)))
 m2=maxval(abs(gvec(2,1:npwwfn)))
 m3=maxval(abs(gvec(3,1:npwwfn)))

 write(message,'(a,i8,a,3i6)')&
& ' setmesh: npwwfn =         ',npwwfn,' ; Max (m1,m2,m3) = ',m1,m2,m3
 call wrtout(6,message,'COLL')

 mm1=maxval(abs(gvec(1,1:npwsigx)))
 mm2=maxval(abs(gvec(2,1:npwsigx)))
 mm3=maxval(abs(gvec(3,1:npwsigx)))

 ! Ensure that G+G0 will fit into the FFT grid, where G is any of the
 ! npwsigx vectors and G0 is (i,j,k) [-2<i,j,k<2]
 ! this is required when npwsigx>npwwfn since we have to take into account
 ! umklapp G0 vectors to evaluate the oscillator matrix elements (see rho_tw_g) or 
 ! to symmetrize these quantities (see also cigfft)
 if (addshell==0 .or. addshell==1 .or. addshell==2) then
  mm1=mm1+addshell
  mm2=mm2+addshell
  mm3=mm3+addshell
 else 
  write(message,'(4a,i4,a)')ch10,&
&  ' setshell : BUG - ',ch10,&
&  ' called with wrong value of addshell = ',addshell,ch10
  call wrtout(6,message,'COLL') ; call leave_new('COLL')
 end if 
 write(message,'(a,i8,a,3i6,2a,i4)')&
& ' setmesh: npweps/npwsigx = ',npwsigx,' ; Max (mm1,mm2,mm3) = ',mm1,mm2,mm3,ch10,&
& '          addshell = ',addshell
 call wrtout(6,message,'COLL')
 !
 if (method==0) then 
  ! FFT mesh defined by user, useful for testing
  write(message,'(5a)')ch10,&
&  ' setmesh: WARNING- ',ch10,&
&  ' mesh size enforced by the user!',ch10
  call wrtout(6,message,'COLL')
  if (verbose==1) call wrtout(ab_out,message,'COLL')
  n1=20 ; n2=20 ; n3=20
  inquire(file='fft.in',exist=fftfile)
  if (fftfile) then 
   open(file='fft.in',unit=987,form='formatted')
   read(987)n1,n2,n3
   close(987)
  end if 
  n1d=n1 ; n2d=n2 ; n3d=n3
  write(message,'(a,i3,a,i3,a,i3)')&
&  ' setmesh: mesh size enforced by the user = ',n1,'x',n2,'x',n3
  call wrtout(6,message,'COLL')
  if (verbose==1) call wrtout(ab_out,message,'COLL')
  return
 else if (method==1) then
  write(message,'(a,i3)')&
&  ' setmesh: using method 1 with aliasing_factor = ',aliasing_factor
  call wrtout(6,message,'COLL')
  m1=m1*aliasing_factor
  m2=m2*aliasing_factor
  m3=m3*aliasing_factor
 else if ((method==2) .or. (method==3)) then
  ! Calculate the radius of the sphere of npwwfn G-vectors.
  ecutwfn= -1.0
  do ig=1,npwwfn
   g1=real(gvec(1,ig))
   g2=real(gvec(2,ig))
   g3=real(gvec(3,ig))
   gsq=       gmet(1,1)*g1**2+gmet(2,2)*g2**2+gmet(3,3)*g3**2+ &
&        2.0*(gmet(1,2)*g1*g2+gmet(1,3)*g1*g3+gmet(2,3)*g2*g3)
   ecutwfn= max(ecutwfn,gsq)
  end do
  rwfn=sqrt(ecutwfn)
  ecutwfn=2.0*ecutwfn*pi**2
  write(message,'(a,f7.3,a)')' setmesh: calculated ecutwfn  = ',ecutwfn,' [Ha]'
  call wrtout(06,message,'COLL')
  ! Calculate the radius of the sphere of npwsigx G-vectors.
  ecutsigx= -1.0
  do ig=1,npwsigx
   g1=real(gvec(1,ig))
   g2=real(gvec(2,ig))
   g3=real(gvec(3,ig))
   gsq=      gmet(1,1)*g1**2+gmet(2,2)*g2**2+gmet(3,3)*g3**2+ &
&       2.0*(gmet(1,2)*g1*g2+gmet(1,3)*g1*g3+gmet(2,3)*g2*g3)
   ecutsigx=max(ecutsigx,gsq)
  end do
  rsigx=sqrt(ecutsigx)
  ecutsigx= 2.0*ecutsigx*pi**2
  write(message,'(a,f7.3,a)')&
&  ' setmesh: calculated ecutsigx/ecuteps = ',ecutsigx,' [Ha]'
  call wrtout(06,message,'COLL')
  !
  ! In the calculation of the GW self-energy or of the RPA dielectric matrix,
  ! we make products Rho_12(r)=u_1*(r) u_2(r) of wavefunctions whose Fourier
  ! coefficients lie in the sphere of radius rwfn. Such products will have non
  ! vanishing Fourier coefficients in the whole sphere of radius 2*rwfn since:
  ! 
  ! Rho(G) = \sum_T u_1*(T) u_2(T+G) 
  !
  ! However, we only need the Fourier coefficients of Rho_12 that lie in the sphere 
  ! of radius rsigx. we can thus allow aliasing outside that sphere, so that the FFT box 
  ! may only enclose a sphere of radius:
  !reff= min(0.5*rsigx+rwfn,2.0*rwfn)
  reff=min(1.0*rsigx+rwfn,2.0*rwfn)
  !
  ! Extreme case: this should yield back the GS FFT grid if full wavefunctions are
  ! considered in the GW calculation...
  !
  if (method==3) reff=2.0*rwfn
  ecuteff=2*(pi*reff)**2
  write(message,'(2a,i4,2a,f7.3,a)')ch10,&
&  ' setmesh: method = ',method,ch10,&
&  '          calculated ecuteff  = ',ecuteff,' [Ha]'
  call wrtout(06,message,'COLL')
  !
  ! Search the limits of that sphere in each direction...
  !
  gsqmax=reff**2
  ig1max=2*m1+1
  ig2max=2*m2+1
  ig3max=2*m3+1
  m1=-1 ; m2=-1 ; m3=-1
  do ig1=0,ig1max
   g1=real(ig1)
   do ig2=0,ig2max
    g2=real(ig2)
    do ig3=0,ig3max
     g3=real(ig3)
     gsq=     gmet(1,1)*g1**2+gmet(2,2)*g2**2+gmet(3,3)*g3**2+ &
&        2.0*(gmet(1,2)*g1*g2+gmet(1,3)*g1*g3+gmet(2,3)*g2*g3)
     if (gsq>gsqmax) cycle
     m1=max(m1,ig1)
     m2=max(m2,ig2)
     m3=max(m3,ig3)
    end do
   end do
  end do
 else
  write(message,'(4a)')ch10,&
&  ' setmesh : BUG- ',ch10,&
&  ' method > 3 or < 0 is not allowed in setmesh ' 
  call wrtout(6,message,'COLL') ; call leave_new('COLL')
 end if
 !
 ! Warning on low npwwfn
 !
 if (m1<mm1 .or. m2<mm2 .or. m3<mm3) then
  write(message, '(9a)' ) ch10,&
&   ' setmesh: COMMENT -',ch10,&
&   '  Note that npwwfn is small with respect to npweps or with respect to npwsigx.',ch10,&
&   '  Such a small npwwfn is a waste:',ch10,&
&   '  You could raise npwwfn without loss in cpu time.',ch10
  call wrtout(std_out, message,'COLL')
 end if
 !
 ! Keep the largest of the m/mm
 !
 m1=max(m1,mm1)
 m2=max(m2,mm2)
 m3=max(m3,mm3)
 !
 ! Find the FFT grid which is compatible with the library 
 ! and, if required, with the symm operations  
 !
 if (enforce_sym==0) then 
  !
  ! Determine the best size for the FFT grid *without* considering the symm ops.
  ! Ideally n=2*m+1 but this  could not be allowed by the FFT library 
  !
  call sizefft(m1,n1)
  call sizefft(m2,n2)
  call sizefft(m3,n3)
  n1d=n1 ; n2d=n2 ; n3d=n3
  ! Calculate the number of FFT grid points
  nr=n1d*n2d*n3d
  ! Check if the FFT is compatible, write ONLY a warning if it breaks the symmetry
  fftnons(1)=n1
  fftnons(2)=n2
  fftnons(3)=n3
  fft_ok= .true.

rd: do ii=1,3 
     do is=1,nsym 
      nt=divisor_tns(tnons(ii,is))
      if (((fftnons(ii)/nt)*nt) /= fftnons(ii)) then 
       fft_ok= .false.
       exit rd
      end if 
     end do 
    end do rd

  if (.not.fft_ok) then 
   write(message,'(5a)')ch10,&
&   ' setmesh: WARNING -',ch10,&
&   '  FFT mesh is not compatible with non-symmorphic translations',ch10
   call wrtout(6, message,'COLL')
   !otherwise tests fail
   if (verbose==1) call wrtout(ab_out, message,'COLL')
  end if 
  ! Check only rotations
  if (.not.(check_rot_fft(nsym,symrel,n1,n2,n3))) then
   write(message, '(5a)' )ch10,&
&   ' setmesh: WARNING -',ch10,&
&   ' FFT mesh is not compatible with rotations',ch10
   call wrtout(6, message,'COLL')
   !otherwise tests fail
   if (verbose==1) call wrtout(ab_out, message,'COLL')
  end if 
 else 
  !
  ! Determine the best size for the FFT grid considering the symm ops.
  ! Ideally n=2*m+1 but this  could not be allowed by the FFT library 
  !
  write(message,'(2a)')ch10,&
&  ' setmesh : finding a FFT mesh compatible with all the symmetries'
  call wrtout(6, message,'COLL')
  fftnons(:)=1
  ! Found a FFT mesh compatible with the non-symmorphic operations 
  do ii=1,3
   fftnons(ii)=1
   do is=1,nsym 
    nt=divisor_tns(tnons(ii,is))
!DEBUG
!write(*,*)tnons(ii,is),nt
!END DEBUG
    if (((fftnons(ii)/nt)*nt)/=fftnons(ii)) fftnons(ii)= mcm(fftnons(ii),nt)
   end do 
  end do 
  write(message,'(2a,3i3)')ch10,' setmesh : divisor mesh',fftnons(:)
  call wrtout(6,message,'COLL')
  !
  ! Check if also rotations preserve the grid
  ! Initial guess from previous m values  
  !
  call sizefft(m1,fftsym(1))
  call sizefft(m2,fftsym(2))
  call sizefft(m3,fftsym(3))

  mdum(1)=m1 ; mdum(2)=m2 ; mdum(3)=m3

  index=0
  do !if a FFT division gets too large the code stops in sizefft
   if (  check_rot_fft(nsym,symrel,fftsym(1),fftsym(2),fftsym(3)) .and. &
&       (mod(fftsym(1),fftnons(1))==0) .and.                            &
&       (mod(fftsym(2),fftnons(2))==0) .and.                            &
&       (mod(fftsym(3),fftnons(3))==0) ) exit 
   ii=mod(index,3)+1
   mdum(ii)=mdum(ii)+1
   call sizefft(mdum(ii),fftsym(ii))
   index=index+1
  end do 

  ! Got a good FFT grid
  n1=fftsym(1) ; n2=fftsym(2) ; n3=fftsym(3)
  ! Calculate the number of FFT grid points
  n1d=n1 ; n2d=n2 ;n3d=n3
  nr=n1d*n2d*n3d

!DEBUG dont uncomment this part for the time being
if (.not.( check_rot_fft(nsym,symrel,n1,n2,n3)).or. &
&        ( mod(fftsym(1),fftnons(1))/=0) .and.      &
&        ( mod(fftsym(2),fftnons(2))/=0) .and.      &
&        ( mod(fftsym(3),fftnons(3))/=0) ) then 
 write(message,'(a)')' setmesh : BUG during the generation of a sym FFT'
 call wrtout(6,message,'COLL') ; call leave_new('COLL')
end if  
!ENDDEBUG
 end if !of if enforce_sym

 write(message,'(a,i3,a,i3,a,i3)')&
& ' setmesh: mesh size selected = ',n1,'x',n2,'x',n3
 call wrtout(6,message,'COLL')
 if (verbose==1) call wrtout(ab_out,message,'COLL')
 
 write(message,'(a,i8,a)')&
& ' setmesh: total size of FFT array nr = ',nr,ch10
 call wrtout(6,message,'COLL')
 if (verbose==1) call wrtout(ab_out,message,'COLL')

end subroutine setmesh
!!***

subroutine select_divisor_mesh(n,nsym,tnons)

 use defs_basis

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01manage_mpi
 use interfaces_15gw, except_this_one => select_divisor_mesh
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!Scalars 
 integer,intent(in) :: nsym
!Arrays
 integer,intent(inout) :: n(3)
 real(dp),intent(in) :: tnons(3,nsym)

!Local variables ------------------------------
 integer :: is,ix,nt
 character(len=500) :: message
!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifndef HAVE_FORTRAN_INTERFACES
 integer :: divisor_tns
 integer :: mcm
#endif
!End of the abilint section

!************************************************************************
!BEGIN EXECUTABLE SECTION

 n(:)=1

 do ix=1,3
  n(ix)= 1
  do is=1,nsym
   nt=divisor_tns(tnons(ix,is))
   !if(((n(ix) / nt) * nt) /= n(ix)) n(ix) = mcm(n(ix),nt)
   if (mod(n(ix),nt)/=0) n(ix)=mcm(n(ix),nt)
  end do
 end do
 
 write(message,'(a,i3)')' divisor mesh = ',n(:)
 call wrtout(6,message,'COLL')

end subroutine select_divisor_mesh

integer function divisor_tns(dd)
!returns the denominator of the rational number d

 use defs_basis

 implicit none

!Arguments ------------------------------------
!Scalars 
 real(dp),intent(in) :: dd
!Local variables ------------------------------
 integer :: ii

!************************************************************************
!BEGIN EXECUTABLE SECTION

 ii=1

 do
  if (abs(dd*ii-nint(dd*ii))<0.0001) then
   divisor_tns=ii
   return
  end if
  ii=ii+1
 end do

end function divisor_tns 

integer function mcm(ii,jj)
! returns the maximum common multiple of ii and jj

 use defs_basis
 implicit none

 integer,intent(in) :: ii,jj

!************************************************************************
!BEGIN EXECUTABLE SECTION
 
 mcm=1

 do
  if ( ((mcm/ii)*ii)==mcm .and. ((mcm/jj)*jj)==mcm ) return
  mcm= mcm + 1
 end do

end function mcm

logical function check_rot_fft(nsym,symrel,nr1,nr2,nr3)
! check that the grid is compatible with the given rotations in real space

 use defs_basis
 implicit none

!Arguments
!Scalar
 integer,intent(in) :: nr1,nr2,nr3,nsym 
!Arrays
 integer,intent(in) :: symrel(3,3,nsym)

!local variables
 integer :: is 
!************************************************************************
!BEGIN EXECUTABLE SECTION

 check_rot_fft=.true.
 !
 ! The grid is compatible with the symmetries (only rotation) if 
 ! for each symmetry, each n_i and n_j ==> $n_i*R_{ij}/n_j$ is an integer
 !
 do is=1,nsym
  if ( mod (symrel(2,1,is)*nr2, nr1) /= 0 .or.   &
       mod (symrel(3,1,is)*nr3, nr1) /= 0 .or.   &
       mod (symrel(1,2,is)*nr1, nr2) /= 0 .or.   &
       mod (symrel(3,2,is)*nr3, nr2) /= 0 .or.   &
       mod (symrel(1,3,is)*nr1, nr3) /= 0 .or.   &
       mod (symrel(2,3,is)*nr2, nr3) /= 0 ) then
   check_rot_fft=.false. 
   exit
  end if 
 end do

end function check_rot_fft
