!{\src2tex{textfont=tt}}
!!****f* ABINIT/setup_hamilt
!! NAME
!! setup_hamilt
!!
!! FUNCTION 
!!  Calculate matrix elements of the Hartree potential from the input wavefunctions 
!!
!! COPYRIGHT
!!  Copyright (C) 2005-2007 ABINIT group (FBruneval, MG, RShaltaf)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!  minb,maxb= minimum and maximum band index considered n the calculation of the matrix elements
!!  mpi_enreg= datatype gathering information on parallelism, variables used 
!!   |gwpara= if 2 bands are spread btw processors
!!  nfft= number of FFT grid points 
!!  ngfft1,ngfft2,ngfft3=FFT grid dimensions
!!  npwvec= number of G vectors  
!!  nsppol=1 for unpolarized, 2 for spin-polarized
!!  ucvol= volume of the unit cell
!!  dtset <type(dataset_type)>=all input variables for this dataset
!!  sp=sigma_parameters (see the definition of this structured datatype)
!!  gvec(3,npwvec)= reduced coordinates of each plane wave in reciprocal space
!!  en(sp%nk,sp%nb,nsppol)= energies per each k-point, band and spin
!!  gmet(3,3)=metrix tensor in G space in Bohr**-2.
!!  kibz(3,sp%nk)= reduced coordinates of the irreducible k-points
!!  occ(sp%nk,sp%nb,nsppol)= occupations numbers for each k-point in the IBZ, band and spin 
!!  wtk(sp%nk)= weights for irreducible k-points
!!  rho(nfft)= density in real space
!!  wfg(sp%npwwfn,sp%nb,sp%nk,nsppol)= wavefunctions in reciprocal space
!!  wfr(nfft,sp%nb,sp%nk,nsppol)= wavefunctions in real space
!!
!! OUTPUT
!!  hhartr(sp%nb,sp%nb,sp%nk,nsppol) contains $\langle b1,k,s | v_{H} | b2,k,s \rangle$
!!
!! SIDE EFFECTS
!!
!! NOTES
!!
!! PARENTS
!!      sigma
!!
!! CHILDREN
!!      zheev
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine hartrham(dtset,sp,gmet,ucvol,ngfft1,ngfft2,ngfft3,nfft,npwvec,gvec,kibz,wtk,&
& nsppol,wfr,wfg,rho,occ,en,minb,maxb,hhartr,mpi_enreg,min_band_proc,max_band_proc)

 use defs_basis
 use defs_datatypes

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01manage_mpi
 use interfaces_12ffts
 use interfaces_13xc
 use interfaces_lib01hidempi
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: max_band_proc,min_band_proc,minb,maxb,nfft,ngfft1,ngfft2,ngfft3
 integer,intent(in) :: npwvec,nsppol
 real(dp),intent(in) :: ucvol
 type(MPI_type),intent(inout) :: mpi_enreg
 type(dataset_type),intent(in) :: dtset
 type(sigma_parameters),intent(in) :: sp
!arrays
 integer,intent(in) :: gvec(3,npwvec)
 real(dp),intent(in) :: en(sp%nk,sp%nb,nsppol),gmet(3,3)
 real(dp),intent(in) :: kibz(3,sp%nk)
 real(dp),intent(in) :: occ(sp%nk,sp%nb,nsppol)
 real(dp),intent(in) :: wtk(sp%nk)
 real(dp),intent(inout) :: rho(nfft)
 complex,intent(in) :: wfg(sp%npwwfn,min_band_proc:max_band_proc,sp%nk,nsppol)
 complex,intent(in) :: wfr(nfft,min_band_proc:max_band_proc,sp%nk,nsppol)
 complex,intent(out) :: hhartr(minb:maxb,minb:maxb,sp%nk,nsppol)

!Local variables -------------------------
!scalars
 integer :: ib,ib1,ib2,ier,ig,ig1,ig2,ig3,ik,ir,ir1,ir2,ir3,is,master,me,nb,spaceComm
 real(dp) :: eband,ecut,ehartr,ekin,g1,g2,g3,gsq,gsqcut,rtmp,coeff,nfftm1
 complex :: ctmp,ctmp1,ctmp2
 character(len=500) :: message
!arrays
 integer :: ngfft(18)
 real(dp) :: qphon(3),rhog(2,nfft),vhartr(nfft)
 real(dp) :: wtk_norm(sp%nk)

!************************************************************************
!BEGIN EXECUTABLE SECTION

!DEBUG
!write(6,*)' hartrham : enter '
!ENDDEBUG

 call xcomm_init(mpi_enreg,spaceComm)  ! Init mpi_comm
 call xme_init(mpi_enreg,me)           ! Init me
 call xmaster_init(mpi_enreg,master)   ! Init master

 ngfft(:)= 0
 ngfft(1)= ngfft1
 ngfft(2)= ngfft2
 ngfft(3)= ngfft3
 ngfft(4)= 2*(ngfft(1)/2)+1
 ngfft(5)= 2*(ngfft(2)/2)+1
 ngfft(6)= ngfft(3)
 ngfft(7)= 200
 ngfft(8)= 256
 ngfft(9)= 0
 ngfft(10)= 1
 ngfft(11)= 0
 ngfft(12)= ngfft2
 ngfft(13)= ngfft3
 ngfft(14)= 0

 gsqcut= -1.0
 do ig=1,npwvec
  g1= real(gvec(1,ig))
  g2= real(gvec(2,ig))
  g3= real(gvec(3,ig))
  gsq=      gmet(1,1)*g1**2+gmet(2,2)*g2**2+gmet(3,3)*g3**2+ &
&      2.0*(gmet(1,2)*g1*g2+gmet(1,3)*g1*g3+gmet(2,3)*g2*g3)
  gsqcut= max(gsqcut,gsq)
 end do

! Calculate rho(G)
 call fourdp(1,rhog,rho,-1,mpi_enreg,nfft,ngfft,4)

!DEBUG
!write(*,*)'DEBUG setup_hamilt *** number of electrons *** ',rhog(1,1)*ucvol
!ENDDEBUG

! Calculate the band energy
! FIXME should add a condition on the occupations
 coeff= 1.0/sum(wtk(:))
 wtk_norm(:)= wtk(:)*coeff

 eband= zero

 do ib=1,sp%nb
  do ik=1,sp%nk
   do is=1,nsppol
    eband= eband+wtk_norm(ik)*en(ik,ib,is)*occ(ik,ib,is)
   end do !is
  end do !ik
 end do !ib

 !if (mpi_enreg%gwpara==2) call xsum_mpi_dpv(eband,spaceComm,ier)

 write(message,'(2a,es17.8)')ch10,' Band Energy    [Ha] = ',eband
 call wrtout(6,message,'COLL')
  
! Calculate Hartree potential and energy
 qphon(:)= zero
 call hartre(1,gmet,gsqcut,0,mpi_enreg,nfft,ngfft,qphon,rhog,vhartr)

 write(message,'(a,es17.8,a)')' Hartree Energy [Ha] = ',0.5*ucvol/nfft*sum( rho(:)*vhartr(:) ),ch10
 call wrtout(6,message,'COLL')

!MG FIXME  here there is a problem since each processor is calculating only the matrix
!elements only related to the sub-block stored in its memory
!These loops should be optimized

 nfftm1=1.0_dp/nfft

 do is=1,nsppol
  do ik=1,sp%nk
   do ib2=minb,maxb
    if (mpi_enreg%gwpara==2 .and. (mpi_enreg%proc_distrb(ib2,ik,is)/=me)) cycle
    do ib1=minb,maxb
     if (mpi_enreg%gwpara==2 .and. (mpi_enreg%proc_distrb(ib1,ik,is)/=me)) cycle
     hhartr(ib1,ib2,ik,is)= sum(vhartr(:)*conjg(wfr(:,ib1,ik,is))*wfr(:,ib2,ik,is))*nfftm1
    end do !ib1
   end do !ib2
  end do !ik
 end do !is

!DEBUG
!Check hartree energy a second time
!ehartr=0
!do is=1,nsppol
! do ik=1,sp%nk
!  do ib1=1,sp%nb
!   ehartr=ehartr+0.5*wtk(ik)/sum(wtk(:))*hhartr(ib1,ib1,ik,is)*occ(ik,ib1,is)
!  end do
! end do
!end do 
!write(*,*)' setup_hamilt hartree2',ehartr
write(*,*)'exit hartree'
!ENDDEBUG

end subroutine hartrham
!!***


!!****m* ABINIT/ham_changebasis
!! NAME
!! ham_changebasis
!!
!! FUNCTION 
!!  Calculate the (approximate) matrix elements of an operator in the quasi-particle basis set.
!!  starting from the corresponding matrix elements in the KS representation
!!  and the components of the QP wavefunctions along the KS basis  
!!
!! COPYRIGHT
!!  Copyright (C) 2005-2007 ABINIT group (FBruneval, MG)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!  nb=number of bands
!!  nk=number of k-points in the IBZ
!!  ns=1 for unpolarized, 2 for spin-polarized
!!  c(nb,nb,nk,ns)= contains $\langle \psi^{KS}_{mks} | \psi^{QP}_{nks} \rangle $
!!
!! OUTPUT
!!  (see side effects)
!!
!! SIDE EFFECTS
!! Input/Output
!!  in input  h(nb,nb,nk,ns) contains the matrix elements of the operator in the KS representation
!!  in output h(nb,nb,nk,ns) contains the matrix elements in the QP basis set
!!
!! NOTES
!!  The operator whose matrix elements are calculated must be diagonal both in spin
!!  and in the crystalline momentum k 
!!
!! TODO
!!  
!!
!! PARENTS
!!  
!!
!! CHILDREN
!!  
!!
!! SOURCE


subroutine ham_changebasis(nb,nk,ns,c,h)

 use defs_basis 

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: nb,nk,ns
 complex,intent(in) :: c(nb,nb,nk,ns)
 complex,intent(inout) :: h(nb,nb,nk,ns)

!Local variables-------------------------------
!scalars
 integer :: ik,ib1,ib2,ib3,ib4,is,istat
 character(len=500) :: message
!arrays
 complex :: ctmp(nb,nb)
 complex,allocatable :: htmp(:,:,:,:)

!************************************************************************
!BEGIN EXECUTABLE SECTION

 allocate(htmp(nb,nb,nk,ns),stat=istat)
  if(istat/=0) call memerr('ham_changebasis','htmp',nb*nb*nk*ns,'spc')

 htmp(:,:,:,:)= h(:,:,:,:)
 h(:,:,:,:)= (0.,0.)

! A_{new} = C^\dagger A_{old} C 
 do is=1,ns
  do ik=1,nk
   !do ib4=1,nb
   ! do ib3=1,nb
   !  do ib2=1,nb
   !   do ib1=1,nb
   !    h(ib3,ib4,ik)=h(ib3,ib4,ik)+conjg(c(ib1,ib3,ik))*c(ib2,ib4,ik)*htmp(ib1,ib2,ik)
   !   end do
   !  end do
   ! end do
   !end do
   ctmp(:,:)= matmul(htmp(:,:,ik,is),c(:,:,ik,is))
   h(:,:,ik,is)= matmul(transpose(conjg(c(:,:,ik,is))),ctmp(:,:))
  end do
 end do 

 deallocate(htmp)

end subroutine ham_changebasis
!!***

!!****m* ABINIT/diago_hamilt
!! NAME
!! diago_hamilt
!!
!! FUNCTION 
!!  Diagonalize the Sigma operator
!!
!! COPYRIGHT
!!  Copyright (C) 2005-2007 ABINIT group (FBruneval, MG)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!
!!  minbnd,maxbnd=min and max band index for GW corrections (for this k-point)
!!  nsppol=1 for unpolarized, 2 for spin-polarized calculations
!!  hhartree(minbnd:maxbnd,minbnd:maxbnd,nsppol)=matrix elements of $T+V_{ext}+V_H$ in the quasi-particle representation
!!  sigxme(minbnd:maxbnd,minbnd:maxbnd,nsppol)=matrix elements of $\Sigma_x$
!!  sigcme(minbnd:maxbnd,minbnd:maxbnd,nsppol)=matrix elements of $\Sigma_c(\epsilon_{KS})$
!!
!! OUTPUT
!!  en_qp(minbnd:maxbnd,nsppol)=quasi particle energies obtained diagonalizing $H+\Sigma$
!!  eigvec_qp(minbnd:maxbnd,minbnd:maxbnd,nsppol)=components of the QP wavefunctions
!!
!! NOTES
!! 
!!
!! TODO
!!  
!!
!! PARENTS
!!  
!!
!! CHILDREN
!!  
!!
!! SOURCE


subroutine diago_hamilt(minbnd,maxbnd,nsppol,hhartree,sigxme,sigcme,eigvec_qp,en_qp)

 use defs_basis

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01manage_mpi
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: minbnd,maxbnd,nsppol
!arrays
 complex,intent(in) :: hhartree(minbnd:maxbnd,minbnd:maxbnd,nsppol)
 complex,intent(in) :: sigxme(minbnd:maxbnd,minbnd:maxbnd,nsppol)
 complex,intent(in) :: sigcme(minbnd:maxbnd,minbnd:maxbnd,nsppol)
 complex,intent(inout) :: eigvec_qp(minbnd:maxbnd,minbnd:maxbnd,nsppol)
 real(dp),intent(out) :: en_qp(minbnd:maxbnd,nsppol)

!Local variables-------------------------------
!scalars
 integer :: ib,is,nmatrix
 integer :: lwork,info
 character(len=500) :: message
!arrays
 complex :: htotal(minbnd:maxbnd,minbnd:maxbnd,nsppol)
 complex(dpc),allocatable :: hdp(:,:),work(:)
 real(dp),allocatable :: eig(:),rwork(:)

#ifdef __VMS
!DEC$ ATTRIBUTES ALIAS:'ZHEEV' :: zheev
#endif

!************************************************************************
!BEGIN EXECUTABLE SECTION

 write(message,'(a)')' diago_hamilt : diagonalize the QP Hamiltonian'
 call wrtout(6,message,'COLL')

! Size of the matrix
 nmatrix= maxbnd-minbnd+1

 htotal(:,:,:)= hhartree(:,:,:)+sigxme(:,:,:)+sigcme(:,:,:)

 write(message,'(a)')' diago_hamilt : hermitianizing the QP Hamiltonian'
 call wrtout(6,message,'COLL')

! Hermitianize the hamiltonian
 do is=1,nsppol
  htotal(:,:,is)= 0.5*( htotal(:,:,is) + transpose(conjg(htotal(:,:,is))) )
 end do 

 lwork= 2*nmatrix-1
 allocate(hdp(nmatrix,nmatrix),eig(nmatrix),work(lwork),rwork(3*nmatrix-2))

 do is=1,nsppol
  hdp(:,:)= htotal(minbnd:maxbnd,minbnd:maxbnd,is)

  call zheev('v','u',nmatrix,hdp,nmatrix,eig,work,lwork,rwork,info)

  eigvec_qp(minbnd:maxbnd,minbnd:maxbnd,is)= hdp(:,:)
  en_qp(minbnd:maxbnd,is)= eig(:)
 end do 

 deallocate(hdp,eig,work,rwork)

!DEBUG
!do ib=minbnd,maxbnd
! write(*,'(i3,1x,9(f12.5)))') ib,hhartree(ib,ib)*Ha_eV,real(sigxme(ib,ib))*Ha_eV,&
!&  sigcme(ib,ib)*Ha_eV,htotal(ib,ib)*Ha_eV,en_qp(ib)*Ha_eV,abs(eigvec_qp(ib,ib))
!end do
!ENDDEBUG

end subroutine diago_hamilt
!!***
