!{\src2tex{textfont=tt}}
!!****f* ABINIT/setup_little_group
!! NAME
!! setup_little_group
!!
!! FUNCTION
!!  Set up little group operations and symmetry tables used in GW calculations
!!
!! COPYRIGHT
!!  Copyright (C) 2006-2007 ABINIT group (MG)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!  gmet(3,3)=reciprocal space metric (bohr**-2).
!!  gvec(3,npwvec) coordinates of G vectors
!!  kbz(3,nkbz)=points in the full Brillouin Zone
!!  ngfft1a,ngfft1,ngfft2,ngfft3=FFT grid dimensions
!!  nop=number of symmetry operations
!!  npwvec=number of G vectors
!!  ninv=if 2, inversion is considered; if 1, inversion is not considered
!!  nkbz=number of points in the full BZ
!!  op(3,3,nop)=symmetry operations in reciprocal space
!!  prtvol=input variable defining the verbosity
!!  xpt(3)= point in the Brillouin zone  
!!
!! OUTPUT
!!  ibzq(nkbz)= 1 if the kpoint belongs to the IBZ defined by xpt, 0 otherwise
!!  ibzq_ind(nkbz)= sequential index of the point in the IBZ defined by xpt
!!  iumklp(ninv,nop)= index of the uklapp vector G_o in the FFT array
!!  fumklp(ninv,nop)= flag for umklapp processes 
!!                    if 1 that the particular operation (IS) requires a G_o to preserve q
!!                       0 otherwise 
!!  lgtab(nkbz)=table giving, for each k-point in the BZ (kBZ), the corresponding 
!!   irreducible point (kIBZ) in the irreducible zone defined by the little group of xpt,
!!   i.e kBZ= (IS) kIBZ where I is either the inversion or the identity and S is an
!!   operation in the little group defined by xpt
!!  lgtabo(nkbz)=the symmetry operation S in the little group that takes kIBZ to each kBZ
!!  lgtabi(nkbz)= defines whether inversion has to be considered in the 
!!   relation kBZ=(IS) kIBZ (1 => only S; -1 => -S)  
!!  ltg(2,nop)= 1 if ISq=q, 0 otherwise, the first index is for the identity or the time reversal symmetry, 
!!  wtksym(2,nop,nkbz)= for each kpoint is equal to 1 if the symmetry operation (with or without time reversal)  
!!   must be considered in the calculation of \chi_o, 0 otherwise  
!!
!! SIDE EFFECTS
!!
!! NOTES
!!
!!
!! PARENTS
!!  
!!
!! CHILDREN
!! 
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine setup_little_group (xpt,gmet,nop,op,ninv,nkbz,kbz,&
& ibzq,ibzq_ind,ltg,lgtab,lgtabo,lgtabi,wtksym,&
& iumklp,fumklp,npwvec,gvec,ngfft1a,ngfft1,ngfft2,ngfft3,prtvol)

 use defs_basis

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01manage_mpi
 use interfaces_12geometry
 use interfaces_13recipspace
 use interfaces_15gw, except_this_one => setup_little_group
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: ninv,nkbz,nop,npwvec,ngfft1a,ngfft1,ngfft2,ngfft3,prtvol
!arrays
 integer,intent(in) :: gvec(3,npwvec)
 integer,intent(inout) :: ibzq(nkbz),ibzq_ind(nkbz),lgtab(nkbz),lgtabi(nkbz),lgtabo(nkbz)
 integer,intent(inout) :: ltg(ninv,nop),wtksym(ninv,nop,nkbz),iumklp(npwvec,ninv,nop),fumklp(ninv,nop)
 real(dp),intent(in) :: gmet(3,3),kbz(3,nkbz),op(3,3,nop),xpt(3)

!Local variables-------------------------------
!scalars
 integer :: enough,idx,ig,igp,gmg01,gmg02,gmg03,iinv,ik,ikp,ind,iold,iop,itest,nkibzq,nsymltg,nsymq
 integer :: ntest,timrev
 logical :: DEBUG=.true.,VERBOSE=.true.,no_identity
 character(len=500) :: message
!arrays
 integer :: dummy(nkbz),g0(3,ninv,nop),identity(3,3)
 integer :: indkpt1(nkbz),op_int(3,3,nop),symxpt(4,2,nop)
 integer,allocatable :: opltg(:,:,:),symafm(:)
 real(dp) :: knew(3),ktest(3,nkbz),op_q(3,3,nop),wtk(nkbz),wtk_folded(nkbz)

! *************************************************************************
!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifndef HAVE_FORTRAN_INTERFACES
 integer :: icmpk2
 integer :: modx
#endif
!End of the abilint section

 identity(:,:)=reshape((/1,0,0,0,1,0,0,0,1/),(/3,3/))

 write(message,'(2a,3f8.5,a)')ch10,&
& ' setup_little_group : Analyzing symmetries at point ',xpt,ch10 
 call wrtout(6,message,'COLL')

 ! symq3 requires an integer argument while in the GW code operations are real
 op_int=nint(op)

 ! dirty trick useful in case of old version of KSS (<5.3?)
 ! symq3 assumes that the identity operation is present
 ! but in the GW part we are removing symmetries related by the inversion 
 ! sometimes it happens that only the inversion is reported in the KSS file (see outkss.F90) 
 ! to bypass the problem we use this trick (note however that the code crashes if ninv=1 and the identity is not found)
 !FIXME this part should be removed since this bug has been fixed
 no_identity=.true. 
 do iop=1,nop
  if (all(op_int(:,:,iop)==identity)) then 
   no_identity=.false.
   exit
  end if
 end do 
 
 if (no_identity) then 
  write(message,'(4a)')&
&  ' setup_little_group : COMMENT -',&
&  ' Only the inversion was found in the set of symmetries read from the KSS file ',ch10,&
&  ' Maybe old version of KSS file? '
  call wrtout(6,message,'COLL')
  op_int=-op_int
 end if 

 ! Find operations in the little group as well as umklapp Go vectors 
 call symq3(nop,xpt,symxpt,op_int,timrev) ! timrev is not used 
 ltg(:,:)=0 ; g0=0 ; fumklp=0
 
 do iop=1,nop
  ! Do not include umklapp processes  for the moment
  if (symxpt(4,1,iop)==1 .and. all(symxpt(1:3,1,iop)==0)) then !no time reversal and G==0
   !if (symxpt(4,1,iop)==1) then ! Sq = q + g0
   if (.not.no_identity)then 
    ltg(1,iop)=1
    g0(:,1,iop)=symxpt(1:3,1,iop)
    if (any(g0(:,1,iop)/=0)) fumklp(1,iop)=1
   else  
    ltg(2,iop)=1
    g0(:,2,iop)=symxpt(1:3,1,iop)
    if (any(g0(:,2,iop)/=0)) fumklp(2,iop)=1
   end if  
  end if
  ! Do not include umklapp processes for the moment
  if (symxpt(4,2,iop)==1 .and. all(symxpt(1:3,2,iop)==0)) then !time reversal and G==0
   !if (symxpt(4,2,iop)==1) then ! -Sq = q +g0
   if (.not.no_identity) then 
    ltg(2,iop)=1
    g0(:,2,iop)=symxpt(1:3,2,iop)
    if (any(g0(:,2,iop)/=0)) fumklp(2,iop)=1
   else 
    ltg(1,iop)=1
    g0(:,1,iop)=symxpt(1:3,2,iop)
    if (any(g0(:,1,iop)/=0)) fumklp(1,iop)=1
   end if
  end if 
 end do !iop

 if (no_identity) then 
  write(message,'(4a,2i5,a,i5,2a,2i5,a)')ch10,&
&  ' Total Number of symmetry operations in the little group ',ch10,&
&  '  with and without time reversal symmetry : ',sum(ltg(2,:)),sum(ltg(1,:)),' / ',nop,ch10,&
   '  umklapp processes                         ',sum(fumklp(2,:)),sum(fumklp(1,:)),ch10
 else
  write(message,'(4a,2i5,a,i5,2a,2i5,a)')ch10,&
&  ' Total Number of symmetry operations in the little group ',ch10,&
&  '  with and without time reversal symmetry : ',sum(ltg(1,:)),sum(ltg(2,:)),' / ',nop,ch10,&
&  '  umklapp processes                         ',sum(fumklp(1,:)),sum(fumklp(2,:)),ch10
 end if
 call wrtout(6,message,'COLL')
 
 nsymltg=sum(ltg(:,:))
 allocate(opltg(3,3,nsymltg))  

 ind=1
 do iinv=1,ninv
  do iop=1,nop  
   if (ltg(iinv,iop)==1) then  
    if (iinv==1) opltg(:,:,ind)= nint(op(:,:,iop)) 
    if (iinv==2) opltg(:,:,ind)=-nint(op(:,:,iop))
    ind=ind+1
   end if
  end do 
 end do

 ! Check the closure of the little group 
 ! (using dummy symafm but should become an input variable)
 allocate(symafm(nsymltg))
 symafm=0
 call chkgrp(nsymltg,symafm,opltg)
 deallocate(symafm)

 ! Find the irreducible Brillouin zone defined by the input point xpt
 wtk=one
 ! Do not use time reversal since it has been introduced previously 
 call symkpt(gmet,indkpt1,kbz,nkbz,nkibzq,nsymltg,0,opltg,0,wtk,wtk_folded)
 write(message,'(a,i5,a,i5,a)')&
& ' Number of points in the IBZ defined by the little group :',nkibzq,' / ',nkbz,ch10
 call wrtout(6,message,'COLL')

 ! Set up a table: 0 if the point does not belong to the IBZ, 1 otherwise
 ibzq=0 ; ibzq_ind=0 ; ind=1
 enough=0
 do ik=1,nkbz
  if (wtk_folded(ik)>tol8 ) then
   ibzq(ik)=1
   ibzq_ind(ik)=ind
   if (enough<=50 .or. prtvol/=0) then 
    write (message,'(1x,i4,a,4f14.6)')ind,') ',kbz(:,ik),wtk_folded(ik)
    call wrtout(6,message,'COLL')
    enough=enough+1
   else if (enough==51) then 
    write(message,'(a)')' prtvol=0, stop printing more k-points information'
    call wrtout(6,message,'COLL')
    enough=enough+1
   end if
   ind=ind+1
  end if
 end do

 ! Reconstruct full BZ starting from the IBZ of the little group  
 ! Calculate the appropriate weight for each point and symmetry operation.
 ! taking into account time reversal 

 lgtab=0 ; lgtabo=0 ; lgtabi=0
 wtksym(:,:,:)=0

 ! Zero no. of k-points found
 ntest=0  ; ktest=zero
 do ik=1,nkbz
  if (ibzq(ik)/=1) cycle
  ! Loop over symmetry operations S and inversion I
  do iop=1,nop
   do iinv=1,ninv
    ! Form IS k only for (IS) pairs in the little group
    if (ltg(iinv,iop)==0) cycle
    call dosym(op(1,1,iop),iinv,kbz(1,ik),knew(1))
    ! Check whether it has already been found (to within a RL vector)
    iold=0
    do itest=1,ntest
     iold=iold+icmpk2(knew,ktest(1,itest))
    end do
    if(iold==0) then !found new point!
     ! For this point the symmetry operation (iop,iinv) must be 
     ! considered to reconstruct the full BZ
     wtksym(iinv,iop,ik)=1
     ntest=ntest+1
     ktest(:,ntest)=knew(:)
     do idx=1,nkbz
      if (icmpk2(knew,kbz(1,idx))==1) then 
       lgtab(idx)=ik
       lgtabo(idx)=iop
       lgtabi(idx)=3-2*iinv
       exit
      end if 
     end do
    end if
   end do
  end do
 end do

 ! For each umkplapp gvectors find the index of G-Go in the gvec array
 ! do this only if G0=/0 
 ! (there is no need to do this if xpt=zero but oh well)
 !  moreover should invert the loops but Im still developping
 iumklp=0 
 do iop=1,nop 
  do iinv=1,ninv
   if (ltg(iinv,iop)==0.or.fumklp(iinv,iop)==0) cycle
   do ig=1,npwvec
    gmg01=modx(gvec(1,ig)-g0(1,iinv,iop),ngfft1)
    gmg02=modx(gvec(2,ig)-g0(2,iinv,iop),ngfft2)
    gmg03=modx(gvec(3,ig)-g0(2,iinv,iop),ngfft3)
    iumklp(ig,iinv,iop)=1+gmg01+gmg02*ngfft1a+gmg03*ngfft1a*ngfft2
    !should save npwvec in little_group
   end do 
  end do !iinv
 end do !iop

 deallocate (opltg)

!DEBUG
!if (DEBUG) then
! if (sum(wtksym)-nkbz/=0) then 
!  write(*,*)' sum(wtksym)-nkbz= ',sum(wtksym)-nkbz
!  call leave_new('COLL')
! end if 
! if(ntest/=nkbz) then 
!  write(*,*)' ntest-nkbz= ',ntest-nkbz
!  call leave_new('COLL')
! end if 
! call findk(ntest,nkbz,ktest,kbz,dummy,1,1)
! do ik=1,ntest-1 
!  do ikp=ik+1,ntest
!  if (dummy(ikp)==dummy(ik))stop
!  end do 
! end do
! do ik=1,nkbz
! if (sum(wtksym(1,:,ik)+wtksym(2,:,ik))/=wtk_folded(ik)) then 
!  write(*,*)'sum(wtksym,ik)-wtk_folded(ik) = ',sum(wtksym(1,:,ik)+wtksym(2,:,ik))-wtk_folded(ik)
!  write(*,*)wtksym(1,:,ik),wtksym(2,:,ik),wtk_folded(ik)
!  write(*,*)ik,kbz(:,ik)
! call leave_new('COLL')
! end if 
! end do
! do ik=1,nkbz 
!  call dosym(op(1,1,lgtabo(ik)),(3-lgtabi(ik))/2,kbz(1,lgtab(ik)),knew(1))
!  if (icmpk2(knew,kbz(1,ik))==0) then 
!   write(*,*)knew,kbz(:,ik) 
!   write(*,*)lgtabo(ik),lgtabi(ik),lgtab(ik)
!   stop 'bug in setup_little_group'
!  end if 
! end do
!end if 
!ENDDEBUG

end subroutine setup_little_group
!!***
