!{\src2tex{textfont=tt}}
!!****f* ABINIT/spectral
!! NAME
!! This file contains functions and subroutines needed to implement the calculation 
!! of the polarizability using the spectral representation as proposed in :
!! PRB 74, 035101 (2006) and PRB 61, 7172 (1999)
!! 
!! 
!! Contains: 
!!  approxdelta 
!!  calc_kkweight
!!

!!****m* ABINIT/approxdelta
!! NAME
!!  approxdelta
!!
!! FUNCTION
!!  Approximate the Dirac function using :
!!  method 1) a triangular funtion centered at the value egwdiff_re (Eq 17 of PRB 74, 035101 (2006) 
!!  method 2) a gaussian of witdth ep%spsmear expandended in Taylor serie 
!!  (at the moment only the 0-th moments) 
!!
!! COPYRIGHT
!!  Copyright (C) 2005-2007 ABINIT group (MG)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!  nomegasfp1=number of frequencies in the grid for Im \chi_0
!!  omegasfp1(nomega)= frequencies (real)
!!  egwdiff_re = transition energy where the delta function is centered 

!!  method= 1 : a triangular shaped function used to approximated the delta
!!         2 : gaussian approximation with standard deviation (smear)
!! smear= used only in case of method==2, defines the width of the gaussian
!!
!! OUTPUT
!!  wl = weight associated to omegal (last omega wich is smaller than egwdiff_re
!!  wr = weight associate to omegar  (first omega larger than egwdff_re
!!  iomegal= index in the array omegasfp1 of the last frequency < egwdiff
!!  iomegar= index in the array omegasfp1 of the first frequency > egwdiff
!!
!! NOTES 
!!  Other approximations could be tried
!!
!! PARENTS
!!     
!!
!! CHILDREN
!!    
!!
!! SOURCE
!!

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine approxdelta(nomegasfp1,omegasfp1,egwdiff_re,smear,iomegal,iomegar,wl,wr,spmeth)
    
 use defs_basis

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01manage_mpi
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: nomegasfp1,spmeth
 integer,intent(out) :: iomegal,iomegar
 real(dp),intent(in) :: egwdiff_re,smear  
 real(dp),intent(out) :: wl,wr
!arrays
 real(dp),intent(in) :: omegasfp1(nomegasfp1)

!Local variables-------------------------------
 integer :: io,iomega
 real(dp) :: omegal,omegar,deltal,deltar
 logical :: DEBUG=.true.
! character(len=500) :: message                 
 
! *************************************************************************
  
 iomega=0
 do io=nomegasfp1,1,-1
  if (omegasfp1(io)<=egwdiff_re) then
   iomega=io  
   exit
  end if 
 end do

 if (iomega==nomegasfp1) then 
  write(*,*)' approxdelta BUG1' 
  call leave_new('COLL')
 end if 
 if (iomega==0) then 
  write(*,*) 'approxdelta BUG2'
  write(*,*)real(omegasfp1(:)) 
  write(*,*)'delta=',egwdiff_re
  call leave_new('COLL')
 end if 
  
 iomegal=iomega
 omegal=omegasfp1(iomegal)
 iomegar=iomegal+1
 omegar=omegasfp1(iomegar)

 select case (spmeth) 
  case (1) 
   !
   ! weights for triangular shaped function
   !
   wr=  (egwdiff_re-omegal)/(omegar-omegal)
   wl= -(egwdiff_re-omegar)/(omegar-omegal)
  case (2) 
   !
   ! weights for gaussian (0-th moment)
   !
   deltal=(egwdiff_re-omegal)/smear
   deltar=(omegar-egwdiff_re)/smear
   if (deltar>=deltal) then
    wl=exp(-deltal*deltal)
    ! this value is used to avoid double counting and speed-up
    wr=huge(0.0_dp)*1.d-10 
   else 
    wl=huge(0.0_dp)*1.d-10
    wr=exp(-deltal*deltal) 
   end if 
 end select

end subroutine approxdelta
!!***

!!****m* ABINIT/calc_kkweight
!! NAME
!!  calc_kkweight
!!
!! FUNCTION
!!  Calculate frequency dependent weights needed to perform the Hilbert transform 
!!  
!!
!! COPYRIGHT
!!  Copyright (C) 2005-2007 ABINIT group (MG)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!! nsp=number of frequencies where the imaginary part of the polarizability is evaluated
!! ne=number of frequencies for the polarizability (same as in epsilon^-1)
!! omegasp(nsp)=real frequencies for the imaginary part of the polarizability 
!! omegae(ne)= imaginary frequencies for the polarizability
!! delta=small imaginary part used to avoid poles, input variables
!!
!! OUTPUT
!! kkweight(ne,nsp)=frequency dependent weights (Eq A1 PRB 74, 035101 (2006)
!!
!! NOTES 
!!
!! PARENTS
!!
!! CHILDREN
!!
!! SOURCE
!!

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine calc_kkweight(ne,omegae,nsp,omegasp,delta,omegamax,kkw)
    
 use defs_basis

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_15gw, except_this_one => calc_kkweight
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: ne,nsp
 real(dp),intent(in) :: delta,omegamax
!arrays
 real(dp),intent(in) :: omegasp(nsp)
 complex,intent(in) :: omegae(ne)
 complex,intent(out) :: kkw(ne,nsp)

!Local variables-------------------------------
!scalars
 integer :: isp,je,method
 real(dp) :: alpha,beta,eta,xx1,xx2,wl,wr
 real(dp) :: den1,den2
 complex :: c1,c2,wt
 complex :: n1,n2,d1,d2
 logical :: DEBUG
!character(len=500) :: message                 
 
! *************************************************************************
!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifndef HAVE_FORTRAN_INTERFACES
 complex :: int_abeta
#endif
!End of the abilint section

 kkw=(0.,0.)
 method=1

 if (method==1) then 
  write(*,*)'using method 1 '
  do je=1,ne
   eta=delta
   wt=omegae(je)
   ! Not include shift at omega==0, what about metallic systems?
   if (abs(real(omegae(je)))<tol6 .and. abs(aimag(wt))<tol6) eta=tol12
   ! Not include shift along the imaginary axis
   if (abs(aimag(wt))>tol6) eta=0.0_dp
   do isp=1,nsp
    !n1= -omegae(je)+omegasp(isp+1)+(0.,1.)*eta
    !d1= -omegae(je)+omegasp(isp  )+(0.,1.)*eta
    !n2=  omegae(je)+omegasp(isp+1)-(0.,1.)*eta
    !d2=  omegae(je)+omegasp(isp  )-(0.,1.)*eta
    !kkw(je,isp)=  - log( (n1*n2)/ (d1*d2) )
    if (isp==1) then  
     ! skip negative point, should check that this would not lead to spurious effects
     c1=(0.,0.)
     den1=1.
    else 
     xx1=omegasp(isp-1)
     xx2=omegasp(isp)
     den1= xx2-xx1 
     c1= -(wt-xx1+(0.,1.)*eta)*log( (wt-xx2+(0.,1.)*eta)/(wt-xx1+(0.,1.)*eta) )&
&        +(wt+xx1-(0.,1.)*eta)*log( (wt+xx2-(0.,1.)*eta)/(wt+xx1-(0.,1.)*eta) )
     c1= c1/den1
    end if 
    xx1=omegasp(isp)
    if (isp==nsp) then 
     ! skip last point should check that this would not lead to spurious effects 
     xx2=omegamax
    else
     xx2=omegasp(isp+1)
    end if
    den2=xx2-xx1
    c2=  (wt-xx2+(0.,1.)*eta)*log( (wt-xx2+(0.,1.)*eta)/(wt-xx1+(0.,1.)*eta) )&
&       -(wt+xx2-(0.,1.)*eta)*log( (wt+xx2-(0.,1.)*eta)/(wt+xx1-(0.,1.)*eta) )
    c2= c2/den2
    !
    kkw(je,isp)=  c1/den1 + c2/den2
    !
    end do
   end do 
 else if (method==2) then 
  do je=1,ne
   beta=real(omegae(je))
   eta=delta
   !do not include shift along the imaginary axis
   if (abs(aimag(omegae(je)))>tol6) eta=0.0_dp
   !dont include shift at omega==0, what about metallic systems?
   if (abs(real(omegae(je)))<tol6) eta=0.0_dp
   do isp=1,nsp 
    if (isp==1) then  
     ! skip negative point, should check that this would not lead to spurious effects
     c1=(0.,0.)
     den1=1.
    else 
     den1= omegasp(isp)-omegasp(isp-1)  
     xx1=omegasp(isp-1)
     xx2=omegasp(isp)
     alpha=omegasp(isp-1)
     ! this is required in case we want to calculate \chi on the im axis
     c1 = -(  int_abeta(xx1,xx2,-alpha,-beta,-eta-aimag(omegae(je)))&
&            +int_abeta(xx1,xx2,-alpha,beta,-eta+aimag(omegae(je))) )
     c1 = c1/den1
   end if 
   if (isp==nsp) then 
    ! skip last point should check that this would not lead to spurious effects 
    den2=omegamax-omegasp(isp)
    xx2=omegamax
    alpha=omegamax
   else
    den2= omegasp(isp+1)-omegasp(isp)
    xx2=omegasp(isp+1)
    alpha=omegasp(isp+1)
   end if
   xx1=omegasp(isp)
   c2=  int_abeta(xx1,xx2,-alpha,-beta,-eta-aimag(omegae(je)))&
&      +int_abeta(xx1,xx2,-alpha,beta,-eta+aimag(omegae(je)))
   c2= c2/den2
   kkw(je,isp)=  c1/den1 + c2/den2
   !kkw(je,isp)=  kkw(je,isp)-(0.0,1.0)*aimag(kkw(je,isp))
   !if ( abs(real(omegae(je))-omegasp(isp))<0.2/Ha_eV) then 
   ! write(*,*)'hello ' 
   !  kkw(je,isp)=(0.0,0.0)
   !end if 
   write(100,'(2i3,1x,2(f8.4,1x),2x,2e18.6)')&
&   je,isp,Ha_eV*real(omegae(je)),Ha_eV*omegasp(isp),real(kkw(je,isp)),aimag(kkw(je,isp))
   end do 
  end do
 end if 

 if (DEBUG) then 
  write(77,*)'kweights method==',method
  do je=1,ne 
  do isp=1,nsp
   write(77,*)je,isp,kkw(je,isp)
  end do
   write(77,*)
  end do 
 end if 

end subroutine calc_kkweight
!!***

!!****m* ABINIT/indef_abeta
!! NAME
!! indef_abeta
!!
!! FUNCTION
!! Calculate analytically the indefinite integral: 
!! 
!! $ \int^{xx} \frac{t+\alpha}{++\beta + i\eta} \,dt $
!!
!! where \alpha, \beta and \eta are real numbers 
!!   
!!
!! COPYRIGHT
!! Copyright (C) 1999-2007 ABINIT group (MG)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! PARENTS
!!
!! INPUTS
!! alpha,beta,eta= coefficients defining the integrand 
!! xx= point where the indefinitive integral must be evaluated 
!!
!! OUTPUT
!!  
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

complex function indef_abeta(alpha,beta,eta,xx)

 use defs_basis

 implicit none

!Arguments ------------------------------------
!arrays
 real(dp),intent(in) :: alpha,beta,eta,xx

!Local variables-------------------------------
!scalars
 real(dp) :: re,im

! *************************************************************************


!FIXME first coding
  if (abs(eta)<tol6) then 
   re= xx + (alpha-beta)/2 *log((xx+beta)**2+tol12)
   im = 0.0
  else 

 re =  xx - eta*atan((xx+beta)/eta) & 
&         + (alpha-beta)/2 * log( (xx+beta)**2+eta**2 ) 
 
 im = -eta*( ((alpha-beta)/eta)*atan((xx+beta)/eta) + 0.5 * log((xx+beta)**2+eta**2) )


 end if 
 !write(100,*)re,im

 indef_abeta = cmplx(re,im)

 end function indef_abeta
!!***

 complex function int_abeta(xx1,xx2,alpha,beta,eta)

 use defs_basis

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_15gw, except_this_one => int_abeta
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!arrays
 real(dp),intent(in) :: alpha,beta,eta,xx1,xx2
!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifndef HAVE_FORTRAN_INTERFACES
 complex :: indef_abeta
#endif
!End of the abilint section

 int_abeta= indef_abeta(alpha,beta,eta,xx2) - indef_abeta(alpha,beta,eta,xx1)

 end function int_abeta
