!{\src2tex{textfont=tt}}
!!****f* ABINIT/symf12
!! NAME
!! symf12
!!
!! FUNCTION 
!!  This subroutine can be used in two different modes:
!!
!!  Mode 1) 
!!   Reconstruct all the reciprocal space matrix elements of a two point function F_{G1,G2}(q,omega) 
!!   for a fixed q-point, using only the set of independent (G1,G2) pairs.
!!   The independent pairs reconstruct the full G \times G space when all the operations of the little group of q are applied. 
!!  Note that F(r1,r2) is supposed to be invariant with respect to all the point group operations when 
!!  applied both to r1 and r2, i.e 
!!   F(R^{-1}(r1-t),R^{-1}(r2-t)) = f(r1,r2)  where t is a fractional translation
!!
!!  Mode 2) 
!!   Check if a full matrix satisfies the above mentioned symmetry properties 
!!   (mainly used to debug the code)
!!
!!  FIXME: the explanation is a little bit messy
!!
!! COPYRIGHT
!!  Copyright (C) 2007 ABINIT group (MG)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!  nop= number of symmetry operations                                                                             
!!  npwc=number of G vectors in the matrix F_{G1,G2}(q)
!!  op(3,3,nop)= symmetry operations in reciprocal space
!!  ngpi=nuber of independent (G1,G1) pairs   
!!  gptab(npweps,npweps)= index of the indendent pair (G1,G2) such as (T1,T2)= S (G1,G2) 
!!  gptabo(npweps,npweps)= index of the symmetry operation S in the array op such as (T1,T2)= S (G1,G2)
!!  ip2fp(2,ngpi)= index of G1 and G2 in the array gvec for each ngpi independent pair 
!!  phgt(npweps,nop)=phase factors associated to non simmorphic operations (e^{-iG \cdot t})
!!
!! OUTPUT
!!  eps(npweps,npweps,nomega), see side effects
!!
!! SIDE EFFECTS
!!  Mode 1) 
!!   in input: eps contains only the matrix elements corresponding to the independent pair 
!!   in output: full symmetrized eps 
!!  Mode 2) 
!!   Only checking
!!   eps matrix is supposed to contain all the elements, 
!!
!! NOTES
!!  This subroutine requires tables indicating the set of independent G1 G1 pairs for 
!!  a given q-point (calculated in findggp) as well as phase factors ....
!! 
!! TODO 
!!  Remember that we can take advantage of hermitianess in case of imaginary frequencies
!!  What about reversal symmetry?
!!
!! PARENTS
!!
!! CHILDREN
!!      leave_new,wrtout
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine symf12(npweps,nomega,eps,omega,nop,op,ngpi,gptab,gptabo,ip2fp,phgt,mode)
    
 use defs_basis

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01manage_mpi
#endif
!End of the abilint section

 implicit none

!Arguments ------------------------------------
!scalars
 integer,intent(in) :: nop,npweps,nomega,ngpi,mode
!arrays
 integer,intent(in) :: gptab(npweps,npweps),gptabo(npweps,npweps)
 integer,intent(in) :: ip2fp(2,ngpi)
 real(dp),intent(in) :: op(3,3,nop)
 complex,intent(in) :: omega(nomega)
 complex,intent(in) :: phgt(npweps,nop)     !phase factors associated to non simmorphic operations (e^{-iG \cdot t})
 complex,intent(inout) :: eps(npweps,npweps,nomega)

!Local variables-------------------------------
!scalars
 integer :: ig1,ig2,iop,io,ip,ind1,ind2,iid,iormax,ioimax
 real(dp),parameter :: delta=10d-1
 complex :: diff 
 complex :: hsym,asym
 character(len=500) :: message                   
 logical :: no_identity
!arrays
 integer :: identity(3,3),op_int(3,3,nop),irmax(2),iimax(2)
 real(dp) :: err(2),den(2),maxerr(2)
 complex,allocatable:: herm(:,:,:),aherm(:,:,:)
 
! *************************************************************************

 !write (std_out,*) ' symf12 : enter'

 if (mode/=1 .and. mode/=2 )then
  write(message,'(a,a,a,a,a,a,i6)') ch10,&
&  ' symf12: BUG -',ch10,&
&  '  The mode option should be 1 or 2,',ch10,&
&  '  however, mode=',mode
  call wrtout(std_out,message,'COLL')
  call leave_new('COLL')
 end if
 
 identity(:,:)=reshape((/1,0,0,0,1,0,0,0,1/),(/3,3/))
 
 op_int=nint(op)

! symq3 assumes that the identity operation is present
! but in the ABINIT < 5.3  we were removing symmetries related by the inversion 
! and sometimes it happened that only the inversion was reported in the KSS file (see outkss.F90) 
! in this case the code will stop
 no_identity=.true. 
 do iop=1,nop
  if (all(op_int(:,:,iop)==identity)) then 
   no_identity=.false.
   iid=iop
   exit
  end if
 end do 
 
 if (no_identity) then 
  write(message,'(4a)')&
&  ' findggp : COMMENT -',&
&  ' Only the inversion was found in the set of symmetries read from the KSS file ',ch10,&
&  ' Maybe old version of KSS file? '
  call wrtout(6,message,'COLL')
  call leave_new('COLL')
  !op_int=-op_int
  else 
  write(message,'(a,i4)')' found identity with index : ',iid
  call wrtout(6,message,'COLL')
 end if 


! The basic property used is: 
! 
!  if q=Sq then F_{SG1, SG2} (q) = e^{+iS(G2-G1)\cdot t} F_{G1,G2)} (q)

 if (mode==1) then  !symmetrize eps

  do ig1=1,npweps
   do ig2=1,npweps 

    iop=gptabo(ig1,ig2)
    if (iop==iid) cycle     !Check that the pair is not independent, to speed up
    ip=gptab(ig1,ig2)       !sequential index of the independent pair
    ind1=ip2fp(1,ip)        !index in the full array gvec, see findggp
    ind2=ip2fp(2,ip)             

    do io=1,nomega
!TODO can speed up by calculating the phase before entering the loop, but require more memory
!     should also take into account hermiticity in case of imaginary frequencies
     eps(ig1,ig2,io)=eps(ind1,ind2,io)*phgt(ig1,iop)*conjg(phgt(ig2,iop))
    end do 
 
   end do 
  end do 

 else 
  
  write(message,'(2a,f5.2,a)')ch10,' symf12 : checking symmetry properties using delta : ',delta,ch10
  call wrtout(std_out,message,'COLL')

  if (all(gptabo==iid)) then 
! A non symmorphyc translation associated to the identity operator means that the 
! unit cell is not primitive (I think the code should realize and stop)
   write(message,'(2a)')' symff12 : all the pairs are independent, return',ch10
   call wrtout(std_out,message,'COLL')
   return
  end if 

  !allocate (herm(npweps,npweps,nomega),aherm(npweps,npweps,nomega))
  !herm=czero ; aherm=czero

  !do io=1,nomega
  ! herm(:,:,io) = half*(eps(:,:,io)+transpose(conjg(eps(:,:,io))))
  ! aherm(:,:,io) = -(0.,1)*half*(eps(:,:,io)-transpose(conjg(eps(:,:,io))))
  !end do

  err=zero
  maxerr(:)=zero

  irmax=0
  iimax=0
  iormax=0
  ioimax=0

  write(message,'(2a,f5.2)')' val1 ... val2  percent err'
  call wrtout(std_out,message,'COLL')

  do ig1=1,npweps
   do ig2=1,npweps 

    iop=gptabo(ig1,ig2)
    if (iop==iid) cycle     !Check that the pair is not independent, to speed up
    ip=gptab(ig1,ig2)       !sequential index of the independent pair
    ind1=ip2fp(1,ip)        !index in the full array gvec, see findggp
    ind2=ip2fp(2,ip)             

    !if ( (ig2>ig1 .and. ind2<ind1) .or. (ig2<ig1 .and. ind2>ind1)) cycle
    

    do io=1,nomega
! TODO can speed up by calculating the phase before entering the loop
     diff = eps(ig1,ig2,io) - eps(ind1,ind2,io)*phgt(ig1,iop)*conjg(phgt(ig2,iop))
     !hsym = herm(ind1,ind2,io)*phgt(ig1,iop)*conjg(phgt(ig2,iop))
     !asym = aherm(ind1,ind2,io)*phgt(ig1,iop)*conjg(phgt(ig2,iop))
     !diff = eps(ig1,ig2,io) - (hsym +(0.,1.)*asym)
     !write(1,'(2es18.6)')diff
     !write(2,'(6es18.6)')eps(ig1,ig2,io),hsym,asym

     err(1)=abs(real(diff))
     den(1)=max(abs(real(eps(ig1,ig2,io),dp)),tol12)

     err(2)=abs(aimag(diff))
     den(2)=max(abs(real(aimag(eps(ig1,ig2,io)),dp)),tol12)

     if (den(1)*delta<err(1)) then 
      write(*,'(3(es18.6,4x),a,es18.6,a,6i4)')&
&   real(eps(ig1,ig2,io)),0.0_dp,real(eps(ind1,ind2,io)*phgt(ig1,iop)*conjg(phgt(ig2,iop))),&
&   '> ',100*err(1)/den(1),' %',ig1,ig2,ind1,ind2,iop,io
      if (err(1)>maxerr(1)) then 
       maxerr(1)=err(1)
       irmax(1)=ig1
       irmax(2)=ig2
       iormax=io
      end if
     end if 
     
     if (den(2)*delta<err(2)) then 
      write(*,'(3(es18.6,4x),a,es18.6,a,6i4)')&
&   0.0_dp,aimag(eps(ig1,ig2,io)),aimag(eps(ind1,ind2,io)*phgt(ig1,iop)*conjg(phgt(ig2,iop))),&
&   '> ',100*err(2)/den(2),' %',ig1,ig2,ind1,ind2,iop,io
      if (err(2)>maxerr(2)) then 
       maxerr(2)=err(2)
       iimax(1)=ig1
       iimax(2)=ig2
       ioimax=io
      end if
     end if 

    end do !iomega
   end do !ig2 
  end do !ig1

  if (iormax/=0) then 
   write(*,'(a,2(2x,f24.6))')' symf12 : max abs error for real part = ',maxerr(1)
   ig1=irmax(1)
   ig2=irmax(2)
   iop=gptabo(ig1,ig2)
   ip=gptab(ig1,ig2)       
   ind1=ip2fp(1,ip)      
   ind2=ip2fp(2,ip)
   write(*,*)'ig1,ig2,ind1,ind2,iormax=',ig1,ig2,ind1,ind2,iormax
   write(*,'(2(2x,es18.6))')&
&   real(eps(ig1,ig2,iormax)),real(eps(ind1,ind2,iormax)*phgt(ig1,iop)*conjg(phgt(ig2,iop)))
  end if 
  
  if (ioimax/=0) then
   write(*,'(a,2(2x,f24.6))')' symf12 : max abs error for imag part = ',maxerr(2)
   ig1=iimax(1)
   ig2=iimax(2)
   iop=gptabo(ig1,ig2)
   ip=gptab(ig1,ig2)       
   ind1=ip2fp(1,ip)      
   ind2=ip2fp(2,ip)
   write(*,*)'ig1,ig2,ind1,ind2,iormax=',ig1,ig2,ind1,ind2,ioimax
   write(*,'(2(2x,es18.6))')&
&   aimag(eps(ig1,ig2,ioimax)),aimag(eps(ind1,ind2,ioimax)*phgt(ig1,iop)*conjg(phgt(ig2,iop)))
  end if 

 end if 

end subroutine symf12
!!***
