!{\src2tex{textfont=tt}}
!!****f* ABINIT/ftgkk
!!
!! NAME
!! ftgkk
!!
!! FUNCTION
!! If qtor=1 (q->r):
!! Generates the Fourier transform of the recip space gkk matrices
!! to obtain the real space ones.
!! If qtor=0 (r->q):
!! Generates the Fourier transform of the real space gkk matrices
!! to obtain the reciprocal space ones.
!!
!! Only carried out for 1 kpoint.
!! Loops over FSkpt and file i/o is done outside ftgkk
!! * qtor FT is normal
!! * rtoq contains additional weights for the rspace points
!!   at the edge of a Wigner Seitz cell.
!!
!! COPYRIGHT
!! Copyright (C) 2004-2007 ABINIT group (MVer)
!! This file is distributed under the terms of the
!! GNU General Public Licence, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!! gprim(3,3)= Normalized coordinates in reciprocal space
!! natom= Number of atoms in the unit cell
!! ngkkband = number of bands kept in gkq and gkr matrix elements (=1 or nband)
!! nqpt= Number of q points in the Brillouin zone
!!           if qtor=0 this number is read in the input file
!! nrpt= Number of R points in the Big Box
!! qtor= ( q to r : see above )
!! rpt(3,nprt)= Canonical coordinates of the R points in the unit cell
!!           These coordinates are normalized (=> * acell(3)!!)
!! spqpt(3,nqpt)= Reduced coordinates of the q vectors in reciprocal space
!!           if qtor=0 these vectors are read in the input file
!! wghatm(natom,natom,nrpt)
!!         = Weights associated to a pair of atoms and to a R vector
!!
!! OUTPUT
!!  (see side effects)
!!
!! SIDE EFFECTS
!! Input/output
!! gkk_qpt(2,3*natom,nFSband,nFSband,nqpt)
!!  = gkk matrices in recip space
!! gkk_rpt(2,3*natom,nFSband,nFSband,nqpt)
!!  = gkk matrices in real space
!!
!! PARENTS
!!      get_all_gkr,interpolate_gkk,test_ftgkk
!!
!! CHILDREN
!!      leave_new,wrtout
!!
!! NOTES
!!   copied from ftiaf9.f
!!
!!    real space elements are complex, but could be reduced, as (-r) = (+r)*
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine ftgkk (wghatm,gkk_qpt,gkk_rpt,gprim,&
                & natom,ngkkband,nqpt,nrpt,qtor,rpt,spqpt)

 use defs_basis
 use defs_datatypes

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01manage_mpi
#endif
!End of the abilint section

 implicit none

!Arguments -------------------------------
!scalars
 integer,intent(in) :: natom,ngkkband
 integer,intent(in) :: nqpt,nrpt,qtor
!arrays
 real(dp),intent(in) :: gprim(3,3),rpt(3,nrpt),spqpt(3,nqpt)
 real(dp),intent(in) :: wghatm(natom,natom,nrpt)

 real(dp),intent(inout) :: gkk_qpt(2,ngkkband,ngkkband,3*natom,3*natom,nqpt)
 real(dp),intent(inout) :: gkk_rpt(2,ngkkband,ngkkband,3*natom,3*natom,nrpt)

!Local variables -------------------------
!scalars
 integer :: ia,iatom,ib,ib1,ib2,idir,ii,ip,iqpt,irpt,jatom
 integer :: jdir,jp,mu,nu
 real(dp) :: asri,asrr,cpu,facti,factr,im,kr,re,wall
 character(len=500) :: message
!arrays
 real(dp) :: coskr(nqpt,nrpt),ftwght(2,natom,natom)
 real(dp) :: gkk_qpt_tmp(2,ngkkband,ngkkband,3*natom,3*natom)
 real(dp) :: gkk_rpt_tmp(2,ngkkband,ngkkband,3*natom,3*natom),kk(3)
 real(dp) :: sinkr(nqpt,nrpt),tmpgkkrim(nrpt),tmpgkkrre(nrpt)

! *********************************************************************

  !  prepare the phase factors
  do iqpt=1,nqpt
     !  Calculation of the k coordinates in Normalized Reciprocal
     !  coordinates
     kk(1)=   spqpt(1,iqpt)*gprim(1,1)+&
          &   spqpt(2,iqpt)*gprim(1,2)+&
          &   spqpt(3,iqpt)*gprim(1,3)
     kk(2)=   spqpt(1,iqpt)*gprim(2,1)+&
          &   spqpt(2,iqpt)*gprim(2,2)+&
          &   spqpt(3,iqpt)*gprim(2,3)
     kk(3)=   spqpt(1,iqpt)*gprim(3,1)+&
          &   spqpt(2,iqpt)*gprim(3,2)+&
          &   spqpt(3,iqpt)*gprim(3,3)
     do irpt=1,nrpt
        !  Product of k and r
        kr =        kk(1)*rpt(1,irpt)+&
             &      kk(2)*rpt(2,irpt)+&
             &      kk(3)*rpt(3,irpt)
        coskr(iqpt,irpt)=cos(two_pi*kr)
        sinkr(iqpt,irpt)=sin(two_pi*kr)
!DEBUG
!if (iqpt < 1000 .and. (irpt == 101 .or. irpt == 901)) then
!   write (*,*) iqpt,irpt,kk,rpt(:,irpt),coskr(iqpt,irpt), sinkr(iqpt,irpt)
!end if
!ENDDEBUG
     end do
  end do



  !Recip to real space
  if (qtor==1) then
     !
     ! initialize
     gkk_rpt(:,:,:,:,:,:) = zero

     do irpt=1,nrpt
!DEBUG
!write (*,*) ' ftgkk : G->R irpt = ',irpt,' / ',nrpt
!ENDDEBUG
        gkk_rpt_tmp(:,:,:,:,:) = zero

        do iqpt=1,nqpt

           !write (*,*) iqpt

           gkk_qpt_tmp(:,:,:,:,:) = gkk_qpt(:,:,:,:,:,iqpt)
           !   Get the phase factor with normalization!
           re=coskr(iqpt,irpt)/nqpt
           im=sinkr(iqpt,irpt)/nqpt


! might use blas routines, or do bunched multiplication by re and im
           do ip=1,3*natom
              do jp=1,3*natom
                 !       Real and imaginary part of the real-space gkk matrices -> exp(-i k.r)
                 do ib1=1,ngkkband
                  do ib2=1,ngkkband
                   gkk_rpt_tmp(1,ib2,ib1,jp,ip) = gkk_rpt_tmp(1,ib2,ib1,jp,ip)&
                      &          +re*gkk_qpt_tmp(1,ib2,ib1,jp,ip) &
                      &          +im*gkk_qpt_tmp(2,ib2,ib1,jp,ip)
                   gkk_rpt_tmp(2,ib2,ib1,jp,ip) = gkk_rpt_tmp(2,ib2,ib1,jp,ip)&
                      &          +re*gkk_qpt_tmp(2,ib2,ib1,jp,ip) &
                      &          -im*gkk_qpt_tmp(1,ib2,ib1,jp,ip)
                  end do
                 end do
              end do
           end do
        end do
        gkk_rpt(:,:,:,:,:,irpt) = gkk_rpt_tmp(:,:,:,:,:)
     end do

  ! Real space to recip space
  else if (qtor==0) then

    ! initialize
    gkk_qpt(:,:,:,:,:,:)=zero

    !write (*,*) 'ftgkk : shape(gkk_qpt) = ', shape(gkk_qpt)

    do irpt=1,nrpt
      !call timein(cpu,wall)
      !write (*,*) 'ft top1',cpu,wall
      gkk_rpt_tmp(:,:,:,:,:) = gkk_rpt(:,:,:,:,:,irpt)

      !call timein(cpu,wall)
      !write (*,*) 'ft top2',cpu,wall


      do iqpt=1,nqpt

        ! Avoid recalculating weights
        do iatom=1,natom
          do jatom=1,natom
            ftwght(1,iatom,jatom)=coskr(iqpt,irpt)*wghatm(iatom,jatom,irpt)
            ftwght(2,iatom,jatom)=sinkr(iqpt,irpt)*wghatm(iatom,jatom,irpt)
          end do
        end do

! might use blas routines, or do bunched multiplication by ftwght

        ip = 0
        do iatom=1,natom
          do idir=1,3
            ip=3*(iatom-1)+idir
            !write (*,*) ' ftgkk : ip, iatom = ', ip, iatom
            do jatom=1,natom
              !   Get phase factor
              re = ftwght(1,iatom,jatom)
              im = ftwght(2,iatom,jatom)

              do jdir=1,3
                jp=3*(jatom-1)+jdir

                do ib1=1,ngkkband
                  do ib2=1,ngkkband
                    !       Real and imaginary part of the gamma matrices
                    gkk_qpt(1,ib2,ib1,jp,ip,iqpt)=&
                       &      gkk_qpt(1,ib2,ib1,jp,ip,iqpt)&
                       &  +re*gkk_rpt_tmp(1,ib2,ib1,jp,ip)&
                       &  -im*gkk_rpt_tmp(2,ib2,ib1,jp,ip)
                    gkk_qpt(2,ib2,ib1,jp,ip,iqpt)=&
                       &      gkk_qpt(2,ib2,ib1,jp,ip,iqpt)&
                       &  +im*gkk_rpt_tmp(1,ib2,ib1,jp,ip)&
                       &  +re*gkk_rpt_tmp(2,ib2,ib1,jp,ip)

                    !if (iqpt < 100 .and. irpt < 100 .and. &
                    !     &   tmpgkkrim(irpt)**2+tmpgkkrre(irpt)**2 > tol6) then
                    !   write (*,'(2I4,2E16.8,x,2E16.8)') &
                    !        &   iqpt,irpt,re,im,tmpgkkrre(irpt),tmpgkkrim(irpt)
                    !end if

                  end do
                end do
              end do! end jdir
            end do  ! end jatom
          end do ! end idir
          !write (*,'(a)') ' ftgkk :gkk_qpt :'
          !write (*,'(4E16.5)') gkk_qpt(:,1,1,1,1,1:nqpt)
        end do ! end iatom

      end do ! end iqpt
    end do ! end irpt

    
  else   ! There is no other space to Fourier transform from ??
     write(message,'(a,a,a,a,a,i4,a)' )&
          &  ' ftiaf9 : BUG - ',ch10,&
          &  '  The only allowed values for qtor are 0 or 1, while',ch10,&
          &  '  qtor=',qtor,' has been required.'
     call wrtout(6,message,'COLL')
     call leave_new('COLL')
  end if

end subroutine ftgkk
!!***
