!{\src2tex{textfont=tt}}
!!****f* ABINIT/wvl_wfsinp_disk
!! NAME
!! wvl_wfsinp_disk
!!
!! FUNCTION
!! This method allocates and initialises wavefunctions with values from disk.
!! See wvl_wfsinp_scratch() or wvl_wfsinp_reformat() from other initialisation
!! routines.
!! 
!! When initialised from scratch or from disk, wvl%wfs%[h]psi comes unallocated
!! and will be allocated inside this routine.
!! When initialised from memory (reformating), wvl%wfs%[h]psi will be reallocated.
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 ABINIT group (DC)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt .
!!
!! INPUTS
!!  dtset <type(dataset_type)>=input variables.
!!  hdr0 <type(hdr_type)>=the header of wf, den and pot files (read from restart)
!!  hdr <type(hdr_type)>=the header of wf, den and pot files
!!  mpi_enreg=informations about MPI parallelization
!!  option=1 for reading a file following ABINIT format, -1 for a BigDFT format.
!!  rprimd(3,3)=dimensional primitive translations in real space (bohr)
!!  wff <type(wffile_type)>= structure with informations on wf file.
!!
!! OUTPUT
!!
!! SIDE EFFECTS
!!  wfs <type(wvl_projector_type)>=wavefunctions informations for wavelets.
!!  xred(3,natom)=reduced dimensionless atomic coordinates (in fact IN but here
!!                because of INOUT xredxcart() behavior).
!!
!! PARENTS
!!
!! CHILDREN
!!
!! SOURCE
#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine wvl_wfsinp_disk(dtset, hdr0, hdr, mpi_enreg, option, &
     & rprimd, wff, wfs, xred)

  use defs_basis
  use defs_datatypes
#if defined HAVE_BIGDFT
  use libbigdft
#endif

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01manage_mpi
 use interfaces_13io_mpi
 use interfaces_lib01hidempi
#endif
!End of the abilint section

  implicit none

!Arguments -------------------------------
  !scalars
  integer, intent(in)                       :: option
  type(dataset_type), intent(in)            :: dtset
  type(hdr_type), intent(in)                :: hdr0
  type(hdr_type), intent(in)                :: hdr
  type(MPI_type), intent(in)                :: mpi_enreg
  type(wffile_type), intent(in)             :: wff
  type(wvl_wf_type), intent(inout)          :: wfs
  !arrays
  real(dp), intent(in)                      :: rprimd(3, 3)
  real(dp), intent(inout)                   :: xred(3, dtset%natom)

!Local variables-------------------------------
  character(len = 500)  :: message
  logical               :: parallel
  integer               :: spaceComm, ierr

  write(message, '(a,a)' ) ch10,&
    &  ' wvl_wfsinp_disk: wavefunction initialisation.'
  call wrtout(6,message,'COLL')

  parallel = (mpi_enreg%paral_compil_fft == 1)

  ! We allocate psi.
  if (parallel) then
     !allocated in the transposed way such as 
     !it can also be used as a work array for transposition
     allocate(wfs%psi(wfs%mvctrp, wfs%mbandp * mpi_enreg%nproc))
  else
     allocate(wfs%psi(wfs%keys%nvctr(0) + 7 * wfs%keys%nvctr(1), wfs%mbandp))
  end if
  write(message, '(a,a,a,a,I0)' ) ch10, &
    &  ' wvl_wfsinp_disk: allocate wavefunctions,', ch10, &
    &  '  size of the compressed array per proc: ', &
    & product(shape(wfs%psi))
  call wrtout(6,message,'COLL')

  call wvl_read(dtset, hdr0, hdr, mpi_enreg, option, rprimd, wff, wfs, xred)

#if defined HAVE_BIGDFT
  ! We orthogonalise.
  if (parallel) then
     !allocate hpsi array (used also as transposed)
     !allocated in the transposed way such as 
     !it can also be used as the transposed hpsi
     allocate(wfs%hpsi(wfs%mvctrp, wfs%mbandp * mpi_enreg%nproc))

     !transpose the psi wavefunction
     !here hpsi is used as a work array
     call switch_waves(mpi_enreg%me, mpi_enreg%nproc, dtset%mband, wfs%mbandp, &
          & wfs%keys%nvctr(0), wfs%keys%nvctr(1), wfs%mvctrp, &
          & wfs%psi, wfs%hpsi)
     !allocate transposed principal wavefunction
     allocate(wfs%psit(wfs%mvctrp, wfs%mbandp * mpi_enreg%nproc))

     call xcomm_world(mpi_enreg, spaceComm)
     call xalltoall_mpi_dp2d(wfs%hpsi, wfs%mvctrp * wfs%mbandp, &
          & wfs%psit, wfs%mvctrp * wfs%mbandp, spaceComm, ierr)
     !end of transposition

     call orthon_p(mpi_enreg%me, mpi_enreg%nproc, dtset%mband, wfs%mbandp, &
          & wfs%mvctrp, wfs%psit)
     !call checkortho_p(iproc,nproc,norb,norbp,nvctrp,psit)

     !retranspose the psit wavefunction into psi
     !here hpsi is used as a work array
     call xalltoall_mpi_dp2d(wfs%psit, wfs%mvctrp * wfs%mbandp, &
          & wfs%hpsi, wfs%mvctrp * wfs%mbandp, spaceComm, ierr)

     call unswitch_waves(mpi_enreg%me, mpi_enreg%nproc, dtset%mband, wfs%mbandp, &
          & wfs%keys%nvctr(0), wfs%keys%nvctr(1), wfs%mvctrp, &
          & wfs%hpsi, wfs%psi)
     !end of retransposition
  else
     call orthon(dtset%mband, wfs%mbandp, wfs%mvctrp, wfs%psi)
     !call checkortho(norb,norbp,nvctrp,psi)
     !allocate hpsi array
     allocate(wfs%hpsi(wfs%keys%nvctr(0) + 7 * wfs%keys%nvctr(1), wfs%mbandp))
  endif
#else
  write(message, '(a,a,a,a)' ) ch10,&
    &  ' wvl_wfs_inp: BigDFT library is not compiled.', ch10, &
    &  '   Action, used the flag --enable-bigdft when configuring.'
  call wrtout(6,message,'COLL')
  call leave_new('COLL')
#endif
end subroutine wvl_wfsinp_disk
!!***

