!{\src2tex{textfont=tt}}
!!****f* ABINIT/wvl_wfsinp_reformat
!! NAME
!! wvl_wfsinp_reformat
!!
!! FUNCTION
!! This method allocates and initialises wavefunctions with values from disk.
!! See wvl_wfsinp_scratch() or wvl_wfsinp_reformat() from other initialisation
!! routines.
!! 
!! When initialised from scratch or from disk, wvl%wfs%[h]psi comes unallocated
!! and will be allocated inside this routine.
!! When initialised from memory (reformating), wvl%wfs%[h]psi will be reallocated.
!! The projectors are also recomputed.
!!
!! The scalar arrays should be reallocated using dtset%nfft after a call to
!! this routine.
!!
!! COPYRIGHT
!! Copyright (C) 2005-2007 ABINIT group (DC)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!
!! OUTPUT
!!
!! SIDE EFFECTS
!!
!! NOTES
!!
!! PARENTS
!!      afterscfloop
!!
!! CHILDREN
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine wvl_wfsinp_reformat(acell, dtset, mpi_enreg, occ, psps, &
     & rprimd, wvl, xred, xred_old)

  use defs_basis
  use defs_datatypes
#if defined HAVE_BIGDFT
  use libbigdft
#endif

!This section has been created automatically by the script Abilint (TD). Do not modify these by hand.
#ifdef HAVE_FORTRAN_INTERFACES
 use interfaces_01manage_mpi
 use interfaces_12geometry
 use interfaces_14wvl_wfs
#endif
!End of the abilint section

  implicit none

  !Arguments ------------------------------------
  type(dataset_type), intent(inout)      :: dtset
  type(MPI_type), intent(inout)          :: mpi_enreg
  type(pseudopotential_type), intent(in) :: psps
  type(wvl_data), intent(inout)          :: wvl
  real(dp), intent(inout)                :: acell(3), rprimd(3,3)
  real(dp), intent(in)                   :: occ(dtset%mband * dtset%nkpt * &
       & dtset%nsppol)
  real(dp), intent(inout)                :: xred_old(3, dtset%natom)
  real(dp), intent(inout)                :: xred(3, dtset%natom)

  !Local variables-------------------------------
  integer                  :: nSize_old(3)
  real(dp), allocatable    :: xcart(:,:), xcart_old(:,:), eigen_old(:)
  real(dp), pointer        :: psi_old(:,:)
  type(wvl_keyArrays_type) :: keys_old
  character(len=500)       :: message
  
  write(message, '(a,a)' ) ch10,&
       & ' wvl_wfsinp_reformat: reformat the wavefunctions.'
  call wrtout(06, message, 'COLL')

  ! Convert input xred_old (reduced coordinates) to xcart_old (cartesian)
  allocate(xcart_old(3, dtset%natom))
  call xredxcart(dtset%natom, 1, rprimd, xcart_old, xred_old)

  ! We store some old values.
  nSize_old = dtset%wvl_internal%nSize

  ! We change the box geometry.
  call wvl_setBoxGeometry(acell, dtset, mpi_enreg, psps, rprimd, xred)
  call wvl_setngfft(dtset, mpi_enreg)

  ! We deallocate the previous projectors.
  call wvl_free_type_proj(wvl%projectors)

  ! We store the previous wavefunctions.
  psi_old =>        wvl%wfs%psi
  allocate(eigen_old(dtset%mband))
  eigen_old = wvl%wfs%eval
  keys_old%nseg  => wvl%wfs%keys%nseg
  keys_old%nvctr => wvl%wfs%keys%nvctr
  keys_old%keyg  => wvl%wfs%keys%keyg
  keys_old%keyv  => wvl%wfs%keys%keyv

  ! We deallocate the previous wavefunctions (except the keys since they
  ! are used by keys_old.
  nullify(wvl%wfs%psi)
  nullify(wvl%wfs%keys%nseg)
  nullify(wvl%wfs%keys%nvctr)
  nullify(wvl%wfs%keys%keyg)
  nullify(wvl%wfs%keys%keyv)
  call wvl_free_type_wfs(wvl%wfs)

  ! We create the new wavefunction descriptor and allocate them.
  call wvl_init_type_wfs(dtset, mpi_enreg, psps, rprimd, wvl%wfs, xred)
  wvl%wfs%eval = eigen_old
  deallocate(eigen_old)

  ! We allocate psi.
  if (mpi_enreg%paral_compil_fft == 1) then
     !allocated in the transposed way such as 
     !it can also be used as a work array for transposition
     allocate(wvl%wfs%psi(wvl%wfs%mvctrp, wvl%wfs%mbandp * mpi_enreg%nproc))
  else
     allocate(wvl%wfs%psi(wvl%wfs%keys%nvctr(0) + 7 * wvl%wfs%keys%nvctr(1), &
          & wvl%wfs%mbandp))
  end if
  write(message, '(a,a,a,a,I0)' ) ch10, &
    &  ' wvl_wfsinp_reformat: allocate wavefunctions,', ch10, &
    &  '  size of the compressed array per proc: ', &
    & product(shape(wvl%wfs%psi))
  call wrtout(6,message,'COLL')

  ! Convert input xred (reduced coordinates) to xcart (cartesian)
  allocate(xcart(3, dtset%natom))
  call xredxcart(dtset%natom, 1, rprimd, xcart, xred)

  ! We transfer the old wavefunctions to the new ones.
#if defined HAVE_BIGDFT
  call reformatmywaves(mpi_enreg%me, dtset%mband, wvl%wfs%mbandp, dtset%natom, &
       & dtset%wvl_hgrid, keys_old%nvctr(0), keys_old%nvctr(1), &
       & nSize_old(1), nSize_old(2), nSize_old(3), xcart_old, &
       & keys_old%nseg(0), keys_old%nseg(1), keys_old%keyg, &
       & keys_old%keyv, psi_old, dtset%wvl_hgrid, wvl%wfs%keys%nvctr(0), &
       & wvl%wfs%keys%nvctr(1), dtset%wvl_internal%nSize(1), &
       & dtset%wvl_internal%nSize(2), dtset%wvl_internal%nSize(3), xcart, &
       & wvl%wfs%keys%nseg(0), wvl%wfs%keys%nseg(1), wvl%wfs%keys%keyg, &
       & wvl%wfs%keys%keyv, wvl%wfs%psi)
#else
  write(message, '(a,a,a,a)' ) ch10,&
       & ' wvl_wfsinp_reformat: BUG -',ch10,&
       & '  BigDFT is not compile. Use --enable-bigdft during configure.'
  call wrtout(06, message, 'COLL')
  call leave_new('COLL')
#endif
  deallocate(xcart, xcart_old)

  ! We free the old descriptors and arrays.
  deallocate(psi_old)
  call wvl_free_type_keyArrays(keys_old)

  ! Reallocate projectors for the new positions.
  call wvl_init_type_proj(dtset, mpi_enreg, wvl%projectors, psps, rprimd, xred)

  ! Allocate the hpsi array.
  !allocated in the transposed way such as 
  !it can also be used as the transposed hpsi
  if (mpi_enreg%paral_compil_fft == 1) then
     allocate(wvl%wfs%hpsi(wvl%wfs%mvctrp, wvl%wfs%mbandp * mpi_enreg%nproc))
  else
     allocate(wvl%wfs%hpsi(wvl%wfs%keys%nvctr(0) + 7 * wvl%wfs%keys%nvctr(1), &
          & wvl%wfs%mbandp))
  end if

end subroutine wvl_wfsinp_reformat
